function y = myFFT(x)

N = length(x);                   % signal length (power of 2 is required)
Nbit = log2(N);                  % number of bits required for sample indexes

%Samples reordering (in bitreverse fashion)
for n=0:N-1
    nc = n;                      % old sample position - copying
    m = 0;                       % new sample position - initialization
    for k=1:Nbit                 % check all bits
        if(rem(n,2)==1)          % check low significant bit
           m = m + 2^(Nbit-k);   % accumulate "m" using bitreversed weight
           n = n - 1;            % decrement by 1
        end
        n=n/2;                   % shift all bits one position right 
    end
    y(m+1) = x(nc+1);            % copy sample into new "bitreversed" position
end
% n=0:N-1; m=dec2bin(n); m=m(:,Nbit:-1:1); m=bin2dec(m); y(m+1) = x(n+1);

% Butterflies
Nlev=Nbit;                       % number of 
for lev=1:Nlev % LEVELS
    bw=2^(lev-1);                % butterflies width
    nbb=2^(lev-1);               % number of butterflies per block
    sbb=2^lev;                   % shift between blocks
    nbl=N/2^lev;                 % number of blocks
    W=exp(-j*2*pi/2^lev);        % correction coefficient
    for bu=1:nbb % BUTTERFLIES
        Wb=W^(bu-1);                             % correction for given butterfly
        for bl=1:nbl % BLOCKS
            up   = 1    + (bu-1) + (bl-1)*sbb;   % up sample index
            down = 1+bw + (bu-1) + (bl-1)*sbb;   % down sample index
            temp = Wb * y(down);                 % temporary value
            y(down) = y(up) - temp;              % new down sample
            y(up)   = y(up) + temp;              % new up sample
        end
    end    
end
