% lab06_ex_fastconv.m - fast convolution and correlation using FFT
clear all; close all;

  sig = 1;    % 1/2, signal: 1=short, 2=long
  if(sig==1)  N=5;    M=3;   x = ones(1,N);  h = ones(1,M);  % signals to be
  else        N=256;  M=32;  x = randn(1,N); h = randn(1,M); % convolved
  end
  n = 1:N+M-1; nn = 1:N; % sample indexes
  figure;
  subplot(211); stem(x); title('x(n)');
  subplot(212); stem(h); title('h(n)'); pause

% Conv by Matlab function
  y1 = conv(x,h);
  figure; stem(y1); title('y1(n)'); pause

% Fast conv - WRONG!
  hz = [ h zeros(1,N-M) ];        % append N-M zeros to the shorter signal only
  y2 = ifft( fft(x) .* fft(hz) ); % fast conv, first M-1 samples are wrong
  figure; plot(nn,y1(nn),'ro',nn,y2(nn),'bx'); title('y1(n) & y2(n)');
  error2 = max(abs(y1(M:N)-y2(M:N))), pause

% Fast conv - GOOD!
  hzz = [ h zeros(1,N-M) zeros(1,M-1) ];  % append zeros to the length N+M-1
  xz = [ x zeros(1,M-1) ];                % append zeros to the length N+M-1
  y3 = ifft( fft(xz) .* fft(hzz) );       % fast conv, all samples are good
  figure; plot(n,y1,'ro',n,y3,'bx'); title('y1(n) & y3(n)');
  error3 = max(abs(y1-y3)), pause

% Fast conv by pieces - OVERLAP-ADD method
  if( sig > 1 )                          % only for long signal
  L = M;                                 % signal fragment length
  K = N/L;                               % number of signal fragments
  hzz = [ h zeros(1,L-M) zeros(1,M-1) ]; % append zeros to the shorter signal
  Hzz = fft(hzz);                        % its FFT
  y4 = zeros(1,M-1);                     % output signal initialization
  for k = 1:K                            % LOOP
      m = 1 + (k-1)*L : L + (k-1)*L;     % select samples indexes
      xz = [ x(m) zeros(1,M-1) ];        % cut signal fragment
      YY = fft(xz) .* Hzz;               % # fast convolution - spectra mult
      yy = ifft( YY );                   % # inverse FFT
      y4(end-(M-2):end) = y4(end-(M-2):end) + yy(1:M-1); % output overlap-add
      y4 = [ y4, yy(M:L+M-1) ];                          % output append
  end
  figure; plot(n,y1,'ro',n,y4,'bx'); title('y1(n) & y4(n)');
  error4 = max(abs(y1-y4)), pause
  end

% Fast cross-correlation by fast convolution
  R1 = xcorr( x, h );                    % cross-correlation via Matlab function
  R2 = conv( x, conj( h(end:-1:1) ) );   % cross-correlation via convolution 
  Kmax=max(M,N); Kmin=min(M,N); R2 = [ zeros(1,Kmax-Kmin) R2 ];
  m = -(Kmax-1) : 1 : (Kmax-1); 
  figure; plot(m,R1,'ro',m,R2,'bx'); title('R1(n) & R2(n)');
  error5 = max( abs( R1-R2 ) ), pause

% Fast computation of the first M coeffs of the autocorrelation function
  R1 = xcorr( x, x );             % Matlab autocorrelation function
  xz = [x zeros(1,M-1)];          % add (append) M-1 zeros on the end of signal
  X = fft(xz);                    % calculate (N+M-1)-point FFT
  X = X.*conj(X);                 % calculate |X(k)|^2
  R2 = ifft(X);                   % calculate inverse (N+M-1)-point FFT
  R2 = real(R2(1:M));             % choose correct values, scale
  m=1:M; figure; plot(m,R1(N:N+M-1),'ro',m,R2(m),'bx'); grid;
  title('Autocorr Rxx(m): Matlab (RED), WE (BLUE)'); xlabel('m'); pause
  error6 = max( abs( R1(N:N+M-1) - R2(m) ) ), pause
