% lab09_ex_fastconv.m - fast signal convolution (FIR fltering) using FFT
  clear all; close all

  N=2048; M=128; x = randn(1,N); h = randn(1,M);
  y1 = conv(x,h);

% continuation of lab06_ex_fastconv.m

% Method #5: Fast convolution by pieces - OVERLAP-SAVE method
  N1 = 2*M;                             % length of one signal fragment
  Lb = floor( (N-(M-1))/(N1-(M-1)) );   % number of signal fragments (blocks)
  hz = [ h zeros(1,N1-M) ];             % appending zeros to h(n)
  Hz = fft(hz);                         % FFT of h(n)
  for k = 0 : Lb-1                      % LOOP: fast conv of signal fragments 
      n1st = 1+k*(N1-(M-2)-1);          % index of next signal fragment beginning
      bx = x( n1st : n1st + N1-1 );     % cutting new samples of x(n) with overlap
      Bx = fft( bx );                   % fft of the x(n) fragment                                        
      by = real( ifft( Bx .* Hz) );     % fast convolution in frequency domain
      y5(n1st+M-1+0 : n1st+M-1+(N1-M)) = by(M:N1); % remove first M-1 samples, store
  end
  Ny5 = length(y5); n5 = M:Ny5;
  figure; plot(n5,y1(n5),'ro',n5,y5(n5),'bx'); title('y1(n) & y5(n)');
  error5 = max(abs( y1(n5)-y5(n5) ))

% Method #6: Fast convolution by pieces - OVERLAP-SAVE method plus h(n) partition
  Nx = N; Nh = M;                       % signal length, filter length
  Nb = 32;                              % partition length of h(n)
  Lb = ceil(Nx/Nb);                     % number of signal fragments
  Lh = ceil(Nh/Nb);                     % number of filter fragments
  for k=1:Lh                            % FFTs of filter weight fragments
      H(k,1:2*Nb) = fft( [h((k-1)*Nb+1:k*Nb) zeros(1,Nb)] );  % with zeros
  end
  bx = zeros(1,2*Nb); X = zeros(Lh,2*Nb); y6 = [];
  for k = 0 : Lb-1                              % FOR ALL SIGNAL FRAGMENTS
      bx = [ bx(Nb+1:2*Nb) x(k*Nb+1:k*Nb+Nb) ]; % take next signal fragment
      X = [ fft(bx); X(1:Lh-1,:) ];             % calculate its FFT and store
      XH = X.*H;                                % # fast
      by = real( ifft(sum(XH)) );               % # convolution
      y6 = [ y6 by(Nb+1:2*Nb) ];                % remove beginning, store result 
  end
  Ny = min( length(y1), length(y6) ); n6 = Nh+1:Ny;
  y1 = y1(n6); y6 = y6(n6);
  figure; plot(n6,y1,'ro',n6,y6,'bx'); title('y1(n) & y6(n)');
  error6 = max(abs(y1-y6)), pause
