% lab10_ex_resampling_demo2 - fast polyphase resampling of real-world signals

clear all; close all;

K = 5;    % UP-sampling K-times
L = 3;    % DOWN-sampling L-times
Np = 50;  % length of one polyphase sub-filter 

R = 600; m = 1:R; mUP=1:K*R; mD=1:R/L;       % for display only

% Signal
%[x,fs,Nb ] = wavread('speech.wav'); x=x';   % old Matlab
 [x,fs ] = audioread('speech.wav'); x=x';     % new Matlab
%fs=8000; Nsec=3; x = sin(2*pi*25/fs*(0:Nsec*fs-1)); % + sin(2*pi*1000/fs*(0:Nsec*fs-1));
Nx=length(x); 
dt=1/fs; t=dt*(0:Nx-1);
figure; plot(t,x); xlabel('t [s]'); title('x(t)'); grid; pause
%soundsc(x,fs);

% New sampling frequencies
fsUP = K*fs;  % new K-times higher sampling frequency
fsD  = fs/L;  % new L-times lower  sampling frequency 

% ####################################
% Polyphase interpolation (K times UP)
% K-times signal convolving with K polyphase components of LowPass interolation filter

% Version #1 - with Matlab function filter()
M = K*Np;                  % length of the filter: number of pp comonents * their length
fc = 1/K;                  % cut-off frequency normalized to Nyquist freq of new sampling 
h = K*fir1( M, fc,'low');  % Matlab function call
NUP = K*Nx;                % signal lenght after filtering
xUP1 = zeros(1,NUP);       % initization of xUP signal
for k=1:K
    xUP1(k:K:NUP) = filter( h(k:K:end), 1, x );   % polyphase components of h(n)
end
tUP = dt/K*(0:NUP-1);
figure; plot(t(m),x(m),'r.-',tUP(mUP),xUP1(mUP+M/2),'b.-'); grid; xlabel('t [s]'); title('xUP1(t)'); pause
%soundsc(xUP1,fsUP); pause

% Version #2 - with our implementation of filtering
h = h(1:end-1);
hpp = reshape(h,K,Np); hpp=hpp'; hpp=fliplr(hpp); % building matrix of polyphase components
xUP2 = zeros(1,NUP);                              % of h(n) with their time-reversing
for k=1:Nx/K-Np
    n =  1+(k-1)*K : K+(k-1)*K; 
    nn = 1+(k-1)   : Np+(k-1);
    xUP2(1,n) = x(nn)*hpp;  % signal fragment is multiplied by matrix with pp h(n) components
end
figure; plot(t(m),x(m),'r.-',tUP(mUP+M-(K-1)),xUP2(1,mUP+M/2),'b.-'); grid; xlabel('t [s]'); title('xUP2(t)'); pause
figure; plot(mUP,xUP1(mUP),'ro-',mUP+M-(K-1),xUP2(mUP),'bx-'); grid; xlabel('m'); title('xUP1(m) & xUP2(m)'); pause
%soundsc(xUP2,fsUP); pause

% ##############################
% Fast decimation (L times DOWN) - samples to be removed are not computed
M = L*Np;                  % length of the filter: number of pp comonents * their length
fc = 0.95*1/L;             % cut-off frequency normalized to Nyquist freq of new sampling 
h = fir1( M-1, fc,'low');  % Matlab function call doing the same
Nz = rem(Nx,L); Napp=L-Nz; x(Nx+1:Nx+Napp) = zeros(1,Napp);  Nx=Nx+Napp;  % appending zeros at the end of x(n)
ND = Nx/L;                 % signal lenght after filtering
xD1 = zeros(1,ND);         % initization of xDOWN signal
for k=1:ND-Np
    nstart = M + (k-1)*L;
    xD1(k) = sum( x( nstart : -1 : nstart-(M-1) ) .* h ); % filtering
end
tD = (L*dt)*(0:ND-1) + dt*M/2;
figure; plot(t(m),x(m),'r.-',tD(mD),xD1(mD),'b.-'); grid; xlabel('t [s]'); title('xDOWN1(t)'); pause
%soundsc(xD1,fsD); pause

% ###################################
% Polyphase decimation (L times DOWN)
xD2 = zeros(1,ND);         % initization of xDOWN signal
for k=1:L
    xD2 = xD2 + filter( (h(k:L:end)), 1, x(L-k+1:L:end) );
end
tD = (L*dt)*(0:ND-1); % + dt*(M-1)/2;
figure; plot(mD,xD1(mD),'ro-',mD,xD2(mD+Np-1),'bx-'); grid; xlabel('m'); title('xDOWN1(m) & xDOWN2(m)'); pause
figure; plot(t(m),x(m),'ro-',tD(mD)+M*dt/2,xD2(mD+Np-1),'bx-'); grid; xlabel('t [s]'); title('xDOWN2(t)'); pause
%soundsc(xD2,fsD); pause
