function [p,L,xi] = lagrangeTZ(t,x,ti)
% Lagrange interpolation
% Input: points (t(n),x(n)), required points for ti(k) 
% Output: p - main polynomial, L - matrix with Lagrange polynomials,
%         xi - interpolated values.

% Calculate polynomial coefficients
N=length(t); L = zeros(N,N); p = zeros(1,N);
for n = 1:N
    if(n==1)     prodt(1) = prod(t(1)-t(2:N));   proots = t(2:N);
    elseif(n==N) prodt(N) = prod(t(N)-t(1:N-1)); proots = t(1:N-1);
    else         prodt(n) = prod(t(n)-[t(1:n-1) t(n+1:N)]); proots=[t(1:n-1) t(n+1:N)];
    end
    Ln = poly(proots)/prodt(n);  % Lagrange polynomial coefficients for n
    L(n,1:N) = Ln;               % storing
    p = p + x(n) * Ln;           % calculation of polynomial coefficients 
end

% Calculate interpolated function values using Lagrange polynomials
Ni=length(ti);
for k=1:Ni
    xi(k)=0;
    for n=1:N
        if(n==1)      dti = ti(k)-t(2:N);
        elseif(n==N)  dti = ti(k)-t(1:N-1);
        else          dti = ti(k)-[t(1:n-1) t(n+1:N)];
        end
        xi(k) = xi(k) + x(n)*prod(dti)/prodt(n);
    end    
end