function [y, e, h] = adaptTZ(d,x,M,mi,gamma,lambda,delta,ialg)
% INPUT:
%    d, x          - input signals
%    M             - adaptive filter length
%    ialg          - 1=LMS, 2=NLMS, 3=RLS adaptive filter algorithm
%    mi, gamma     - adaptation variables for NLMS (1) and NLMS (1+2)
%    lambda, delta - adaptation variables for RLS
% OUTPUT:
%    y, e          - output signals
%    h             - final filter weights

% Initialization
h  = zeros(M,1);                % filter weights
bx = zeros(M,1);                % input buffer for x(n) samples
Rinv = delta*eye(M,M);          % inverse of auto-correlation matrix Rxx^{-1}

% Main loop
for n = 1 : length(x)           % adaptive filtering loop
    bx = [ x(n); bx(1:M-1) ];   % take new sample of x(n) into the buffer
    y(n) = h' * bx;             % filter x(n): y(n) = sum( h .* bx)
    e(n) = d(n) - y(n);         % calculate error e(n)
    if(ialg==1)      % LMS
        h = h + mi * e(n) * bx;                 % update filter weights
    elseif(ialg==2)  % NLMS
        energy = bx' * bx;                      % energy of signal x(n)
        h = h + mi/(gamma+energy) * e(n) * bx;  % update filter weights
    else             % RLS
        Rinv = (Rinv - Rinv*bx*bx'*Rinv/(lambda+bx'*Rinv*bx))/lambda; % update
        h = h + Rinv * bx * e(n);                                     % update
    end
end