function [we, de, Ae, pe] = BY1_LC(x,NI)
 % Leakage correction for BY1 interpolated DFT
 % x - input signal x=A*cos(w*n+p).*exp(-d*n)
 % NI - number of iterations, for NI=0 BY1 without LC is computed
 % Estimated values:
 % we - frequency (rad), de - damping, Ae - amplitude, pe - phase (rad)
 N = length(x);
 Xw = fft(x);                          % computation of DFT

[Xabs, ind] = max(abs(Xw(1:round(N/2))));  % find maximum 
%stem( abs(Xw) ), pause

 k = [ind-1 ind ind+1];                % DFT bins around maximum
 dw = 2*pi/N;                          % DFT frequency step
 wkm1= (k(1)-1)*dw;	               % frequency of DFT bin with index k-1
 wk = (k(2)-1)*dw;	               % frequency of DFT bin with index k
 wkp1= (k(3)-1)*dw;	               % frequency of DFT bin with index k+1
 [we, de, Ae, pe, lam] = BY1_in_LC(wkm1,wk,wkp1,Xw(k),N);
 
 %% Leakage correction for negative frequencies
 wkk = [wkm1 wk wkp1];
 for iter=1:NI
 	for m=1:3;
 		Xw_correction(m) = ...
 		(Ae/2)*( exp(-j*pe)*(1-conj(lam)^N)/(1-conj(lam)*exp(-j*wkk(m)))); 
    end
 	Xw_correction = Xw(k) - Xw_correction;
 	[we, de, Ae, pe, lam] = BY1_in_LC(wkm1, wk, wkp1, Xw_correction, N);
 end
 
 function [we, de, Ae, pe, lam] = BY1_in_LC(wkm1, wk, wkp1, Xw, N);
 r = ( -exp(-j*wk)+exp(-j*wkm1) )/( -exp(-j*wkp1)+exp(-j*wk) );
 R = ( Xw(1)-Xw(2) )/( Xw(2)-Xw(3) );
 lam= exp(j*wk)*(r-R)/( r*exp(-j*2*pi/N)-R*exp(j*2*pi/N) );
 we = imag(log(lam));
 de = -real(log(lam));
 if round(1e6*R)==-1e6  %% coherent sampling, d=0
 	Ae = 2*abs(Xw(2))/N;
 	pe = angle(Xw(2));
 else
 	c = (1-lam^N)/(1-lam*exp(-j*wk));
 	c = 2*Xw(2)/c;
 	Ae = abs(c);
 	pe = angle(c);
 end
