function [we, de, Ae, pe] = BY1_LC_multi(x,ind,NI,P)
% Leakage correction for BY1 interpolated DFT
% x - input signal x: sum of Ae*cos(we*n+pe).*exp(-de*n) (1)
% ind - FFT bin of selected maxima
% NI - number of iterations, for NI=0 BY1 without LC is computed
% P - buffer size 2*P+1 around maxima

% Estimated values:
% we - frequency (rad)
% de - damping
% Ae - amplitude
% pe - phase (rad)

N = length(x);
Xw = fft(x);   % computation of DFT (3)

%figure; subplot(111); stem(abs(Xw)); title('abs(Xfft)'); grid; pause
%size(Xw), pause
sc = 1;         % scale
dw = 2*pi/N; 	% DFT frequency step

% Correct for one maximum only:
% [Xabs, ind] = max(abs(Xw(1:round(N/2))));
% k = [ind-1 ind ind+1], pause

M = length(ind);
k3 = [ ind-1; ind; ind+1 ];
Xw3 = [ Xw(ind-1); Xw(ind); Xw(ind+1) ];
wk3 = (k3-1)*dw;

[we, de, Ae, pe, lam] = BY1_in_LC_multi( wk3, Xw3, N );

%###############################################
if(1)  % Plus leakage correction
    
kP = ind;
for k = 1:P
    kP = [ ind-k; kP; ind+k ];
end

wkP = (kP-1)*dw;
XwP = Xw(ind);
for k=1:P 
    XwP = [ Xw(ind-k); XwP; Xw(ind+k) ];
end    
XwP_corrected = zeros(2*P+1,M);

%% Leakage correction for other frequencies
%wkk = [wkm1; wk; wkp1];    % teraz wk3

for iter=1:NI
  % iter
    XwP_corrected = XwP;
    for m = 1:M    % component to be corrected
        for p = 1:2*P+1   % its frequency lags 
            for mm = 1 : M    % alliasing from the other components
                if(mm~=m) 
	  	           Xw_correction_mm = ...
		           (Ae(mm)/sc)*( exp(j*pe(mm))*(1-(lam(mm))^N)/(1-(lam(mm))*exp(-j*wkP(p,m))));  %(4)
                   XwP_corrected(p,m) = XwP_corrected(p,m) - Xw_correction_mm;                   %(9)
                end
            end
        end    
    end
    if( P > 1 )
      [ dummy indP ] = max(abs(XwP_corrected));
    % indP, pause
    % indP = (P+1)*ones(1,M);
      for m=1:M
          Xw3_corrected(:,m) = XwP_corrected( indP(m)-1 : indP(m)+1, m );
        % k3_corrected(:,m) = k3(:,m) + [ indP(1,m)-4; indP(1,m)-4; indP(1,m)-4 ]; wk3_corrected = (k3_corrected-1)*dw;
      end
      k3_corrected = k3 + [ indP-(P+1); indP-(P+1); indP-(P+1) ]; wk3_corrected = (k3_corrected-1)*dw;
    % k3, indP, k3_corrected, pause
    else
       Xw3_corrected = XwP_corrected;
       k3_corrected = k3; wk3_corrected = (k3-1)*dw;
    end   
    
    [we, de, Ae, pe, lam] = BY1_in_LC_multi(wk3_corrected, Xw3_corrected, N);
  
end

end
%###############################################
