function [Om, D]=fFastTLS(x,r)
% Total least squares for only one damped sine
% x - analyzed signal
% r - reduction/decimation order: 1=no reduction, 2 = 2 times, 3 = 3 times, ...
% J. Yuan, M. Torlak: "Modeling and Estimation of Transient Carrier
% Frequency Offset in Wireless Transceivers", IEEE Trans. on Wireless
% Communications, vol. 13 (2014), no.7, pp.4038-4049

N = length(x);  % signal length

Y(:,1) = x(1:N-2*r);
Y(:,2) = x(r+1:N-r);
Y(:,3) = x(2*r+1:N).';

[U,S,V] = svd(Y);           % SVD
a = V(:,3);                 % right singular vector <--> the least singular value

z = roots(a); z = log(z);   % without fliplr(a)
Om = abs(imag(z(1)))/r;     % 
D  = real(z(1))/r;          % without minus
 
  
 

  
