function [Om, D]=fKT(x,K)
% Kumaresan-Tufts method
% x - analyzed signal
% K - assumed number of real damped sine components

  inside = 1;                              % 1/0=inside/outside the unit circle
  M = 2*K;                                 % number of complex components
  N = length(x);                           % numer of signal samples 
  P = floor(N*3/4);                        % linear prediction order P = 3/4*N 
  Y = hankel( x(2:N-P+1), x(N-P+1:N) );    % backward prediction data matrix 
  y = x(1:N-P)';                           % backward prediction data vector 
  [U,S,V] = svd(Y,0);                      % singular value decomposition 
  S = diag(S);                             % take only main diagonal 
  bias = mean(S(M+1:min([N-P,P])));        % bias compensation 
  b = -V(:,1:M) * (diag(1./(S(1:M)-bias)) * (U(:,1:M)'*y));     % LP coefficients 
  if(inside==1)                            % INSIDE THE CIRCLE
      z = roots([b(length(b):-1:1);1]);    % roots of poly
      p=log(z); p = p( find(real(p)<0) );  % ln()
      Om = imag(p); [Om indx] = sort( Om, 'ascend' );  Om=Om(K+1:2*K); % pulsation
      D = -real(p(indx(K+1:2*K)));                                     % damping
  else                                     % OUTSIDE THE CIRCLE
      z = roots([ 1; b]);                  % roots of poly
      p=log(z); p = p( find(real(p)>0) );  % ln()
      Om = imag(p); [Om indx] = sort( Om, 'descend' );  Om=Om(1:K); % pulsation
      D = real(p(indx(1:K)));                                      % damping
  end  
% plot(real(z),imag(z),'o'); pause, plot(real(p),imag(p),'o'); pause
