function [Om, D]=fSTMB(x,K,NI)
% x - analysed signal
% K - number of damped components A*cos(Om*n+ph).*exp(-D*n)
% NI - number of iterations, for NI=0 the Prony method is used
% Om - estimated radial frequency [rad]
% D - estimated damping

N = length(x);                             % number of signal samples
P = 2*K;                                   % prediction order
FirstCol = x(P:N-1); FirstRow = x(P:-1:1); % first row and column of Toeplitz matrix
X = toeplitz( FirstCol, FirstRow ); y = x(P+1:N).'; % signal matrix
a = X\y; % or inv(X'*X)*X'*y; or a = pinv(X)*y;     % first Prony solution
a1 = [1; -a];                              % first filter from Prony method
%a1 = [1; zeros(P,1) ];                    % or simpler filter initialization
delta = zeros(1,N); delta(1)=1;            % unit impulse
while ( NI )                               % iterative solution improvement
   v = filter(1, a1, x);                              % input filtering
   u = filter(1, a1, delta);                          % impulse filtering
   V = toeplitz( v, [ v(1) zeros(1,P) ] );            % input matrix
   U = toeplitz( u, [ u(1) zeros(1,P-1) ] );          % impulse matrix
   VU = [ V(1:N,:) U(1:N,:) ];                        % combining two matrices
   A = VU(:,2:end); d = VU(:,1);                      % 
   c = A\d; % c = inv(A'*A)*A'*d; % c = pinv(A)*d;    % solution, three alternatives
   a1 = [ 1; -c(1:P) ];                               % iterative update
   NI = NI-1;                                         % decrease counter
end                                                            %
p = roots( a1 );  p = log(p);                                  % roots, ln()
Om = imag(p); [Om indx] = sort( Om, 'ascend' ); Om = Om(K+1:2*K); % pulsation
D = -real( p(indx(K+1:2*K)) );                                    % damping
