% lab14_ex_speech_spectrum_envelope

  clear all; close all;
  
  [x,fs] = audioread('a8.wav');  % a8, e8, i8, o8, u8
% [x,fs] = audioread('speech.wav',[21000,23550]);  % read it from disc, [from, to]
  
  x = x.';
  
  Nx = length(x); n=0:Nx-1;            % find number of samples, sample numbers
  dt = 1/fs; t = dt*n;                 % calculate sampling period and time stamps
  figure; plot(t,x,'b-'); xlabel('t (s)'); title('speech(t)'); grid; pause
  
  Mlen  = 240;  % 240 Fragment length for fs=8000; samples corresponding to 30 ms
  Mstep = 180;  % 180
  if(fs==8000)  Np = 8;                % 8; number of poles of IIR LPC filter
  else          Np = 12                % 12; number of poles of IIR LPC filter
  end    
  Nc = 16;                             % 16, 24; number of cepstral coefficients
  
  c=[]; cw=[]; Cx=[];                  % clearing
  
  Nframes = floor((Nx-Mlen)/Mstep+1);  % number of speech frames to process
  x = x - 0.9375*[0 x(1:Nx-1)];        % initial filtration (high-freq pre-emphasis)
  
  figure;
  Nfft=1024; k=1:Nfft/2+1;  fk=fs/Nfft*(k-1);  % for spectrum display

% MAIN LOOP
  alf = 0 : pi/100 : 2*pi; cosi=cos(alf); sini=sin(alf);  % TZ
  X1hist = zeros(Nframes,Nfft/2+1);
  X2hist = zeros(Nframes,Nfft/2+1);
  X3hist = zeros(Nframes,Nfft/2+1);
  fehist = zeros(Nframes,Np/2);
  for  nr = 1 : Nframes
      % Take next data to the buffer
        n = 1+(nr-1)*Mstep : Mlen + (nr-1)*Mstep;
        bx = x(n);
      % Do signal pre-processing
        bx = bx - mean(bx);                      % remove mean value
      % bx = bx .* hamming(Mlen)';               % use window
       
      % METHOD #1: Classical FFT of signal fragment
        X1   = abs( fft( bx, Nfft ) );
        X1dB = 20*log10( X1 );
      
      % METHOD #2: Spectrum envelope via prediction filter coefficients
      % [a] = lpc(bx,Np);                                        % Matlab lpc() function
        r=xcorr(bx,'biased');[a] = levinson(r(Mlen:Mlen+Np),Np); % Octave/Matlab levinson() function 
        p = roots(a); p = p.';                   % roots of LPC polynomial
        phi = angle(p); [ phi indx ] = sort( phi, 'descend' );
        Ome = phi(1:Np/2); fe = Ome/(2*pi)*fs;   % estimation of peak frequencies
        p = p(indx(1:Np/2));                     % poles for positive frequencies
        X2   = abs( freqz(1,a,fk,fs) );	         % MATLAB LPC filter frequency response
        X2dB = 20*log10( X2 );
      
      % METHOD #3: Spectrum envelope via cepstrum
        c = abs(fft(bx,Nfft)).^2;
        indx = find( c < 1000*eps ); c(indx) = 1000*eps*ones(1,length(indx)); % against NaN, Inf
        c = ifft( log10(c) );
        cfiltr = zeros(1,Nfft); cfiltr(1:Nc) = c(1:Nc); cfiltr(Nfft:-1:Nfft-Nc+2) = c(2:Nc);
        X3 = sqrt( 10.^ real(fft( cfiltr )) );
        X3dB = 20*log10( X3 );
        
        if(0) % Here we switch ON/OFF 1/0 ploting whole cespstrum of speech fragment
           subplot(111)
           plot(k,abs(c(k)),'b.-'); grid; title('Cepstrum |c(n)|');  % for choosing Nc
           pause(0.05); % here we waiting 0.05 s, like in cinema, fig after fig
        end
        if(1) % Here we switch ON/OFF 1/0 ploting local signal spectrum and its envelope
           subplot(211);
           plot(fk,X1(k),'m-',fk,X2(k),'b-',fk,X3(k),'r-');
           xlabel('f (Hz)'); title('|X(f)|'); grid; % hold on;
           subplot(212);
           plot(fk, X1dB(k),'m-',fk,X2dB(k),'b-',fk,X3dB(k),'r-');
           xlabel('f (Hz)'); title('|X(f)| (dB)'); axis([0,fs/2,-40,40]); grid; % hold on;
           pause(0.05); % here we waiting 0.05 s, like in cinema, fig after fig
        end
        if(0) % Here we switch ON/OFF 1/0 showing movement of transfer function poles
           subplot(111);
           plot(real(p), imag(p),'bx',cosi,sini,'k-'); title('Poles-Zeros'); grid; % hold on;
           pause(0.05); % here we waiting 0.05 s, like in cinema, fig after fig
        end

        X1hist(nr,:) = X1dB(k);  % DFT spectrum history
        X2hist(nr,:) = X2dB(k);  % spectral envelope via LPC coefficients
        X3hist(nr,:) = X3dB(k);  % spectral envelope via cepstral coefficients
        fehist(nr,:) = fe;       % frequency of spectral peaks
  end
    
  X1mean = mean( X1hist );   % mean spectrum for DFT
  X2mean = mean( X2hist );   % mean spectral envelope via LPC coefficients
  X3mean = mean( X3hist );   % mean spectral envelope via cepstral coefficients

  figure;                    % display of mean spectra
  plot(fk, X1mean,'m-', fk,X2mean,'b-', fk,X3mean,'r-');
  xlabel('f (Hz)'); title('FFT (green), LPC (blue), CEPSTRUM (red)');
  %axis([0,fs/2,-40,40]);
  grid; pause                
  
  figure;                    % display of formant frequencies
  plot(fehist); title('Change of peak resonance frequencies');
  grid; pause
  
