% lab15_ex_filterbank.m
% MP2 filter bank - regular version

clear all; close all;

Nmany = 2000;                 % number of audio frames
N = 512;                      % window/filter length
M = 32;                       % number of sub-bands/channels 

% Test signal
Nx = N+M*(Nmany-1);           % number of signal samples
num = [ 2, 14 ];              % number of freq channels with signal components
fs = 44100;                   % sampling frequency
df = (fs/2)/M;                % sub-band width
fcentr = (0:M-1)*df + df/2;   % sub-band central frequencies
fc = fcentr(num);             % signal component frequencies
x = zeros(1,Nx);              % signal samples initialization
n = 1:Nx; f = fs/Nx*(0:Nx-1); % initialization: sample indexes, frequency values
for k=1:length(fc)                        %
    x = x + cos(2*pi*fc(k)/fs*(0:Nx-1));  % test signal generation
end                                       %
% x = randn(1,Nx);                                 % alternative signal 1
%[x,fs]= audioread('bach44100.wav',[1,Nx]); x=x';  % alternative signal 2

    figure; plot(x(n)); title('x(n)'); pause
    figure; plot(f,20*log10(abs(fft(x(n))))/Nx); title('|X(f)| [dB]'); pause

% Filter bank matrices
win = 2*sqrt(M)*prototype(N);             % prototype filter
figure; plot(win); title('w(n)'); pause   %
win = ones(M,1)*win;                      %
[n,k] = meshgrid(0:(N-1),0:(M-1));        %
C = win.*cos(pi/M*(k+1/2).*(n-M/2));      % analysis matrix with filter
D = win.*cos(pi/M*(k+1/2).*(n+M/2));      % synthesis matrix with filter
D = D';                                   %

% Checking FB frequency response
figure;
K=5; KN=K*N; f = fs/KN*(0:KN-1);
figure; plot(f,20*log10(abs(fft(C',K*N)))); title('Bank of filters [dB]'); xlabel('f [Hz]'); grid; pause

% Analysis - synthesis
bx = zeros(1,N);
sb = zeros(Nmany,M);
y1 = zeros(1,Nx); % output 1
for m=1:Nmany
    bx = [ x(m*M:-1:(m-1)*M+1) bx(1:N-M) ];
    BX = C*bx';   % Analysis filter bank
    sb(m,:) = BX;
  % doing something 
  % BX(num(2)) = 0;
    by = D*BX;    % Synthesis filter bank  
    n1st = 1+(m-1)*M; nlast = N + (m-1)*M;
    n = n1st : 1 : nlast;
    y1( n ) = y1( n ) + by';
end    

    figure;
    subplot(411); plot(sb(:,num(1)));   title('sb( no1 )'); 
    subplot(412); plot(sb(:,num(1)+2)); title('sb( no1 + offs)');
    subplot(413); plot(sb(:,num(2)));   title('sb( no2 )');
    subplot(414); plot(sb(:,num(2)+2)); title('sb( no2 + offs )'); pause

% Output signal
Noffs=N-M+2; n1=1:(Nx-N)-Noffs+1; n2=Noffs:(Nx-N);
xr=x(n1); yr=y1(n2);
SNR_sb1_dB = 10*log10( sum(xr.^2) / sum((xr-yr).^2) ), pause

    figure; plot(n1,xr,'r',n1,yr,'b'); title('INPUT (red) OUTPUT (blue)');  pause
    figure; plot(n1,xr-yr); title('Difference IN-OUT');  pause

% Quantization of the original signal
b = 6; xq = fix( 2^b * x ) / 2^b;

% Quantization of sub-band signals
sbref = sb;
    figure; mesh(sb); xlabel('sb'); ylabel('time'); title('sb'); pause
sbmax = max(abs(sb));
sb = sb ./ sbmax;
    figure; mesh(sb); title('sb/sbmax'); pause
b  = [ 8*ones(1,M/4) 6*ones(1,M/4) 4*ones(1,M/4) 0*ones(1,M/4) ]; sc = 2.^b; % bits for subbands 
%b = 1*ones(1,M); sc = 2.^b;
sb = sb .* sc;
    figure; mesh(sb); title('sb * scale'); pause
sb = fix( sb );
sb = sb ./ sc;
    figure; mesh(sb); title('fix(sb) / scale'); pause   
sb = sb .* sbmax;
    figure; mesh(sb); title(' sb * sbmax'); pause
    figure; mesh(sb-sbref); title('sb - sbref'); pause

% Signal synthesis from sub-bands
y2 = zeros(1,Nx); % output 2
for m=1:Nmany
    BX = sb(m,:)';
    by = D*BX;    % synthesis filter bank
    n1st = 1+(m-1)*M; nlast = N + (m-1)*M;
    n = n1st : 1 : nlast;
    y2( n ) = y2( n ) + by';
end
xqs = y2;

% Spectra of original and quantized signals
%x = x / max(abs(x));
if(1)
   w = hanning(Nx)'; f=fs/Nx*(0:Nx-1);
   X = 20*log10( 2*abs(fft(w.*x))/sum(w) );
   Xq = 20*log10( 2*abs(fft(w.*xq))/sum(w) ); 
   Xqs = 20*log10( 2*abs(fft(w.*xqs))/sum(w) );
else
   [X, f]=periodogram(x, [],512,fs,'power','reassigned'); X=10*log10(X);
   [Xq,f]=periodogram(xq,[],512,fs,'power','reassigned'); Xq=10*log10(Xq);
   [Xqs,f]=periodogram(xqs,[],512,fs,'power','reassigned'); Xqs=10*log10(Xqs);
end
    figure; plot(f,X,'r-',f,Xq,'b-',f,Xqs,'g-'); xlabel('f (Hz)');
    title('Power (dB) - original (red), quant orig (blue), quant sb (green) '); grid; pause

% Comparison of the original and quantized signals (two methods)
yr = y2(n2);
SNR_sb2_dB = 10*log10( sum(xr.^2) / sum((xr-yr).^2) ), pause
SNR_all_dB = 10*log10( sum(xr.^2) / sum((xr-xq(n1)).^2 )  ),
pause
    figure; plot(n1,xr,'r',n1,yr,'b'); title('INPUT (red) OUTPUT (blue)');  pause
    figure; plot(n1,xr-yr); title('Difference IN-OUT');  pause
