% lab15_ex_fundamentals.m
% Psycho-acoustics fundamentals of human hearing system
  clear all; close all;
  
% General parameters
  fs = 44100;       % sampling frequency
  Nfft = 1024;      % FFT length
  Nfft2 = 513;      % number of displayed FFT coefficients

% Parameters for sampling in frequency 
  df = fs/Nfft; fmax = fs/2; f = 0:df:fmax; fkHz = f/1000; Nf = length(f);  

% Critical bands  
  fcrit = [    0  100  200  300  400  510  630  770  920 1080  1270  1480 1720  ...
            2000 2320 2700 3150 3700 4400 5300 6400 7700 9500 12000 15500 25000 30000 ];

% Changing:  kHz --> bark  and  bark --> kHz
  b1 = 13*atan(0.76*fkHz) + 3.5*atan((fkHz/7.5).^2); b = b1; Nb = length(b);
  b2 = 7*asinh(1000*fkHz/650);
  figure; plot(fkHz, b1, 'r', fkHz, b2, 'b'); title('f (kHz) -> b (bark)'); xlabel('f (kHz)'); grid; pause
  
  fb2 = 650*sinh( b2/7 )/1000;
  figure; plot(b2, fb2); title('b (bark) -> f (kHz)'); xlabel('b (bark)'); ylabel('f (kHz)'); grid; pause
  
% Width of a critical band as a function of frequency
  dfcrit = 25 + 75*(1+1.4*fkHz.^2).^0.69; dfcrit = dfcrit/1000;
  figure; plot(fkHz, dfcrit); title('Critical band width'); xlabel('f (kHz)'); ylabel('df (kHz)'); grid; pause
  
% Absolute threshold of hearing
  progABS = 3.64*fkHz.^(-0.8) - 6.5*exp(-0.6*(fkHz-3.3).^2) + 10^(-3)*fkHz.^4;  % in decibels
  indx = find( (f>0) & (f<18000) ); 
  figure; plot( fkHz(indx), progABS(indx) ); title('Absolute threshold of hearing'); xlabel('f (kHz)'); grid; pause
  
% Energy distribution along the basilar membrane
  dbark = -10 : 0.1 : 20;  
  sprd = 15.81 + 7.5*(dbark+0.474) - 17.5*sqrt(1+(dbark+0.474).^2); 
  figure; plot(dbark, sprd); title('Energy distrubution along basilar membrane'); xlabel('f (Bark)'); grid; pause
  
% ======================================  
% Example of audio signal analysis
% ======================================  

% Parameters
  fm = [ 2500 5800];       % frequencies of tones (Hz)
  am = [    1    1];       % amplitudes of tones in the range [0, 1]
  Em = 2^15*am;            % amplitudes scaled to quantization levels
  EmdB = 20*log10( Em );   % amplitudes in dB
  fmkHz = fm/1000;         % frequencies in kHz
  Nmask = length(fm);      % numbers of signals
  bm = 13*atan(0.76*fmkHz) + 3.5*atan((fmkHz/7.5).^2);  % kHz --> bark 
  
% Test signal generation  
  N = Nfft; Nx = 5*fs; dt=1/fs; t=(0:Nx-1)*dt;
  x = zeros(1,Nx);
  for k = 1 : Nmask
      x = x + am(k)*sin(2*pi*fm(k)*t);
  end
% x = randn(1,N);  
  figure; plot(x); title('Signal x(n)'); grid;
  disp('Please, adjust speakers volume, press any key and wait!'); pause
  sound(x,fs);
  
% Window
  w = hanning(N,'periodic'); w = w';
  
% Power spectral density
  X = fft(x(1:N).*w,N)/sum(w); X = 2*X(1:N/2+1);
  X = X.*conj(X)/2;
  XdB = 10*log10(X);
  XdB = XdB + (96-max(XdB)); % normalization to level 96 dB
  figure; plot(fkHz,XdB); title('PSD |X(k)|^2 (V^2)'); grid; pause
  
% Spectral flatness measure (SFM) in critical bands with tones
  bnr = ceil(bm);    % number of critical band with excitation
  fkmax = fcrit(bnr+1); fkmin = fcrit(bnr);
  for k = 1 : Nmask
      nk = find( (fkmin(1,k)< f) & (f<=fkmax(1,k)) ); Nk = length( nk );   
      SFM(k) = max( 10*log10( prod(X(nk))^(1/Nk) / ((1/Nk)*sum(X(nk)))), -300 )
  end   
  
% Tonality index alfa
  SFMmax = -60; alfa = min( SFM/SFMmax, 1)
  
% Masking index
  imask = 2 + 2.05*atan(0.25*fmkHz) + 0.75*atan( fmkHz.^2/2.56)
  
% Offset between excitation level and masking threshold
% Offs = alfa*(14.5+bnr) + (1-alfa)*5.5                    % approximately
  Offs = alfa.*(14.5+bnr) + (1-alfa).*imask                % more precisely
  
% ===============================================
% Excitation and masking curves in Hz
% ===============================================  
  E = zeros(Nmask,Nf); EdB = zeros(Nmask,Nf); % basilar membrane excitations in rows
  Esum = zeros(1,Nf);  Tsum = zeros(1,Nf);    % total: excit. energy, masking threshold
  for k = 1 : Nmask                           % consecutive signals (maskers)
      S1(k) = 31;                                          % front slope (dB/bark)
      S2(k) = 22 + min(0.23/fmkHz(k),10) - 0.2*EmdB(k);    % back slope  (dB/bark)
      ib = 1; figure;                                      % work counter
      for ff = 0 : df : fmax                               % in Hz scale
          ffk = ff/1000;                                   % Hz  --> kHz
          bb = 13*atan(0.76*ffk) + 3.5*atan((ffk/7.5).^2); % kHz --> bark 
          if( bb <= bm(k) ) EdB(k,ib) = EmdB(k) - S1(k)*(bm(k)-bb);
          else              EdB(k,ib) = EmdB(k) - S2(k)*(bb-bm(k));
          end
          ib = ib+1;
       end
       E(k,:) = 10.^(EdB(k,:)/20);                               % dB --> linear
       Esum(1:Nf) = Esum(1:Nf) + E(k,:).^2;                      % total excitation
       TdB(k,:) = EdB(k,:)-Offs(k)*ones(1,Nf);                   % k-th threshold
       Tsum(1:Nf) = Tsum(1:Nf) + ( E(k,:)/10^(Offs(k)/20) ).^2;  % total theshold
       
       plot(fkHz, EdB(k,:)); title('k-th excitation (dB) '); xlabel('f (Hz)'); grid; pause
       plot(fkHz,   E(k,:)); title('k-te excitation      '); xlabel('f (Hz)'); grid; pause
  end
  TsumALL = sqrt( Tsum + (10.^(progABS/20)).^2 ); Tsum = sqrt( Tsum ); Esum = sqrt( Esum );
  figure; plot(f, Esum, 'r', f, Tsum, 'b', f, TsumALL,'k');
  title('LINEAR: Excitation (RED) Threshold (BLUE) ThresABS (BLACK)'); xlabel('f (Hz)'); grid; pause
  
  TsumALLdB = 20*log10( TsumALL );  TsumdB = 20*log10( Tsum ); EsumdB = 20*log10( Esum );
  figure; plot(f, EsumdB, 'r', f, TsumdB, 'b', f, TsumALLdB,'k');
  title('DECIBELS: Excitation (RED) Threshold (BLUE) ThresABS (BLACK)'); xlabel('f (Hz)'); grid; pause

% Calculate Signa-to_Mask Ratio (SMR)
  SMR = XdB - TsumALLdB;
  figure; plot(f, SMR); title('SMR function (dB))'); xlabel('f (Hz)'); grid; pause
