% lab16_ex_filter_design.m
% Design of 2D filters
clear all; close all; figure;

L = 15;  % width of filter mask (square)  

K = (L-1)/2; df = 0.5/K;         % variables used for filter weights
m = ones(L,1)*(-K:K);            % generation and figure description
n = (-K:K)'*ones(1,L);           %
fm = ones(L,1)*(-0.5:df:0.5);    %
fn = (-0.5:df:0.5)'*ones(1,L);   %
  
% Read image to be processed
[x,cmap] = imread('Cameraman.tif');
imshow(x,cmap), title('Image'); pause
[N, M] = size(x);
x = double(x);

% Filters originated from the Gaussian function
sigma = 1.4; df = 0.5/K;
g0  =  1/(2*pi*sigma^2) .* exp( -(m.^2+n.^2)/(2*sigma^2) ); % Gaussian function
g1m = -m/(sigma^2) .* g0;                                   % derivative over "m"   
g1n = -n/(sigma^2) .* g0;                                   % derivative over "n"
g2  = (m.^2 + n.^2 - 2*sigma^2)/(sigma^4) .* g0; % second derivative over "m", "n"

figure
subplot(221); mesh(m,n,g0); title('Gaussian filter');
subplot(222); mesh(m,n,g2); title('Laplacian of Gaussian');
subplot(223); imshow( conv2(x,g0,'same'),cmap );
subplot(224); imshow( conv2(x,g2,'same'),cmap ); pause;
figure
subplot(231); mesh(m,n,g1m); title('Gradient "m"');
subplot(232); mesh(m,n,g1n); title('Gradient "n"');
subplot(234); imshow( conv2(x,g1m,'same'),cmap );
subplot(235); imshow( conv2(x,g1n,'same'),cmap );
subplot(236); imshow( sqrt( conv2(x, g1m,'same').^2 + conv2(x, g1n,'same').^2 ), cmap ); pause

% 2D WINDOW METHOD
chka = 1;   % 0 = rectangular mask, 1 = cicular mask
w = hamming(L); w = w * w'; % 2D window
figure; mesh(m,n,w); colormap([0 0 0]); title('2D Window'); pause
   
if(chka==0) % Rectangular freq response - imp. responses of two low-pass filters 
   f0=0.25; wc = pi*f0; sinc = sin(wc*(-K:K))./(pi.*(-K:K)); sinc(K+1)=f0; lp1=sinc'*sinc; 
   f0=0.50; wc = pi*f0; sinc = sin(wc*(-K:K))./(pi.*(-K:K)); sinc(K+1)=f0; lp2=sinc'*sinc; 
else        % Circular freq response  - imp. responses of two low-pass filters
   f0=0.25; wc = pi*f0; lp1 = wc * besselj( 1, wc*sqrt(m.^2 + n.^2) ) ./ (2*pi*sqrt(m.^2 + n.^2) ); lp1(K+1,K+1)= wc^2/(4*pi);
   f0=0.50; wc = pi*f0; lp2 = wc * besselj( 1, wc*sqrt(m.^2 + n.^2) ) ./ (2*pi*sqrt(m.^2 + n.^2) ); lp2(K+1,K+1)= wc^2/(4*pi);
end
   
lp = lp1;                                     % LowPass without 2D window
lpw = lp .* w;                                % with window
hp = - lp1; hp(K+1,K+1) = 1 - lp1(K+1,K+1);   % HighPass without 2D window
hpw = hp .* w;                                % with window
bp = lp1 - lp2;                               % BandPass without 2D window
bpw = bp .* w;                                % with window
bs = - bp; bs(K+1,K+1) = 1 - bp(K+1,K+1);     % BandStop without 2D window
bsw = bs .* w;                                % with window
for typ = 1 : 4                % show imp. response, its spectrum and filtered image
    switch (typ)               % choose filter type
    case 1, h = lp; hw = lpw;  % LP 
    case 2, h = hp; hw = hpw;  % HP
    case 3, h = bp; hw = bpw;  % BP
    case 4, h = bs; hw = bsw;  % BS 
    end
    figure
    subplot(221); mesh(m,n,h);  title('Filter h(m,n)');
    subplot(222); mesh(m,n,hw); title('Filter hw(m,n)');
    subplot(223); mesh(fm,fn,abs( fftshift2D(fft2(h)) ) ); title('|H(fm,fn)|'); 
    subplot(224); mesh(fm,fn,abs( fftshift2D(fft2(hw)) ) ); colormap([0 0 0]); title('|Hw(fm,fn)|'); pause 
    figure
    subplot(121); y = conv2(x, h,'same'); imshow(y,[min(min(y)),max(max(y))]);  title('After h(m,n)');
    subplot(122); y = conv2(x, hw,'same'); imshow(y,[min(min(y)),max(max(y))]); title('After hwind(m,n)'); pause
end
  
% FILER DESIGN IN FREQUENCY DOMAIN

% Work variables 
N=L+1; df = 0.5/(K+1);
m  = ones(L+1,1)*(-(K+1):K);       n  = (-(K+1):K)'*ones(1,L+1);
fm = ones(L+1,1)*(-0.5:df:0.5-df); fn = (-0.5:df:0.5-df)'*ones(1,L+1);
% 2D window - its shape and 2D spectrum  
w = hamming(N); w = w * w';

% Required frequency response - circular or rectangular
Q = round(K/2); % LP filter width          
H = zeros(N,N);
for k = N/2+1-Q : N/2+1+Q
    for l = N/2+1-Q : N/2+1+Q
        if( (k-N/2-1)^2 + (l-N/2-1)^2 <= Q^2) H(k,l) = 1; else H(k,l) = 0; end % circular   
    end      
end
% H(N/2+1-Q : N/2+1+Q, N/2+1-Q : N/2+1+Q) = ones(2*Q+1,2*Q+1); % rectangular

% Filter design and its testing
h = real( ifft2(fftshift2D(H)) ); h = fftshift2D(h);  % impulse response
hw = h .* w;                                          % with window
Hw = abs( fftshift2D( fft2( hw ) ) );                 % 2D spectrum
y = conv2(x, hw, 'same');                             % 2D filtration

% Figures
figure;
subplot(121); mesh( m,n,w ); title('2D window w(m,n)');
subplot(122); mesh( fm,fn,abs( fftshift2D(fft2(w)) ) ); title('|W(fm,fn)|');
colormap([0 0 0]); pause
figure;
subplot(221); mesh(fm,fn,H);  title('Required |H(fm,fn)|');
subplot(222); mesh(m,n,h);    title('2D filter h(m,n)');
subplot(223); mesh(fm,fn,Hw); title('2D filter freq response |Hw(fm,fn)|');
subplot(224); mesh(m,n,hw);   title('2D filter with window hw(m,n)');
colormap([0 0 0]); pause
figure;
subplot(121); imshow(y, cmap); title('Image after filtering'); y = y(1:128,65:192); 
subplot(122); imshow(y,[min(min(y)),max(max(y))]); title('Image fragment'); pause
