% lab16_ex_transforms.m
% 2D orthogonal transforms of images
% In Octave: >>pkg load signal[ENTER] 
clear all; close all; figure;

% Initialization - read image
[x,cmap] = imread('Cameraman.tif');    % read image to "x", color palette to "cmap"
imshow(x,cmap), title('Image'); pause  % display image using its palette
[M, N] = size(x);                      % ask for number of rows and columns
x = double(x);                         % pixel values to double (gray scale)
MN = min(M,N); N=MN; x=x(1:N,1:N);     % reduce image to square matrix
 
% Transformation matrices of 1D DCT (C) and 1D DFT (F)
c = [sqrt(1/N) sqrt(2/N)*ones(1,N-1)]; f = 1/sqrt(N); % normalizing coefficients
n = 0:N-1;                                            % indexes of samples
for k=0:N-1                                           % indexes of frequencies
    C(k+1,1:N) = c(k+1) * cos( pi*k*(n+1/2) / N );    % basis functions of 1D DCT
    F(k+1,1:N) = f      * exp(j*2*pi/N*k*n);          % basis functions of 1D DFT
end

% FILTERING using 2D DCT
  K = 64; H = zeros(M,N); H(1:K,1:K) = ones(K,K);        % frequency 0/1 mask
  X = conj(C) * x * conj(C).';                           % 2D DCT - IN
  X1 = dct2(x);  X2=dct(dct(x')');                       % 2D DCT - Matlab
  err1 = max(max(X-X1)), err2 = max(max(X-X2)), pause    % errors
  Y = X .* H;                                            % filtering in freq domain
  y = C.' * Y * C;                                       % 2D IDCT - OUT
  y1 = idct2(Y); y2 = ( idct( idct( Y )' ) )';           % 2D IDCT - Matlab
  err1 = max(max(y-y1)), err2 = max(max(y-y2)), pause    % errors
  XdB = scaledB( X ); YdB = scaledB( Y );                % scaling in dB
  
  figure;
  subplot(221); imshow(x, cmap);  title('Image');        % below only display
  subplot(222); imshow(XdB, cmap); title('2D DCT');
  subplot(223); imshow(YdB, cmap); title('2D DCT + Mask');
  subplot(224); imshow(y(1:128,65:192), cmap); title('Filtered image'); pause 
 
% FILTERING using 2D DFT (fftshift2D - re-ordering of 2D DFT coefficients)
  K = 32; H = zeros(M,N);                                      % frequency mask H (MxN)
  H(M/2+1-K : M/2+1+K, N/2+1-K : N/2+1+K) = ones(2*K+1,2*K+1); % center = (M/2+1, N/2+1)
  h = fftshift2D( real( ifft2( fftshift2D(H) ) ) );            % impulse response
  
  figure;
  subplot(121); imshow(255*H,cmap); title('Freq Mask');        % show freq response
  subplot(122); mesh(h); title('Filter imp. response'); pause  % show imp response
  
  X = conj(F) * x * conj(F).';  % 2D DFT - IN
  X1 = fft2(x)/N;               % 2D DFT - Matlab 1
  X2 = fft(fft(x').')/N;        % 2D DFT - Matlab 2
  err1 = max(max(X-X1)), err2 = max(max(X-X2)), pause % errors
  Xp = fftshift2D(X);           % 2D spectrum centering
  Yp = Xp .* H;                 % filtering = 2D DFT multiplied by filter H
  Y = fftshift2D(Yp);           % 2D spectrum de-centering (original order)
  y = F.' * Y * F;              % inverse 2D DFT - IN
  y1 = ifft2(Y)*N;              % inverse 2D DFT - Matlab 1
  y2 = (ifft(ifft(Y).'))'*N;    % inverse 2D DFT - Matlab 2
  err1 = max(max(abs(y-y1))), err2=max(max(abs(y-y2))), pause % errors
  y1 = real(y1);                % real part only, imaginary equal to 0
  y1f = y1(1:128,65:192);       % image fragment for display 
  XdB = scaledB( X ); XpdB = scaledB( Xp );
  YdB = scaledB( Y ); YpdB = scaledB( Yp );
  
  figure;
  subplot(231); imshow(x,    cmap); title('1. Image IN');
  subplot(232); imshow(XdB,  cmap); title('2. 2D DFT');
  subplot(233); imshow(XpdB, cmap); title('3. Centering');
  subplot(234); imshow(YpdB, cmap); title('4. Filtering');
  subplot(235); imshow(YdB,  cmap); title('5. De-centering');
  subplot(236); imshow(y1f,  cmap); title('6. Image OUT'); pause 

% 2D FILTERING using 2D Matlab convolution function conv2()
  L = 32; y2 = conv2(x, h(M/2+1-L:M/2+1+L, N/2+1-L:N/2+1+L),'same');
  
  figure;
  subplot(121); imshow(y1,cmap); title('Image after FREQ filter - cyclic conv');
  subplot(122); imshow(y2,cmap); title('Image after CONV filter - linear conv'); pause
