% lab17_ex_IQ_SpectrumDFT_Decode.m  
  
  clear all; close all;

  m=128; cm_plasma=plasma(m); cm_inferno=inferno(m); % color maps for gray printing
  cm = plasma;
  
% Read recorded IQ signal
% FileName = 'SDRSharp_DABRadio_229069kHz_IQ.wav';    demod=0; % DABRadio signal
  FileName = 'SDRSharp_FMRadio_101600kHz_IQ.wav';     demod=1; % FMRadio signal
% FileName = 'SDRSharp_VORAirplane_112849kHz_IQ.wav'; demod=2; % VOR airplane
% FileName = 'SDRWeb_Unknown_3671.9kHz.wav';          demod=3; % speech from WebSDR
  inf = audioinfo(FileName), pause           % what is "inside" 
  fs = inf.SampleRate;                       % sampling rate
  Ksec = 5;                                  % number of seconds of interest
  [x,fs] = audioread(FileName,[1,Ksec*fs]);  % read data 
  whos, pause                                % what we have in the memory now
  Nx = length(x),                            % signal length

% Parameters  
  Nfft = min(2^17,Nx); Nstft = 512;          % lengts of FFT and STFT

% Reconstruct the complex-value IQ data if necessary
  [N,M] = size(x);
  if(M==2) x = x(:,1) + j*x(:,2); else x = x(1:Nx,1) + j*zeros(Nx,1); end 
      nd = 1:2500;
      figure(1); plot(nd,real(x(nd)),'bo-',nd,imag(x(nd)),'r*--'); xlabel('n'); grid; 
      title('I(n) = (o) BLUE/solid    |    Q(n)= (*) RED/dashed'); pause

% Power Spectral Density (PSD) of the signal
  N = Nfft;                             % FFT length
  df = fs/N;                            % df - step in frequency
  f = df * (0 : 1 : N-1);               % frequency axis [ 0, fs ]
  fshift = df * (-N/2 : 1 : N/2-1);     % frequency axis [ -fs/2, fs/2 ]
  w = kaiser(N,10);                     % window function used
  X = fft( x(1:Nfft) .* w );            % DFT of windowed signal
  P = 2*X.*conj(X)/(fs*sum(w.^2));      % Power Spectral Density (dB/Hz)
  Pshift = fftshift( P );               % circularly shifted PSD
  
  % Short Time Fourier Transform (STFT) of the signal
  N = Nstft; df = fs/N; ff = df*(0:1:N-1);  ffshift = df*(-N/2:1:N/2-1);

      figure(2)
      subplot(211); plot(f,10*log10(abs(P))); xlabel('f (HZ)'); ylabel('(dB/Hz)')
      axis tight; grid; title('PSD for frequencies [0-fs)');
      subplot(212); spectrogram(x(1:Nfft),kaiser(Nstft,10),Nstft-Nstft/4,ff,fs);
      colormap(cm); pause

      figure(3)
      subplot(211); plot(fshift,10*log10(abs(Pshift))); xlabel('f (HZ)'); ylabel('(dB/Hz)')
      axis tight; grid; title('PSD for frequencies [-fs/2, fs/2)');
      subplot(212); spectrogram(x(1:Nfft),kaiser(Nstft,10),Nstft-Nstft/4,ffshift,fs);
      colormap(cm); pause
      subplot(111);
  
  if(demod==1) % Mono FM radio decoding
     bwSERV=200e+3; bwAUDIO=25e+3; w = kaiser(Nstft,10);                      % Params
     D1 = round(fs/bwSERV); D2 = round(bwSERV/bwAUDIO);           % Downsamplin ratios          %
     f0 = 0e+6; x = x .* exp(-j*2*pi*f0/fs*(0:Nx-1)'); % Station? shift in freq 0.575? 
     x = resample(x,1,D1);                             % Resample down to service freq
     x = real(x(2:end)).*imag(x(1:end-1))-real(x(1:end-1)).*imag(x(2:end)); % FM demod
     x = resample(x,1,D2);                               % Resample down to audio freq 
     soundsc(x,bwAUDIO);                                                   % Listening
   end

  if(demod==2) % VOR voice control signal decoding
    fc = 5.5353*10^4;            % carrier? your choice after spectrum inspection
    fam = 10000;                 % frequency width of AM modulation around fc
    f1 = fc-fam/2; f2 = fc+fam/2; df = 500;           % Band-Pass filter h design
    h = cfirpm(500,[-fs/2 (f1-df) (f1+df) (f2-df) (f2+df) fs/2]/(fs/2),@bandpass);
    x = conv(x,h,'same');                                  % Band-pass filtration
    x = sqrt( real(x).*real(x) + imag(x).*imag(x) );            % AM demodulation
    x = decimate(x,round(fs/fam));  %[U,D] = rat(fs/fam); x = resample( x, U, D );
    soundsc(x,fam);                                                   % Listening
  end    

  if(demod==3) % amateur HAM radio
     soundsc(x,fs);
  end   

  if(demod==11) % Mono FM radio decoding - LONG
     bwSERV=200e+3; bwAUDIO=25e+3; ppi = 2*pi; w = kaiser(Nstft,10);
     D1 = round(fs/bwSERV); D2 = round(bwSERV/bwAUDIO);
   % Rotate the spectrum - center the station you want to listen to 0 Hz 
     f0 = 0.e+6; % Hz: [0,0.575, 1.29]e+6 - your choice after spectrum inspection
     x = x .* exp(-j*ppi*f0/fs*(0:Nx-1)');
     spectrogram(x(1:2^16),w,Nstft-Nstft/4,ffshift,fs); title('Freq-DOWN'); pause
   % Resample the signal: low-pass filter around 0Hz and down-sample  
     x = decimate(x,D1);  % [U1,D1]=rat(fs/bwSERV,0.01); x = resample(x,U1,D1);
     spectrogram(x(1:2^16),w,Nstft-Nstft/4,1/D1*ffshift,bwSERV); title('Resample #1'); pause
   % Calculate instantaneous frequency: two methods
     dy = (x(2:end).*conj(x(1:end-1)));  clear x;                             % #1
     y = atan2( imag(dy), real(dy) );    clear dy;                            % #1
   % y = real(x(2:end)).*imag(x(1:end-1)) - real(x(1:end-1)).*imag(x(2:end)); % #2
     spectrogram(y(1:2^16),w,Nstft-Nstft/4,1/D1*ffshift,bwSERV); title('Demod'); pause
   % Resample the signal: low-pass filter around 0Hz and down-sampling 
     y = decimate(y,D2);  % [U2,D2]=rat(bwSERV/bwAUDIO,0.01); y = resample(y,U2,D2); 
     spectrogram(y(1:2^16),w,Nstft-Nstft/4,1/(D1*D2)*ffshift,bwAUDIO); title('Resample #2'); pause
   % Listening to mono FM radio signal
     soundsc(y,bwAUDIO);
     disp('You are listening to mono FM radio signal');
  end

    if(demod==21) % VOR voice control signal decoding - LONG
    fc=5.5353*10^4;    % carrier? your choice after spectrum inspection
    Nfft=Nx; df = fs/Nfft; f = df* (-Nfft/2 : 1 : Nfft/2-1);
  % Design of BandPass filter weights for extraction of an AM-modulated signal 
    fam = 5000;        % frequency width of AM modulation around fc
    f1 = fc-fam/2; f2 = fc+fam/2; df = 500; 
    h = cfirpm(500,[-fs/2 (f1-df) (f1+df) (f2-df) (f2+df) fs/2]/(fs/2),@bandpass);
  % BandPass filtering
    x = conv(x,h,'same'); Nx = length(x);
  % Spectrum of the filtered signal  
    X = fftshift( fft(x) );
    figure; plot(f,20*log10(abs(X))); xlabel('f [HZ]'); grid;
    title('Spectrum after BandPass filtration'); pause
  % AM demodulation of multicomponent signal: speech+30Hz+9.96kHz
    x = sqrt( real(x).*real(x) + imag(x).*imag(x) );  % AM
  % Down-sampling by resamplind  
    x = decimate(x,round(fs/fam)); %[U,D] = rat(fs/fam); x = resample( x, U, D );
  % Spectrum
    X = fftshift( fft(x) );
    Nfft = length(x); df = fam/Nfft; f = df*(-Nfft/2 : 1 : Nfft/2-1);
    figure; plot(f,20*log10(abs(X))); xlabel('f [HZ]'); grid;
    title('Spectrum of AM demodulated signal'); pause
  % Listening to voice broadcast control  
    soundsc(x,fam);
  end

  if(demod==22) % VOR airplane localization signal decoding
    M = 501; M2=(M-1)/2;         % filter length
    fc = 5.5353*10^4;            % carrier? your choice after spectrum inspection
    fam = 25000; dt=1/fam;       % frequency width of AM modulation around fc
    f1 = fc-fam/2; f2 = fc+fam/2;% Band-Pass filter h design
    h = cfirpm(M-1,[-fs/2 (f1-df) (f1+df) (f2-df) (f2+df) fs/2]/(fs/2),@bandpass);
    x = conv(x,h); x=x(M:end-M+1);                      % Band-pass filtration
    x = sqrt( real(x).*real(x) + imag(x).*imag(x) );            % AM demodulation
    x = decimate(x,round(fs/fam));  %[U,D] = rat(fs/fam); x = resample( x, U, D );
    x = x - mean(x);
    spectrogram(x,512,512-64,512,fam); title('AM demodulated'); pause
  % soundsc(x,fam);   % Listening
   
  % 9.96 kHz reference
    if(0) % only for test
       N=length(x); t=dt*(0:length(x)-1);
       x=sin(2*pi*30*t) + cos(2*pi*(9960*t-480/(2*pi*30)*cos(2*pi*30*t)));
       x=x';
    end    
    xc = x;

    href = fir1(M-1,[9000,11000]/(fam/2),'bandpass');
    x = conv(x,href); x=x(M2+1:end-M2);
    x = hilbert(x);
    x = unwrap(angle(x));
    x = x(3:end)-x(1:end-2);
    x = (1/(2*pi))*x/(2*dt);

    figure; t=dt*(0:length(x)-1); plot(t,x); title('finst(t)'); pause
    figure; Nf=length(x); n=1:Nf; f=fam/Nf*(n-1);
            plot(f,20*log10(abs(fft(x(n)))/length(x)));
            title('f inst'); pause
    
  % indx=find(x>12000); x(indx)=12000;
  % indx=find(x<8000);  x(indx)=8000;
    
    h30 = fir1(M-1,50/(fam/2),'low');
    x = conv(x,h30); x=x(M2+1:end-M2);
        figure; plot(t,x); title('LP finst(t)'); pause
    X = abs(fft(x(n)))/length(x);
        figure; plot(f,20*log10(abs(X))); pause
    K=5; [xmax,indx] = max(X(K+1:Nf/2));
    fmod = f(K+indx),
    pause

    x = x - mean(x);
    phi_ref = x/max(x); Nref=length(x);
        figure; plot(phi_ref); title('30Hz ref'); pause

  % 30 Hz azimuth  
    x = conv(xc,h30); x = x(M2+1:end-M2);
    x = x - mean(x);
    phi_azim = x(2:end-1)/max(x);  % 2,-1 due to finst computation

    N=length(phi_ref); n=1:N;
        figure; plot(phi_azim); title('30Hz azim'); pause
        figure; plot(n,phi_ref,'r-',n,phi_azim,'b-'); title('30Hz Both'); pause
    
    xx = hilbert(phi_azim).*conj(hilbert(phi_ref));
    phi_inst = angle( xx );
        figure; plot(phi_inst); title('phi inst'); grid; pause
        figure; phi_estim = angle( sum(xx)/N ), pause

  end    
