% SDR_IQ_Spectrum_Viewer.m
% Show spectrum of an IQ signal recorded by ADALM-PLUTO or RTL-SDR USB stick
% using the program SDR_IQ_Recorder.m

  clear all; close all;

% LOAD recorded IQ samples
  load('samples.mat'); whos    % IQ complex-value numbers stored vertically
  x = samples; clear samples;  % copy, free memory 
  fs = 30.72e6;                % one should remember sampling frequency value
  N = length(x),               % find number of samples

% DISPLAY them  
  figure;
  n = 1 : min(10000,N);        % indexes of samples selected for display 
  plot(n,real(x(n)),'b.-',n,imag(x(n)),'r.-'); xlabel('sample number');
  title('I = Real( x(n) ) (BLUE),   Q = Imag( x(n) ) (RED)'); grid;
  pause

% ###############################################
% Use MATLAB FUNCTIONS: Welch PSD and spectrogram 
  figure;

% Welch PSD
  subplot(211);
  Nfft = 2048; 
  pwelch( x(n), Nfft, Nfft/2, Nfft, fs,'centered');
  
% STFT - short-time Fourier transform (time-frequency signal representation)
  subplot(212);
  Nstft = 2048; ff = fs/Nstft * (-Nstft/2 : Nstft/2-1); 
  spectrogram( x(n), Nstft, Nstft-Nstft/4, ff, fs ); title('STFT( x(n) )'); pause 

% ##########################################################################
% CALCULATE YOURSELF DFT and PSD for the signal fragment x( n1 : n1+Nfft-1 )

  figure
  
% Parameters  
  Nfft = min( 2^16, N );                 % your choice
  n1 = round(N/4); n = n1 : n1+Nfft-1;   % your choice 
  df = fs/Nfft;                          % FFT resolution
  f = df*(-Nfft/2:1:Nfft/2-1) / 1e+6;    % frequencies expressed in MHz

% DFT
  X = 20*log10( abs( fft(x(n)/Nfft )));  % FFT spectrum magnitude in decibels 
  X = fftshift(X);                       % set DC component in the middle
  subplot(211); plot(f,X); xlabel('f [MHz]'); title('|X(f)| [dB]'); grid;

% PSD
  w = hamming(Nfft)'; sw = sum(w.^2);    % window function used
  X = fft( x(n) .* w );                  % fFT of windowed signal
  P = 10*log10( 2*X.*conj(X)/(fs*sw) );  % Power Spectral Density (dB/Hz)
  P = fftshift( P );                     % set DC component in the middle
  subplot(212); plot(f,P); xlabel('f (HZ)'); title('P(f) [dB/Hz]'); grid; pause
  