% lab17_ex_Service_UpDown_long.m
  clear all; close all;

% PARAMETERS
  isignal = 1;           % signal type: 1=FM, 2=OFDM, 3=AM-real, 4=AM-complex
  Nx = 2000;             % number of signal samples
  fs = 2000;             % sampling frequency [Hz]
  fc = 400;              % carrier frequency [Hz]
  dfc = 0;               % carrier frequency shift [Hz]: 0, 100
  phase = 0;             % phase shift; 0, pi/4
  
  dt = 1/fs; t = dt*(0:Nx-1);                  % time
  f0 = 1/(Nx*dt); f = f0*(-Nx/2:1:Nx/2-1);     % frequency
  kk = find( f>=-fc & f<=fc );                 %
  N1st = 1; Nlast = 400; nn = N1st : Nlast;    % for signal plot
  Nstft = 128; Nshift = 4;                     % for STFT plot
  f0stft = 1/(Nstft*dt); fstft = f0stft*(-Nstft/2:1:Nstft/2-1); % for STFT plot

% SIGNALS
  if(isignal==1) % FM 
     fm = 2; df = 50;       % modulation frequency and modulation depth
     xm = sin(2*pi*fm*t);   % modulating signal, can be speech
     x = exp( j *( 2*pi*0*t + 2*pi*df/fs*cumsum(xm) ) );
  end  
  if(isignal==2) % Complex-value OFDM
     fk = -100 : 50 : 100;  % frequencies of signal components
     Nk = length(fk);       % number of components
     Ak = 1 : 1 : Nk;       % their amplitudes
     pk = pi/Nk *(0:Nk-1);  % their phase shifts
     x = zeros(1,Nx);       % initial values of signal samples
     for k = 1:Nk
         x = x + Ak(k)*exp( j * (2*pi*fk(k)*t + pk(k)) );
     end
  end
  if(isignal==3) % AM-real DSB-CS 
     fm = [ 50 100 ];       % frequencies of modulating real-value sinusoids
     x = 5*sin(2*pi*fm(1)*t) + 10*sin(2*pi*fm(2)*t);
  end  
  if(isignal==4) % AM-complex SSB-CS          
     fm = [ 50 100 ];       % frequencies of modulating complex-value harmonics
     x = 2.5*exp(j*2*pi*fm(1)*t) + 5*exp(j*2*pi*fm(2)*t);
  end  

% SPECTRUM
  X = fftshift( fft(x)/Nx );

% FIGURES - ADD FIGURES after each operation (UP, REAL, DOWN, FILTER) performed below!
  figure;
  subplot(211); stem(t(nn),real(x(nn)),'filled'); grid; title('Real( x(t) )');
  subplot(212); stem(t(nn),imag(x(nn)),'filled'); grid; title('Imag( x(t) )');
  xlabel('t [s]'); pause
  figure;
  subplot(311); stem(f, abs(X),'filled'); grid; title('Abs( X(f) )'); 
  subplot(312); stem(f,real(X),'filled'); grid; title('Real( X(f) )');
  subplot(313); stem(f,imag(X),'filled'); grid; title('Imag( X(f) )');
  xlabel('f [Hz]'); pause
  figure;
  subplot(111); spectrogram(x,Nstft,Nstft-Nshift,fstft,fs); title('|STFT(t,f)| of x(t)');
  axis tight; ylabel('Time (ms)'); xlabel('Frequency (kHz)'); pause
 
% Frequency UP
  c = cos( 2*pi*fc*t );
  s = sin( 2*pi*fc*t );
  cs = c + j*s;                   % complex-value carrier
  xUp = x .* cs;                  % signal - modulation of carrier amplitude by x(n)
  XUp = fftshift( fft(xUp)/Nx );  % spectrum
  
% Take REAL part
% xUpReal = real(x).*c  - imag(x).*s;    % # quadrature modulator
  xUpReal = real( xUp );                 % signal
  XUpReal = fftshift( fft(xUpReal)/Nx ); % spectrum

% Frequency DOWN
  c = cos( 2*pi*(fc+dfc)*t + phase);     % @ carrier
  s = sin( 2*pi*(fc+dfc)*t + phase);     % @ recovery?
  xDownCos =  2*xUpReal.*c;              % # quadrature
  xDownSin = -2*xUpReal.*s;              % # demodulator
  xDown = xDownCos + j*xDownSin;         % signal
  XDown = fftshift( fft(xDown)/Nx );     % spectrum
     
% LowPass FILTER
  if(1) % Ideal 0/1 filtering in frequency domain
     XDownFilt = zeros(1,Nx);
     XDownFilt(kk) = XDown(kk);                     % taken DFT bins
     xDownFilt = ifft( fftshift( XDownFilt ) )*Nx;  % signal #1
  else  % Practical filtering - weighted averaging
     h = fir1(51, fc/(fs/2));                       % filter weights
     xDownFilt = filter(h, 1, xDown);               % signal #2
  end
  XDownFilt = fftshift( fft(xDownFilt)/Nx );        % spectrum

% ERROR of frequency UP & DOWN  
  ERROR_SPECTRUM_UP_DOWN_FILTERED = max( abs( X - XDownFilt ) ),
  ERROR_SIGNAL_UP_DOWN_FILTERED = max( abs( x - xDownFilt ) ), pause
  
% Correction
  xDownFiltCorrect = xDownFilt .* exp( j*(2*pi*dfc*t+phase) );
  XDownFiltCorrect = fftshift( fft(xDownFiltCorrect)/Nx );
  ERROR_SPECTRUM_UP_DOWN_FILTERED_CORRECTED = max( abs( X - XDownFiltCorrect ) ),
  ERROR_SIGNAL_UP_DOWN_FILTERED_CORRECTED = max( abs( x - xDownFiltCorrect ) ),
