
% FM: Spectrum of SFM

clear all; close all;

% Parameters
  fs = 2000;            % sampling frequency 
  fc = 1000;             % carrier frequency
  df = 10; L=8;              % FM modulation depth: {df=10,L=8},{df=0.001,L=2}
  fm = 100;              % mofulating frequency
  delta = df/fm,        % modulation index

% Modulating signal
  N = fs;                           % number of samples
  dt=1/fs;  t = (1/fs)*(0:N-1)';    % time axis
  f0=fs/N;  f = f0*(fs/N)*(0:N-1);  % frequency axis
  x = -sin(2*pi*fm*t);              % signal
  figure; plot(f,20*log10(abs(fft(x)/(N/2)))); grid;
  xlabel('f (Hz)'); ylabel('(dB)'); title('|X(f)|'); pause

% FM modulation
  y = exp(j*2*pi*(fc*t + df*cumsum(x)*dt) ); K=1;            % complex carrier
% y = cos(2*pi*(fc*t + df*cumsum(x)*dt) ); K=2;              % real carrier
  ym = fmmod(x,fc,fs,df); err = max(abs(ym-real(y))), pause  % Matlab function
  figure; plot(f,20*log10(abs(fft(y)/(N/K)))); grid;
  xlabel('f (Hz)'); ylabel('(dB)'); title('|Y(f)|'); pause

% Shape of Bessel functions Jn(delta) of the first kind
  d=0:0.05:25;
  J0 = besselj(0,d); J1 = besselj(1,d); J2 = besselj(2,d);
     figure; plot(d,J0,'k',d,J1,'b',d,J2,'r');
     xlabel('delta'); title('J(delta)'); grid; pause

% Verification of experimental and theoretical signal spectrum   
  n = -L:L;              % Bessel function order; 
  Jn = besselj(n,delta);   % function values
  fb = fc + n*fm;          % frequencies of expected spectral components
  figure; plot(f,20*log10(abs(fft(y)/(N/K))),'bx-',fb,20*log10(abs(Jn)),'ro' ); grid;
  xlabel('f (Hz)'); ylabel('(dB)'); title('|Y(f)|'); pause
