% lab18_ex_fm_diff.m
% Example of FM modulation and demodulation using BandPass differentiation filter
  clear all; close all;

% Modulating signal m(t)
  [x,fs] = audioread('GOODBYE.WAV');  % read speech from file
  x = x(:,1)';                        % take only one channel, 1-st or 2-nd
  x = resample(x,4,1); fs = 4*fs;     % up-sample 4 times
  Nx = length(x);                     % number of signal samples
  dt=1/fs; t=dt*(0:Nx-1);             % time
  figure; spectrogram(x,256,192,512,fs,'yaxis'); title('STFT of x(t)'); pause

% FM modulation
  fc = fs/8;                          % carrier frequency
  BW = 1.75*fc;  fmax=4000;           % available bandwidth, max signal frequency
  df = (BW/(2*fmax)-1)*fmax,          % calculated freq modulation depth or 1000
  y = cos( 2*pi*(fc*t + df*cumsum(x)*dt) );        % carrier frequency modulation 
  figure; spectrogram(y,256,192,512,fs,'yaxis'); title('STFT of y(t)'); pause

% BandPass (BP), differentiation (D) and LowPass (LP) filter design
  M = 100; L=2*M+1; BW2 = BW/2;
  hBP = firls( L-1,[ 0 fc-BW2 fc-0.95*BW2 fc+0.95*BW2 fc+BW2 fs/2 ]/(fs/2),[0 0 1 1 0 0 ], [1 1 1]);
  m=-M:M; hD  = cos(pi*m)./m; hD(M+1)=0; w=kaiser(L,10).'; hD = hD.*w; 
  hLP = fir1( L-1, fc/(fs/2), kaiser(L,7) );
  if(1) % figures of BandPass filter
    figure; stem(hBP); title('h_{BP}(n)'); pause   % filter weights
    f=0:10:fs/2; HBP=freqz(hBP,1,f,fs);
    figure;
    subplot(211); plot(f,20*log10(abs(HBP))); title('abs(H_{BP}(f))');
    subplot(212); plot(f,unwrap(angle(HBP))); title('angle(H_{BP}(f))'); pause
   end
  if(1) % figures of Differentiation filter
    figure; stem(hD); title('h_D(n)'); pause   % filter weights
    f=0:10:fs/2; HD=freqz(hD,1,f,fs);
    figure;
    subplot(211); plot(f,20*log10(abs(HD))); title('abs(H_D(f))');
    subplot(212); plot(f,unwrap(angle(HD))); title('angle(H_D(f))'); pause
  end
  
  xR = conv(y,hBP); xR = xR(M+1:end-M);     % BP filtering
  figure; spectrogram(xR,256,192,512,fs,'yaxis'); title('STFT after BP filter'); pause
  xR = conv(y,hD); xR = xR(M+1:end-M);      % DIFF filtering
  figure; spectrogram(xR,256,192,512,fs,'yaxis'); title('STFT after DIFF filter'); pause
  xR = xR.^2;                               % .^2 (power of 2)
  figure; spectrogram(xR,256,192,512,fs,'yaxis'); title('STFT after ()^2'); pause
  xR = conv( xR, hLP ); xR = xR(M+1:end-M); % low-pass filtering
  figure; spectrogram(xR,256,192,512,fs,'yaxis'); title('STFT after LP filter'); pause
  xR = sqrt( xR );                          % square root
  figure; spectrogram(xR,256,192,512,fs,'yaxis'); title('STFT after sqrt'); pause
  
  n = L:length(x)-L; x=x(n); xR=xR(n);        % remowing transient parts
  xR = xR/(2*pi); xR = (xR - fc/fs)/(df/fs);  % instantaneous frequency minus fc
  xR = xR-mean(xR);                           % mean value subtraction
  figure; spectrogram(xR,256,192,512,fs,'yaxis'); title('STFT after scaling'); pause
  figure; plot(n,x,'r-',n,xR,'b-'); pause     % comparing input and output signals
  soundsc(xR,fs);                             % playing demodulated sound
  