% lab18_ex_nanosat.m - first, the program lab17_ex_IQ_DFT.m is repeated
% DSP NanoSat Receiver - frequency demodulation
  close all; clear all;
  
  m=128; cm_plasma=plasma(m); cm_inferno=inferno(m); % color maps for gray printing
  cm = plasma;

% Reading RF data   
  filename = 'SDRSharp_NanoSat_146000kHz_IQ.wav';
  [x,fs] = audioread( filename );
  
% Creation of a complex-value IQ signal
  x = x(:,1) - j*x(:,2);                   % 1-st column to real(), 2-nd to imag()
  x = x - mean(x);                         % mean value subtraction
  Nx = length( x ); nc = round(Nx/2);      % length, central sample

% Power Spectral Density (PSD) of the signal
  Nfft = 2^16;                             % FFT length
  df = fs/Nfft;                            % df - step in frequency
  f = df * (0 : 1 : Nfft-1);               % frequency axis [ 0, fs ]
  fshift = df * (-Nfft/2 : 1 : Nfft/2-1);  % frequency axis [ -fs/2, fs/2 ]
  w = kaiser(Nfft,10);                     % window function used
  X = fft( x(nc-Nfft/2:nc+Nfft/2-1).*w );  % FFT of windowed signal
  P = 2*X.*conj(X)/(fs*sum(w.^2));         % Power Spectral Density (dB/Hz)
  Pshift = fftshift( P );                  % circularly shifted PSD
  
% Parameters for Short Time Fourier Transform (STFT) of the signal
  Nstft=512; df = fs/Nstft; ff = df*(0:1:Nstft-1);  ffshift = df*(-Nstft/2:1:Nstft/2-1);

% We see one NanoSat massage
     figure
     subplot(211); plot(fshift,10*log10(abs(Pshift))); xlabel('f (HZ)'); ylabel('(dB/Hz)')
     axis tight; grid; title('PSD for frequencies [-fs/2, fs/2)');
     subplot(212); spectrogram(x,kaiser(Nstft,10),Nstft-Nstft/8,ffshift,fs);
     colormap(cm); title('STFT for frequencies [-fs/2, fs/2)'); pause

% ########################################################################   
% ADDED NanoSat DEMODULATION #############################################
% ########################################################################   
     
% Shifting in frequency a chosen NanoSat service
  f0 = 1.98e+4;
  x = x .* exp(-j*2*pi*f0/fs*(0:Nx-1)');
  
    figure
    spectrogram(x,kaiser(Nstft,10),Nstft-Nstft/8,ffshift,fs);
    title('STFT after frequency shift to 0 Hz [-fs/2, fs/2)');
    colormap(cm); pause
   
% Downsampling to NanoSat audio sampling frequency  
  fAudio = 48e+3; Down= round(fs/fAudio);
  h = fir1(500,12500/fs,'low');            % filter design
  x = filter(h,1,x);                       % filtration
  x = x(1 : Down : end);                   % decimation
  
     figure
     spectrogram(x,kaiser(Nstft,10),Nstft-Nstft/8,ffshift/Down,fAudio);
     title('STFT after downsampling [-fAudio/2, fAudio/2)'); colormap(cm); pause

% Frequency demodulation
  dt = 1/fAudio;
  x = x(2:end).*conj( x(1:end-1) );  % complex signal with phase/angle difference
  x = angle(x) / (2*pi*dt);          % instantaneous frequency calculation 
  disp('You are listening to FM demodulated NanoSat signal'); pause
  soundsc(x,fAudio);                 % listening
  
     figure; plot(x); grid; 
     title('Digital signal in time domain - all samples'); 
     xlabel('sample number'); ylabel('Signal values'); pause
     
     figure;
     df = fAudio/Nstft; ff = 0:df:(Nstft/2)*df; 
     spectrogram(x,kaiser(Nstft,10),Nstft-Nstft/8,ff,fAudio);
     title('STFT after FM demodulation [0, fAudio/2)'); colormap(cm); pause
