function hpsf = firrcos( Npsf, fcut, RDF, fs, RDFselection, shape);
% Rasied cosine (RC) and squre-root raised cosine (SRRC) FIR pulse-shaping filter design
%
% INPUT:
% Npsf - filter length
% fcut - cutoff frequency (Hz)
% R    - rolloff factor in the range [0,1] for RDFselection='rolloff'
% DF   - fiter transition bandwidth (Hz)   for RDFselection='' (not implemented)
% RDFselection - 'rolloff' or [] (empty)
% fs   - sampling frequency (Hz)
% shape - 'normal' (RC, raised-cosine) or 'sqrt' (SQRT, square-root raised cosine)
% 
% Exemplary call with input identical to rcosdesign(BETA,SPAN,SPS,SHAPE):
% BETA=R, SPAN=number of symbols per PSF, SPS=Nsps=number of samples per symbol
if(0) % TEST ONLY
   BETA = 0.35; SPAN = 8; SPS = 6; shape='normal'; % input values of rcosdesign()
   Npsf = SPAN*SPS;                                % length of the PSF filter
   fcut = 1/(2*SPS); R=BETA; fs=1;                 %         
   RDFselection='rolloff';                         % obligatory
end

if( ~strcmp( RDFselection, 'rolloff' )  &&  ~strcmp( RDFselection, 'rollof') )
    disp ('XXX'); % The DF option is not implemented. Use rolloff R instead!');
    return
else
    R = RDF;
end

Nsps = fs/(2*fcut);         % SPS - number of samples per symbol, interpolation order
n = 0 : Npsf; Nc = Npsf/2;  % indexes of filter samples, index of the central sample
if( isequal( shape, 'normal') ) % definition of raised cosine
    hpsf = sin(pi*(n-Nc)/Nsps) ./ (pi*(n-Nc)/Nsps) .* ...
           cos(pi*R*(n-Nc)/Nsps) ./ (1-(2*R*(n-Nc)/Nsps).^2);
    hpsf(Nc+1) = 1;   
    n0 = find( abs(1-(2*R*(n-Nc)/Nsps).^2 ) < sqrt(eps) );
    hpsf(n0) = R/2 * sin(pi/(2*R));
elseif( isequal( shape, 'sqrt') ) % definition of sqrt of raised cosine
    n0 = find( abs(1-(4*R*(n-Nc)/Nsps).^2) < 5*eps );
    if( prod(size(n0)) > 0 ) disp('Div by 0. Change parameters'); end
    hpsf = 4*R/pi * ( cos(pi*(1+R)*(n-Nc)/Nsps)+(Nsps./(4*R*(n-Nc)) ) ...
           .* sin(pi*(1-R)*(n-Nc)/Nsps) ) ./ (1-(4*R*(n-Nc)/Nsps).^2);
    hpsf(Nc+1) = (1+R*(4/pi-1));   
end
hpsf = hpsf/Nsps; % normalization

if(0) % Comparison with Matlab, change name of this function
   fs = 1; fcut = 1/(2*Nsps)
   hpsf0 = firrcos( Npsf, fcut, R, fs,'rolloff',shape); % MATLAB function
   plot(1:length(hpsf0),hpsf0,'ro-',1:length(hpsf),hpsf,'bx-');
   error = max(abs(hpsf-hpsf0)), pause
end
