% fmradio_encoder_stereo_RDS.m
% RADIO FM ENCODER EXAMPLE
  clear all; close all;

  ifigs = 1;       % 0/1 = OFF/ON displaying figures
  fmradio_params   % read parameters from this file

% Read two monophonic audio signals - audioread or wavread
  [x1, fx1 ] = audioread('GOODBYE.WAV'); x1=x1.';  % plot(x1);
  [x2, fx2 ] = audioread('DANKE.WAV');   x2=x2.';  % soundsc(x1,fx1);

% For test only
% N = 200000; t = dt*(0:N-1);  fx1=48000;
% x1 = 0.5*cos( 2*pi*(5555*t - 3333/(2*pi*5)*cos(2*pi*5*t)) - pi/7);
% x2 = x1; fx2 = fx1;
   
% ERROR! Gives spikes in signal spectrum!
% Pre-emphasis, flat freq response to 2.1 kHz, than increasing 20 dB per decade
% x1 = filter(b_pre,a_pre,x1); x2 = filter(b_pre,a_pre,x2);

% Up-sampling to FM radio service frequency fs
  [N1,D1] = rat(fs/fx1,1e-6); x1 = resample(x1,N1,D1);      % left  channel
  [N2,D2] = rat(fs/fx2,1e-6); x2 = resample(x2,N2,D2);      % right channel
  Nx = min( length(x1),length(x2) ); n=1:Nx; x1=x1(n); x2=x2(n);
  x2 = zeros(1,Nx); % for test

% RDS BITS #########################################
% Generation of dummy RDS bits for encoding
  Nrds = ceil(Nx/(fs/fsymb)); rds = [];
  for k=1:ceil(Nrds/10), rds = [ rds 1 1 1 1 0 0 0 1 1 0 ];
  end
% rds = round(rand(1,Nrds)); % for test
% Differential encoding of RDS bits
  Nrds = length(rds); r = 0;
  for k=2:Nrds, r(k)=abs(rds(k)-r(k-1));
  end
  Nr = ceil(fs/fsymb *length(r));
% Generation of bi-phase impulses: -1 -> +1, +1 -> -1, 
  bip = [-1 1; 1 -1 ]; brds = bip( r+1,: ); brds=brds'; brds=brds(:);
% Zero insertion between -1/+1 impulses
  uprds = zeros(1,Nr); t = dt*(0:Nr-1);
  clk = abs(diff(sign(sin(2*pi/Tsymb*t))))/2; clk = [clk 0];
  indx=1;
  for n=1:Nr
      if(clk(n)==1) uprds(n)=brds(indx); indx=indx+1;
      end
  end
% Signal smoothing using pulse shaping filter (PSF)   
  prds = conv( uprds, hpsf); prds = prds(1:Nx);
  clear t clk r brds uprds;
% RDS BITS #########################################

% Generation of multiplex (MPX) signal of FM radio: L+R, pilot, L-R, RDS
  n=0:Nx-1; alpha = 2*pi*fpilot/fs*n;
  x = 0.9*(x1+x2)+0.1*cos(alpha)+0.9*(x1-x2).*cos(2*alpha)+0.05*prds.*cos(3*alpha);
  A = max(x); x=x/A; clear prds alpha;

  if(ifigs==1)
     m=1:2000; N = length(x); f = fs/N*(0:N-1);
     figure, plot(x(m)); title('MPX signal'); xlabel('n'); grid; pause;
     figure; plot(f,20*log10(abs(fft(x)))); grid; title('FFT of MPX signal'); pause
     figure, spectrogram(x, 512, 512-64, 512, fs); title('STFT of MPX signal'); pause
  end

% Final frequency modulation of the complex carrier in the base-band
  BW = 160000;                % overall FM radio bandwidth > 2*fmax=2*60000 Hz
  fmax = 60000;               % maximum modulating frequency
  df = (BW/(2*fmax)-1)*fmax,  % from Carson's bandwidth rule
  beta = df/fmax,             % modulation index
  fc = 0;                     % carrier frequency of FM radio in the baseband
  x = exp( j*2*pi*( fc/fs*n + df*cumsum(x)/fs ) ); x=x.';  % FM modulation
  I = real(x); Q = imag(x);                                % I, Q components
  audiowrite('FMRadio_IQ_250kHz.wav',[I Q],fs);            % store result
% audiowrite('FMRadio_IQ_250kHz_Calibrate.wav',[I Q],fs);  % store result
  
  if(ifigs==1)
     m=1:5000; N = length(x); f = fs/N*(0:N-1);
     figure, plot(m,I(m),m,Q(m)); title('Signal Re/Im'); xlabel('n'); grid; pause;
     figure; plot(f,20*log10(abs(fft(x)))); grid; title('Signal FFT'); pause
     figure, spectrogram(x, 512, 512-64, 512, fs); title('Signal STFT'); pause
  end
