% lab19_ex_am.m
% Example of AM modulation and demodulation
clear all; close all;
  
sig = 1;     % signal type: 1=speech, 2=cos, 3=cos+cos, 4=SFM
mod = 1;     % AM type: 1=DSB-LC, 2=DSB-SC, 3=SSB-U=USB, 4=SSB-L=LSB, 5=DSB-SC-CX
demod = 1;   % 1 = Hilbert transform demodulator, 2 = quadrature demodulator
nosynch = 0; % 0/1 carrier synchronization in frequency down-conversion 
disturb = 0; % 0/1 disturbance - second AM service using frequency (2*fc)
noise = 0;   % 0/1 presence of noise
fc = 10000;  % AM carrier frequency 
dA = 0.5;    % AM modulation depth for DSB-LC
Nwin = 256; Nover = Nwin-32; Nfft = 2*Nwin;  % for STFT plot
  
% Modulating signal x(t)
[x,fss] = audioread('speech11025.wav'); % read mono audio from file, fs=11025 Hz
K=5; x = resample(x,K,1); fs=K*fss;     % upsample K-times for frequecy-UP
N = length(x);                          % number of signal samples
dt = 1/fs; t = dt*(0:N-1)';             % time
df=fs/Nfft; f=df*(-Nfft/2:Nfft/2-1);    % frequencies for STFT display
                                        % alternative signals for tests
if(sig==2) x = cos(2*pi*2000*t); end                                 % 1x cos  
if(sig==3) x = cos(2*pi*2000*t) + cos(2*pi*3000*t); end              % 2x cos
if(sig==4) x = cos(2*pi*(2000*t + 1000/(2*pi*5)*sin(2*pi*5*t))); end % SFM
   n = 1:250; 
   figure; plot(t(n),x(n),'b-'); xlabel('t [s]'); grid; title('x(t)'); pause
   figure; spectrogram(x,Nwin,Nover,f,fs,'yaxis'); title('STFT x(t)'); pause

% Create base-band signal for AM modulation of the carrier: x(t) --> a(t)
if(mod==1) a = (1+dA*x);                     end   % DSB-LC
if(mod==2) a = x;                            end   % DSB-SC
if(mod==3) a = x + j*imag(hilbert(x));       end   % SSB-R = USB
if(mod==4) a = x - j*imag(hilbert(x));       end   % SSB-L = LSB
if(mod==5) x = x + j*sin(2*pi*4000*t); a=x;  end   % DSB-SC-CMPLX
   figure; spectrogram(a,Nwin,Nover,f,fs,'yaxis'); title('STFT a(t)'); pause
  
% Carrier AM modulation - frequency-up conversion: y(t) = a(t)*c(t)
c = exp( j*2*pi*fc*t );
y = real(a).*real(c) - imag(a).*imag(c);  % y = real( a .* c );  % the same
   figure; plot(t(n),y(n)); xlabel('t [s]'); grid; title('y(t)=a(t)c(t)'); pause
   figure; spectrogram(y,Nwin,Nover,f,fs,'yaxis');
   title('STFT UP y(t)=a(t)c(t)'); pause
   % Possible spectrum observation of the y.^2 signal
     if(1)
       figure; spectrogram(y.^2,Nwin,Nover,f,fs,'yaxis');
       title('STFT UP y(t)^2'); pause
     end

% Possible service using frequency (2*fc) (optional)
if(disturb == 1)
  y = y + real(a).*cos(2*pi*(2*fc)*t) - imag(a).*sin(2*pi*(2*fc)*t);
    figure; spectrogram(y,Nwin,Nover,f,fs,'yaxis');
    title('STFT UP - with second service using freq (2*fc)'); pause
end

% Possible lack of synchronization between frequency up-shifter and down-shifter
if(nosynch == 0)
  c = exp( j*(2*pi*(fc+100)*t + pi/4) ); % carrier used for freq down-conversion
end

% Additive noise
if(noise == 1) y  = y + 0.025*randn(N,1); end

% Carrier AM demodulation
if(demod==1)   
   yH = hilbert( y );               % Hilbert filter - analytic signal
      figure; spectrogram(yH,Nwin,Nover,f,fs,'yaxis'); title('STFT yH(t)'); pause
   a1 = yH .* conj( c );            % frequency-down conversion to base-band
      figure; spectrogram(a1,Nwin,Nover,f,fs,'yaxis'); title('STFT DOWN yH(t)'); pause
end
if(demod==2)
   a1 = 2*y.*real(c) - 2*j*y.*imag(c);    % quadrature demodulator
      figure; spectrogram(a1,Nwin,Nover,f,fs,'yaxis'); title('STFT a1(t)'); pause
end
M = 100; h = fir1(2*M,(fss/2)/(fs/2),kaiser(2*M+1,12)); % LP filter design
a2 = conv(a1,h); a2=a2(M+1:end-M);                      % filtering                    
   figure; spectrogram(a2,Nwin,Nover,f,fs,'yaxis'); 
   title('STFT a2(t) filtered'); pause
   figure; plot(t,real(a2),'b-',t,imag(a2),'r-'); xlabel('t [s]'); grid;
   title('a(t): real (BLUE) & imag (RED)'); pause

  % Recovering x(t) from a(t) 
if(mod==1)                   xdem = (abs(a2)-1)/dA;   end % DSB-LC
if(mod==2 | mod==3 | mod==4) xdem = real(a2);         end % DSB-SC, SSB-R, SSB-L
if(mod==5)                   xdem = a2;               end % DSB-SC-CMPLX
n = 500:N-500+1; t=t(n); x=x(n); xdem=xdem(n);            % this line optionally
   figure; plot(t,real(xdem),'b-',t,imag(xdem),'r-'); xlabel('t [s]'); grid;
   title('xdem(t): real (BLUE) & imag (RED)'); pause

% ERROR of demodulation 
ERROR_Demod_SIGNAL   = max( abs( x - xdem ) ), pause
ERROR_Demod_SPECTRUM = max( abs( fft(x) - fft(xdem) ) )/N,
   figure;
   subplot(211); plot(t,real(x),'r-',t,real(xdem),'b-'); 
   title('REAL: x(t) & xdem(t)'); xlabel('t [s]'); grid;
   subplot(212); plot(t,real(x-xdem),'b-'); xlabel('t [s]');
   title('ERR signal'); xlabel('t [s]'); grid;
   disp('Playing demod REAL'); soundsc(real(xdem),fs); pause
   if(mod==5)
      figure;
      subplot(211); plot(t,imag(x),'r-',t,imag(xdem),'b-'); 
      title('IMAG: x(t) & xdem(t)'); xlabel('t [s]'); grid;
      subplot(212); plot(t,imag(x-xdem),'b-');
      title('ERR signal');  xlabel('t [s]'); grid; pause
      disp('Playing demod IMAG'); soundsc(imag(xdem),fs); pause
   end
