% lab19_ex_Costas.m - digital Costas loop
clear all; close all;

% Parameters
isignal = 1;                  % 1=const, 2=fm, 3=speech, 4=bpsk, 5=qpsk
fs = 240000; dt=1/fs;         % sampling frequency, sampling period
fpilot = 19000;               % FM radio pilot frequency
fc = 3*fpilot; dfreq = 100;   % Costas loop: carrier frequency, frequency locking

% Choice of modulating signal
if(isignal==1)                   % const value
    N=100000; x = 5*ones(1,N);
elseif(isignal==2)               % FM signal
   N = 100000; t = dt*(0:N-1); 
   x = cos( 2*pi*(5000*t - 4000/(2*pi*5)*cos(2*pi*5*t)) - pi/7);
elseif(isignal==3)               % speech
   [x, fx ] = audioread('DANKE.WAV'); x=x.'; x=[x x]; x = interp(x,round(fs/fx));
elseif(isignal==4 | isignal==5)  % 
   frds = fpilot/16; Nrds = round( fs / frds );
   if(isignal==4)    % BPSK - two symbols sin() and -sin()
      Nsymb = 2;
      xsymb(1,:) =  sin(2*pi*(0:Nrds-1)/Nrds);             % bit 1
      xsymb(2,:) = -sin(2*pi*(0:Nrds-1)/Nrds);             % bit 0
   else              % QPSK - four symbols cos(Om*n+phi),
      Nsymb = 4;     % phi = pi/4, 3*pi/4, 5*pi/4, 7*pi/4 
      xsymb(1,:) = cos(2*pi*(0:Nrds-1)/Nrds +   pi/4);     % bits 00
      xsymb(2,:) = cos(2*pi*(0:Nrds-1)/Nrds + 3*pi/4);     % bits 01
      xsymb(3,:) = cos(2*pi*(0:Nrds-1)/Nrds + 5*pi/4);     % bits 10
      xsymb(4,:) = cos(2*pi*(0:Nrds-1)/Nrds + 7*pi/4);     % bits 11
   end
   x = [];                                           % DIGITAL SIGNAL SYNTHESIS:
   for k = 1:100                                     % building a sequence 
       x = [ x xsymb( ceil(Nsymb*(rand(1,1))),: ) ]; % of signals corresponding
   end                                               % to symbols (one-by-one)
   figure; plot(x); title('x(t) - modulating signal without PSF' ); pause                                    % 
   hpsf = firrcos( 6*Nrds, frds, 0.35, fs,'rollof','sqrt'); % PSF filter design
   x = filter( hpsf, 1, x );         % low-pass filtering - bandwidth reduction                          
end
N = length(x); n=0:N-1; t = n*dt;    % sample indexes, sampling moments
figure; plot(x); title('x(t) - modulating signal');  % pause

disp('Wait, wait, wait ...');

% Modulation AM-DSB-SC - frequency-up conversion
ph = 2*pi*(fc+25)*t + pi/4; % carrier errors: freq 25 Hz, phase pi/4 rad
c  = cos( ph );             % carrier samples
y  = x .* c;                % modulated signal, input to the Costas loop

% Disturbances
% y  = y + 0.01*randn(1,N);                                 % additive noise
% D=75; n=D+1:N; y =[y(n) y(1:D)]; x=[x(n) x(1:D)]; c=[c(n) c(1:D)]; % delay

% Calculation of adaptation constants
damp = sqrt(2)/2;                                       % standard damping
omeg = (dfreq/fs) / (damp+1/(4*damp));                  % constant  
mi1   = (4*damp*omeg) / (1 + 2*damp*omeg + omeg*omeg);  % adapt speed const #1
mi2   = (4*omeg*omeg) / (1 + 2*damp*omeg + omeg*omeg);  % adapt speed const #2

% Costas loop - joint frequency-down conversion and demodulation
y = hilbert( y );                             % real to analytic signal
%y = y .* exp(-j*(2*pi*(fc)*t+pi/3)); fc = 0; % test base-band adjustment only
omega = zeros(1,N+1); omega(1) = 2*pi*fc/fs;
theta = zeros(1,N+1);
for n = 1 : N                                           % Costas adaptation loop
    bb(n) = y(n) * exp(-j*theta(n));                    % base-band signal
    delta(n) = real( bb(n) ) .* imag( bb(n) ) / (bb(n).*conj(bb(n))); % error
    theta(n+1) = theta(n) + omega(n) + mi1*delta(n);    % 1-st update
    omega(n+1) = omega(n) + mi2*delta(n);               % 2-nd update
end
cest = cos(  theta(1:N) );      % recovered carrier
xest = real( bb );              % demodulated signal

% Figures
n=1:N;
figure; subplot(211); plot(real(bb)); title('real( bb(n) )');
        subplot(212); plot(imag(bb)); title('imag( bb(n) )');            % pause
figure; plot(n,theta(n),'r-');            title('theta(n)');       grid; % pause
figure; plot(n,ph(n)-theta(n),'r-');      title('ph(n)-theta(n)'); grid; % pause
figure; plot(n,c(n)-cest(n),'b-');        title('c(n)-cest(n)');   grid; % pause
figure; plot(n,x(n),'r-',n,xest(n),'b-'); title('x(n), xest(n)');  grid; % pause
figure; plot(n,x(n)-xest(n),'b-');        title('x(n)-xest(n)');   grid; % pause
figure; plot(omega*fs/(2*pi));            title('fc(n)');          grid; % pause
