% lab20_ex_pulse_shaping: IQk --> IQ0 --> IQ(n) --> IQk

% When THIS program IS NOT CALLED from lab20_ex_IQpoints.m, SET 1 IN IF()
if( 0 ) % 0/1 Optional starting program from this data
  clear all; close all;
  modtype = '4QAM'; % 2PAM, 4PAM, 8PAM, BPSK, QPSK, DQPSK, 8PSK, 4QAM, 16QAM
  Nsymbols = 250;   % number of symbols to transmit
  [IQcodes, Nstates, Nbits, R ] = IQdef( modtype );
  numbers = floor(Nstates*(rand(Nsymbols,1)-10*eps)); % symbol generation
  IQk = numbers2IQ( numbers, modtype, IQcodes );      % state numbers to IQ values
end

do_figures = 1;            % 0/1 plotting figures
do_updown = 1;             % 0/1 frequency up-down conversion
do_disturb = 1;            % 0/1 addition of disturbances
rctype = 'sqrt';           % raised cosine filter type: 'normal, 'sqrt'
r = 0.35;                  % PS filter roll-off factor
K = 24; Ns = 8;            % samples per symbol, symbols per PS filter 
fs = 250000;               % sampling frequency in Hz: 1, 250e+3, 240e+3
fcar = 50000;              % carrier frequency in Hz

fcut = fs/(2*K);                    % PSF cut-off frequency
Npsf = Ns*K+1; Mpsf=(Npsf-1)/2;     % PSF filter length and its half

numbers  = floor( Nstates*(rand(2*Ns,1)-10*eps) ); % 2*Ns random carrier states
dummy = numbers2IQ( numbers, modtype, IQcodes );   % dummy prefix and postfix
IQdum = [ dummy IQk dummy ];                       % appending prefix & postfix

IQ0 = zeros( 1, length(IQdum)*K ); IQ0(1:K:end) = IQdum; % zero insertion
hpsf = firrcos(Npsf-1, fcut, r, fs,'rolloff',rctype);    % 'normal' or 'sqrt'
IQn = conv( IQ0, K*hpsf ); IQn = IQn(Mpsf+1 : end-Mpsf); % pulse shaping in TX

if(do_figures) lab20_ex_figures_TX, end % OPTIONAL FIGURES

if( do_updown )  % Frequency UP and DOWN, channel and disturbances are in-between
    
 % Frequency UP conversion in TX - quadrature modulator
   N = length(IQn); n = 0:N-1;    % signal length, sample indexes
   y = real(IQn).*cos(2*pi*fcar/fs*n) - imag(IQn).*sin(2*pi*fcar/fs*n);
   if( do_figures )
      Nf = 512; df = fs/Nf; ff = df*(0:1:Nf-1);  fshift = df*(-Nf/2:1:Nf/2-1);
      figure, spectrogram(IQn,Nf,Nf-64,fshift,fs); title('STFT( IQ(n) )'); pause
      figure, spectrogram(y,Nf,Nf-64,fshift,fs); title('STFT(UP IQ(n))'); pause
      figure; Ks=20; NN = min(N,Ks*K-(K-1)); nn=1:NN; kk = 1:K:NN;
      plot( nn, y(nn),'b-',kk,real(y(kk)),'bo','MarkerFaceColor','blue' );
      xlabel('n'); title('Re(y(n))'); grid;
      pause
   end 
  
 % See into the program lab19_ex_single_carrier.m for more details
 % 1) Digital to analog conversion (DAC) and analog signal transmission.
 % 2) Channel is filtering our signal (h = unknown channel impulse response)
 % 3) In analog part, channel and amplifiers add noise, Signal-to-Noise-Ratio (SNR). 
 % 4) In the receiver, signal from antenna is amplified and band-pass filtered (BPF).
 % 5) Then analog to digital conversion (ADC) takes place with sampling ratio
 % equal to the BPF filter bandwidth.

% Frequency DOWN conversion in RX 
  df = 0; dphi = 0; % carrier frequency and phase offsets
  IQnn = 2* y .* exp( -j*( 2*pi*(fcar/fs + df/fs)*n + dphi ) );
  if( do_figures )
    figure, spectrogram(IQnn,Nf,Nf-64,fshift,fs); title('STFT(DOWN IQ(n))'); pause
  end

end

% Disturbances - additive noise, channel gain and phase shift, RX carrier offsets, ADC error 
if( do_disturb )
   SNR=160; chanG=1; chanPh=0; carDf=0; carDph=0; ADCppm=100*1e-6;    
 % SNR=20; chanG=0.25; chanPh=pi/7; carDf=0.001; carDph=pi/11; ADCppm=100; % Params
 % Add noise  
   s = IQnn(Mpsf : end-Mpsf+1);
   scale = sqrt( sum( s.*conj(s) ) / (2*length(s)*10^(SNR/10)) ); clear s
   IQnn = IQnn + scale * (randn(1,N)+j*randn(1,N)); % noise addition with proper SNR
 % IQnn = awgn( IQnn, SNR, 'measured' );            % alternative noise addition
 % Add channel influence
   IQnn = IQnn .* ( chanG * exp(j*chanPh ) );       % equivalent channel in the BB
 % Add carrier frequency and phase offsets
   carDf = carDf - ADCppm/(1+ADCppm)*fcar/fs;
   IQnn = IQnn .* exp(j*(2*pi*carDf*(0:length(IQn)-1)-carDph)); % carrier offsets
   if( do_figures )
       figure, spectrogram(IQnn,Nf,Nf-64,fshift,fs); title('STFT(IQ(n))+Noise)'); pause
   end
end

 IQnn = conv( IQnn, hpsf ); IQnn = IQnn(Mpsf+1:end-Mpsf); % pulse shaping in TX
 IQnn = IQnn( 2*Npsf-1: 1 : end-(2*Npsf-1)+1 );           % signal synchronization
 IQkk = IQnn( 1: K : end );                               % symbol extraction                                  

if(do_figures) % OPTIONAL FIGURES
    figure, spectrogram(IQnn,Nf,Nf-64,fshift,fs); title('STFT(PSF IQ(n))'); pause
    lab20_ex_figures_RX 
end

IQk = IQkk; % when we are going back to the program lab20_ex_IQpoints.m 
