% lab21_ex_channel.m
% Testing channel simulation methods
clear all; close all;

modtype = '4QAM';  % 2PAM, 4PAM, 8PAM, BPSK, QPSK, DQPSK, 8PSK, 4QAM, 16QAM
Nsymbols = 50;     % number of sumbols to transmit
rctype = 'sqrt';   % PSF: raised cosine filter type: 'normal, 'sqrt'
r = 0.35;          % PSF: filter rolloff factor
K = 200; Ns = 8;   % PSF: samples per symbol, symbols per pulse shaping filter 
fs = 250000;       % sampling frequency in Hz: 1, 250e+3, 240e+3
fcar = 65000;      % carrier frequency in Hz
dt = 1/fs;         % sampling period

fcut = fs/(2*K);                  % PSF cut-off frequecy
Npsf = Ns*K+1; Mpsf=(Npsf-1)/2;   % PSF filter length and its half

% Channel
%h = [0.7 0 0.3 0.2 0.05 ];                     % our choice, try-and-check
 h = [0.5 zeros(1,9) 0.1 zeros(1,24) 0.02 ];    % wireless
%h = load('TelephoneLine.dat')';   h=h(23:end); % ADSL telephone csaloop2
%h = load('TelephoneLineHP.dat')'; h=h(23:end); % csaloop2 with HP filter
df=fs/1000; f = 0 : df : fs/2;                  % frequency
H = freqz( h, 1, f, fs );
nc = round( fcar/df ); chanGain = abs(H(nc)), chanPhase = angle(H(nc)), 
    figure; stem(h); grid; xlabel('n'); title('h(n)'); pause
    figure;
    subplot(211); plot(f/1000,abs(H),f(nc)/1000,abs(H(nc)),'ro'), grid; xlabel('f (kHz)'); title('|H(f)| (dB)');
    subplot(212); plot(f/1000,angle(H),f(nc)/1000,angle(H(nc)),'ro'), grid; xlabel('f (kHz)'); title('angle(H(f)) (rad)'); pause

% Data
[IQcodes, Nstates, Nbits, R ] = IQdef( modtype );
numbers = floor(Nstates*(rand(Nsymbols,1)-10*eps));   % symbol generation
IQk = numbers2IQ( numbers, modtype, IQcodes );

% Prefix and postfix
numbers  = floor( Nstates*(rand(2*Ns,1)-10*eps) ); % 2*Ns random carrier states
dummy = numbers2IQ( numbers, modtype, IQcodes );   % dummy prefix and postfix
IQdum = [ dummy IQk dummy ];                       % appending pre & post-fix

% Pulse shaping in TX
IQ0 = zeros( 1, length(IQdum)*K ); IQ0(1:K:end) = IQdum; % zero insertion
hpsf = firrcos(Npsf-1, fcut, r, fs,'rolloff',rctype);    % 'normal' or 'sqrt'
IQn = conv( IQ0, K*hpsf ); IQn = IQn(Mpsf+1 : end-Mpsf); % pulse shaping in TX

% TESTING CHANNEL SIMULATION ##############################################
% Frequency UP conversion in TX
N = length(IQn); n = 0:N-1;               % signal length, sample indexes
y = real(IQn).*cos(2*pi*fcar/fs*n) - imag(IQn).*sin(2*pi*fcar/fs*n);
%y = 0.5 * IQn .* exp(j*2*pi*fcar/fs*n);  % alternative
Y = freqz( y, 1, f, fs )/N;
figure; plot(f,abs(H),'b',f,abs(Y),'r'); grid; xlabel('f (Hz)'); title('H(f), Y(f)'); pause
% Channel simulation in high frequency
y = filter( h,1,y);
% Frequency DOWN conversion in RX 
IQnn = 2 * y .* exp( -j*( 2*pi*fcar/fs*n ) );
% Pulse shaping in RX
IQdown1 = conv( IQnn, hpsf ); IQdown1 = IQdown1(Mpsf+1:end-Mpsf); % pulse shaping in TX

% Channel simulation in the baseband 
hdown = h .* exp( -j*( 2*pi*fcar/fs*(0:length(h)-1) ) );
hdown = conv( hdown, hpsf );
IQdown2  = conv( IQn,hdown ); IQdown2 = IQdown2(Mpsf+1:end-Mpsf);

% Other simplified method - one point deformation, good for small signal bandwidth
IQdown3 = IQn .* chanGain * exp(j*chanPhase); % equivalent channel in the BB
IQdown3 = conv( IQdown3, hpsf ); IQdown3 = IQdown3(Mpsf+1:end-Mpsf);

figure;
plot(1:length(IQdown1),real(IQdown1),'ro-',...
     1:length(IQdown2),real(IQdown2),'bx-',...
     1:length(IQdown3),real(IQdown3),'gs-'); grid; title('Compare I(n)'); pause
figure 
plot(1:length(IQdown1),imag(IQdown1),'ro-',...
     1:length(IQdown2),imag(IQdown2),'bx-',...
     1:length(IQdown3),imag(IQdown3),'gs-'); grid; title('Compare Q(n)'); pause
 
% Calculation of the group delay and the phase delay
P=unwrap(angle(H));     df=f(2)-f(1);       % phase shift, frequency step
k1=find((f>fcar-df/2)&(f<=fcar+df/2));      % freq response index for f=fc;
A=abs(H(k1));                               % channel gain for f=fc;
tp=-P(k1)/(2*pi*fcar),                      % phase delay (tp) in (s)
tg=-(1/(2*pi))*(P(k1+1)-P(k1-1))/(2*df),    % group delay (tg) in (s)
tg_vs_dt=tg/dt,                             % tg as a fraction of sampling period
tp_vs_dt=tp/dt,                             % tp as a fraction of sampling period
