% lab21_ex_equ_mse_nlms.m
% Transmission channel estimation and equalization:
% 1) MSE (mean squared error) - block-based
% 2) NLMS (normalized least mean squares) adaptive filter - sample-based
% In Octave: >>pkg load communications[ENTER] 
clear all; close all;

% Parameters
N = 3200;    % number of input symbols
L = 1000;    % MSE: number of input symbols used by the block-based equalizer
M = 10;      % NLMS: adaptive filter length                   
mi = 0.075;  % NLMS: initialization of adaptation speed constant

% Transmitted signal (carrier states)
x0 = (2*round(rand(1,N))-1) + j*(2*round(rand(1,N))-1);  % IQ 4-QAM states
    figure; plot(x0,'b*'); axis([-1.5,1.5,-1.5,1.5]);
    xlabel('I(k)'); ylabel('Q(k)'); title('IQ(k) - input'); pause

% Channel
h = [ exp(j*2*pi*rand)*0.5;  zeros(2,1); ...
      exp(j*2*pi*rand)*0.25; zeros(3,1); ...
      exp(j*2*pi*rand)*0.1 ]; % channel impulse response
%h = real(h);  
y = conv( x0, h );
    figure; plot(y,'b*'); axis([-1.5,1.5,-1.5,1.5])
    xlabel('I(k)'); ylabel('Q(k)'); title('IQ(k) - after the channel'); pause

% Receiver
y = awgn(y,30,'measured');   % noise addition
    figure; plot(y,'b*'); axis([-1.5,1.5,-1.5,1.5])
    xlabel('I(k)'); ylabel('Q(k)'); title('IQ(k) - after LNA in the RX'); pause

% MSE Equalizer 
V = toeplitz(y(M:L),y(M:-1:1));  % matrix of output symbols 
u = x0(M:L).';                   % input symbols
g = V\u,                         % channel inverse filter
g1 = g;                          % storing

xest = V*g;                      % data correction #1
errors1a = sum( round( x0(M:L).' ) ~= round( xest ) ), pause
    figure; plot( xest,'b*'); axis([-1.5,1.5,-1.5,1.5]);
    xlabel('I(k)'); ylabel('Q(k)'); title('IQ(k) for EQU mtx #1'); pause
    
xest = conv( y, g.' );           % data correction #2
errors1b = sum( round( x0(M:N) ) ~= round( xest(M:N) ) ),
    figure; plot( xest(M:N),'b*'); axis([-1.5,1.5,-1.5,1.5]);
    xlabel('I(k)'); ylabel('Q(k)'); title('IQ(k) for EQU mtx #2');pause

% Adaptive NLMS equalizer
xest = [];                               % removing from memory
by = zeros(M,1);                         % initialization of input buffer
g  = zeros(M,1);                         % initialization of filter weights
for n = 1 : N                            % main loop beginning
   by = [ y(n); by(1:M-1) ];                              % buffer actualization
   xest(n) = sum( by .* conj(g(:,n)) );                   % signal filtering 
   err(n) = x0(n) - xest(n);                              % error signal
   g(:,n+1) = g(:,n) + mi * conj(err(n)) * by / (by'*by); % weights adaptation
   if(rem(n,100)==0) figure(6), plot( xest(n-50:n ),'*'); pause(0.1); end 
end                                      % main loop end
errors2 = sum( round( x0(N/2:end) ) ~= round( xest(N/2:end) ) ), pause
    figure; plot( xest(N/2:end),'b*'); axis([-1.5,1.5,-1.5,1.5]);
    xlabel('I(k)'); ylabel('Q(k)'); title('% IQ(k) for EQU mtx #3'); pause
    figure; n = 1:N;
    plot(n,abs(g(1,n)),'b-',n,abs(g(4,n)),'r-',n,abs(g(7,n)),'k-'); title('g(n)'); pause
    
g2 = g(:,end);                           % storing
[ g1 g2 ], pause                         % comparison
