% lab21_ex_gdelay.m
% Testing phase and group delay
clear all; close all;

sigtype = 1;              % signal envelope type: 1=Gaussian, 2=raised cosine
fs = 250000; fcar=50000;  % sampling and carrier frequencies in Hz
Ksample = 25; Nsymb = 12; % PSF: samples per symbol, symbols per pulse shaping filter (PSF)
r = 0.35;                 % PSF: filter rolloff factor
fcut = fs/(2*Ksample);    % PSF: cut-off frequecy
Npsf = Ksample*Nsymb+1;   % PSF: filter length
N = Npsf;                 % number of samples
Nc = Ksample*Nsymb/2;     % index of central sample

dt = 1/fs;

% Channel/filter parameters    
h = [0.7 0 0.35 0.2 0.05 ];     % weights (channel impulse response), try-and-check
df=fs/1000; f = 0 : df : fs/2;  % frequency
H = freqz( h, 1, f, fs );       % channel frequency response 
nc = round( fcar/df ); chanG = abs(H(nc)), chanPh = unwrap(angle(H(nc))), 
tp = -chanPh/(2*pi*fcar), np = tp/dt,                              % phase delay
tg = -(unwrap(angle(H(nc+1)))-unwrap(angle(H(nc-1))))/(2*pi*2*df), % group delay
ng = tg/dt, pause
  figure; stem(h); grid; xlabel('n'); title('h(n)'); pause
  figure;
  subplot(211); plot(f/1000,abs(H),f(nc)/1000,abs(H(nc)),'ro'), grid; xlabel('f (kHz)'); title('|H(f)| (dB)');
  subplot(212); plot(f/1000,angle(H),f(nc)/1000,angle(H(nc)),'ro'), grid; xlabel('f (kHz)'); title('angle(H(f)) (rad)'); pause

% Signals
dt=1/fs; n=0:N-1; t=dt*n;                             % sampling period
if( sigtype==1)                                       % Gaussian envelope
    xenv = exp(-(n-Nc).^2/(10*Npsf));                 % ENV transmitted
    xenv_ref = exp(-((n-ng)-((N+1)/2)).^2/(10*Npsf)); % ENV delayed
else                                                  % raised cosine envelope 
    xenv = myrcos((Npsf+1)/2,Npsf,Nsymb,r);           % ENV transmitted
    xenv_ref = myrcos((Npsf+1)/2+ng,Npsf,Nsymb,r);    % ENV delayed
end               
xc = cos(2*pi*fcar/fs*n);                             % carrier
xc_ref = cos(2*pi*fcar/fs*(n-np));                    % carrier delayed
x  = xenv .* xc;                                      % modulation proces  
    figure; plot(t,x,'r.-',t,xenv,'b-'); grid; xlabel('t (s)'); title('x(t)'); pause

% Signal passing through the channel/filter  
y = filter(h,1,x);                                    % channel output
y_ref = chanG * xc_ref .* xenv_ref;                   % channel output expected
y_error = y - y_ref;                                  % output error
  figure; plot(n,y_error); grid; xlabel('n'); title('Error of y(n)'); pause
  figure; plot(n,x*chanG,'ko-',n,y_ref,'bo-',n,y,'rx-',n,xenv*chanG,'k.-',n,xenv_ref*chanG,'b.-');
          grid; xlabel('n'); title('Comparison');
  legend('G*x(n)','y_{ref}(n)','y(n)','G*xenv(n)','G*xenv_{ref}(n)'); pause

function hpsf = myrcos(Nc,Npsf,Nsymb,r)
n=0:Npsf-1;
hpsf = sin(pi*(n-Nc)/Nsymb) ./ (pi*(n-Nc)/Nsymb) .* ...
        cos(pi*r*(n-Nc)/Nsymb) ./ (1-(2*r*(n-Nc)/Nsymb).^2);
if( rem(Nc+1,round(Nc+1))==0) hpsf(Nc+1) = 1; end  
n0 = find( abs(1-(2*r*(n-Nc)/Nsymb).^2 ) < sqrt(eps) );
hpsf(n0) = r/2 * sin(pi/(2*r));
end
