% lab21_ex_nanosat.m - slow mode bit transmission from AMSAT nano-satellites  
  clear all; close all;

  m=128; cm_plasma=plasma(m); cm = plasma;         % color maps for gray printing

% Read recorded IQ signal - choose one
  FileName = 'SDRSharp_NanoSat_146000kHz_IQ.wav';  % file to be read
  [x,fs] = audioread(FileName);                    % reading, fs = 192 kHz
  Nx = length(x),                                  % signal length
  x = x(:,1) - j*x(:,2);                           % IQ signal formation
      N = 512; df = fs/N; ff = df*(0:1:N-1);  ffshift = df*(-N/2:1:N/2-1);
      figure; nd = 1:2500;
      plot(nd,real(x(nd)),'bo-',nd,imag(x(nd)),'r*--'); xlabel('n'); grid; 
      title('I(n) (BLUEo),  Q(n)(RED*)'); pause
      figure; Nstft = N;
      spectrogram(x,kaiser(Nstft,10),Nstft-Nstft/4,ffshift,fs);
      colormap(cm); pause
% FM demodulation and NanoSat voice control signal decoding
  faudio = 48000; Down= round(fs/faudio);              % parameter values
  f0 = 1.98e+4; x = x .* exp(-j*2*pi*f0/fs*(0:Nx-1)'); % carrier? spectrum shifting 
      figure;
      N = 512; df = fs/N; ffshift = df*0.125*(-N/2:1:N/2-1);
      spectrogram(x,kaiser(N,10),N-N/4,ffshift,fs);
      colormap(cm); pause
  h = fir1(500,12500/fs,'low',chebwin(501,120));       % low-pass filter design 
  x = filter(h,1,x); x = x(1:Down:end);                % filtration
  dt = 1/faudio; x = x(2:end).*conj(x(1:end-1)); y=angle(x)/(2*pi*dt); % FM demod
  %soundsc(y,faudio);                                                  % listening
      figure; plot(y); grid; 
      title('Digital signal in time domain - all samples'); 
      xlabel('sample number'); ylabel('Signal values'); pause
  
% Bits decoding ###########################################################

% fsymb = 200Hz = 200 symbols per second, Nsps = 240 samples per symbol
  fsymb = 200;               % symbol rate [Hz], 200 symbols per second 
  r = 0.5;                   % PSF: 0.5 - roll-off factor of RC filter 
  Nspan = 2;                 % PSF: 2 - number of symbols per RC filter
  Nsps = faudio/fsymb;       % PSF: nominal number of samples per symbol, equal 240

% Taking signal fragment, first or second voice beacon
  n = 170000:655000;         % selected manually voice beacon, visual inspection
  y = y(n); y = y-mean(y);   % cutting signal fragment
      figure;
      n = 1:length(y);  
      plot(n,y); grid; title('y(n) - the whole voice beacon'); 
      xlabel('sample number'); ylabel('Signal values'); pause
      figure;
      n = 5000 + (1 : 50*Nsps);
      plot(n,y(n));
      title('y(n) - beginning of the voice beacon'); axis tight;
      xlabel('sample number'); ylabel('Signal values'); pause
      figure;
      n = 240000 : 260000;
      plot( n, y(n),'.-' ); title('y(n) - zooming of the beacon center');
      xlabel('sample number'); grid; axis tight;
      pause
      figure;  N=256; df = faudio/N; f = 0.25*df*(-N/2:N/2-1); 
      spectrogram(y,kaiser(N,10),N-N/8,f,faudio); colormap(cm); pause

% Low-pass, square root raised cosine pulse shaping filter (PSF) design
  h = firrcos(Nspan*Nsps, fsymb, r-0.001, faudio,'rollof','sqrt');  % PS filter; 0.001 for Octave
      f = 0 : 1 : 2000; H = freqz(h,1,f,faudio);
      figure; plot(h,'b.-'); xlabel('n'); title('h(n)'); grid; pause
      figure; plot(f,20*log10(abs(H))); xlabel('f [Hz]'); grid; title('| H(f)| [dB]'); pause
% Low-pass filtering
  ypsf = conv(y,h);
  ypsf = ypsf - mean(ypsf);
% Removing filter delay - transients  
  Nh = length(h); Nskip = (Nh-1)/2;
  ypsf = ypsf(Nskip+1:end-Nskip);
      figure;
      n = 1:length(ypsf);  
      plot(ypsf,'.-'); title('y(n) LP filtered - the whole voice beacon'); xlabel('sample number'); grid;
      pause
      figure
      n = 8000:18000;  % observe the synchro #1 word in the beginning
      plot(n, ypsf(n),'.-' ); xlabel('sample number'); grid;
      title('y(n) LP filtered - zooming the beacon beginning '); pause
      figure;
      n = 240000 : 260000;
      plot( n, ypsf(n),'.-' ); title('y(n) LP filtered - zooming of the beacon center');
      xlabel('n'); grid;
      pause

% Manual selection of the position of the starting symbol (-1 or +1)
% It is present just before the first synchronization word of the beacon
  Nmax = length(ypsf); n = 1:Nmax;
  indstart = 9383;  % 9383/9384 starting position of the voice beacon

% Adaptive bits detection - changing step +/-1, approx. 240-samples long
  bstring = '';    % message (bits) received, written to string - at present nothing
  bpos = [];       % indexes of bit positions, at present nothing
  pos = indstart;  % position of the starting bit "0"(-1) or "1"(1), found manually
  m = 1;           % working variable for indexing vector elements
  ybits = [];      % vector of signal values at bit positions, to be filled
  while( pos + 1 <= Nmax )  % MAIN LOOP
      ybits(m) = ypsf(pos); % first symbol/bit
      bpos = [ bpos pos ];  % storing its position 
      if( ypsf(pos) > 0)    % bit "1=+1", should be maximum
          bstring=strcat( bstring, '1');
          if ( (ypsf(pos-1) < ypsf(pos)) && (ypsf(pos) < ypsf(pos+1)) )
              pos = pos+1;  % increase position by one, you are early, positive slope
          elseif ( (ypsf(pos-1) > ypsf(pos)) && (ypsf(pos)>ypsf(pos+1)) )
              pos = pos-1;  % decrease position by one, you are late, negative slope
          end
      else                  % bit "0=-1", should be minimum
          bstring=strcat( bstring, '0');
          if ( ypsf(pos-1)<ypsf(pos) && ypsf(pos)<ypsf(pos+1) )
              pos = pos-1;  % decrease position by one, you are late, positive slope
          elseif (ypsf(pos-1)>ypsf(pos) && ypsf(pos)>ypsf(pos+1) )
            pos = pos+1;    % increase position by one, you are early, negative slope
          end
      end                   % MAIN LOOP END   
      pos = pos + Nsps;     % do next step, after position correction
      m = m + 1;            % increase index of detected bits
  end
  figure; plot(ybits,'bo'); xlabel('n'); title('Carrier values at constellation points'); grid; pause
  figure; plot(n,ypsf,'r.-',bpos, ypsf(bpos),'bo'); title('Synchronized bits!'); pause

% Searching for synchronization words
  synchro1 = '1100000101';
  synchro2 = '0011111010';
% bstring(100:109)=synchro1;  bstring(200:209)=synchro2; % for test
  for n = 1 : length(bstring)-9
      result1(n) = strcmp( bstring(n:n+9), synchro1 );
      result2(n) = strcmp( bstring(n:n+9), synchro2 );
  end
    figure;
    n = 1 : length(bstring)-9;
    plot(n,result1,'ro',n,result2,'bo'); grid; xlabel('n');
    axis([ n(1), n(end), 0, 1.1 ] );
    title('Detection of Synchronization Words');
    pause

% Finding bit positions and number of bits between synchronization words
  ind1 = find( result1 == 1 );   % 1st bit of sync word #1
  ind2 = find( result2 == 1 );   % 1st bit of sync word #2
  ind = sort( [ ind1, ind2 ] );  % sorting the bits
  i1 = ind(1)+10; i2 = ind(2)-1; % indexes of 1st and last bit in 1st data block           
  len1 = i2-i1+1,                % number of bits in the first data block
  i3 = ind(2)+10; i4 = ind(3)-1; % indexes of 1st and last bit in second data block
  len2 = i4-i3+1,                % number of bits in the second data block

% Storing found bits to disc for further analysis - optional
if(0)                                                % first block of 960 bits
   sync1 = bstring( ind(1) : ind(1)+9 );             % sync 1  
   msg1  = bstring( i1 : i2 );                       % data 1
   sync2 = bstring( ind(2) : ind(2)+9 );             % sync 2 
   fid = fopen('msg1.dat','w');                      % file open
   fprintf(fid,'%s \n %s \n %s',sync1, msg1, sync2); % sync1, data1, sync2
   fclose(fid);                                      % file close
                                                     % second block of 960 bits 
   sync2 = bstring( ind(2) : ind(2)+9 );             % sync 2             
   msg2  = bstring( i3 : i4 );                       % data 2
   sync3 = bstring( ind(3) : ind(3)+9 );             % sync 3 
   fid = fopen('msg2.dat','w');                      % open file
   fprintf(fid,'%s \n %s \n %s',sync2, msg2, sync3); % sync2, data2, sync3
   fclose(fid);                                      % close file
end
