% lab21_ex_rdsbits_decoder_decim_costas_earlylate.m
% RDS bits decoder with Costas loop synchronization and early-late-gate timining recovery 
% WITH ADDITIONAL SIGNAL DECIMATION
clear all; close all;

FMradio_params  % parameters from included file: frequency values, filter weights, etc.
n = 1: 2^14;    % for display only

[y,fs] = audioread('FMRadio_IQ_250kHz_LR.wav');  % read Goodbye+Danke, L, R, LR
%[y,fs] = audioread('SDRSharp_FMRadio_96000kHz_IQ_one.wav',[1,15*250000]);
y = y(:,1) + sqrt(-1)*y(:,2);                    % IQ --> complex

% DECODING MONO L+R SIGNAL
dy = (y(2:end).*conj(y(1:end-1)));       % calculation of instantaneous frequency
y = atan2(imag(dy), real(dy)); clear dy; % frequency demodulation
ym = filter( hLPaudio, 1, y );           % low-pass filtration of L+R (mono) signal
ym = ym(1:fs/faudio:end);                % leaving only every fs/faudio-th sample
%disp('LISTENING TO MONO: Left + Right'); sound(ym,faudio); pause % listening to L+R

% DECODING RDS SIGNAL

% Band-pass filtering around 57 kHz
  K = (length(hBP57)-1)/2;
  y = conv(y,hBP57); y = y(K+1:end-K);
  figure, spectrogram(y(n), 512, 512-64, 512, fs); title('After BPF 57kHz'); pause
  
% Down-sampling M times
  M = 14; fsnew=fs/M;       % M=16 instead of M=14?
  y = y(1:M:end); n = 1 : min( length(y), 2^14);
  frdsdown = frds-3*fs/M;   % RDS new freq 3428.6 Hz for M=14 - TO BE CHANGED for M=16
  figure, spectrogram(y(n), 512, 512-64, 512, fs/M); title('After M-times down-sampling'); pause

% Costas loop working with lower frequency
  y = hilbert( y );                                      % analytic signal
  theta = zeros(1,length(y)+1); omega=theta; omega(1)=2*pi*frdsdown/(fs/M);
  mi1 = 2*0.0011; mi2 = 2*5.6859e-07;                    %#
  for n = 1 : length(y)                                  %# Costas loop
      bb(n) = y(n) * exp(-j*theta(n));                   %# base-band signal
      delta(n) = real( bb(n) ).*imag( bb(n) ) / (bb(n).*conj(bb(n))); % error
      theta(n+1) = theta(n) + omega(n) + mi1*delta(n);   %# 1-st update
      omega(n+1) = omega(n) + mi2*delta(n);              %# 2-nd update
  end                                                    %#
  y = real( bb ); y = y'; clear bb delta theta           %# freq-down RDS
  figure; plot(fs*omega/(2*pi)); title('fcostas(n)'); pause; clear omega

% #######################################
% RDS BITS DETECTION
% #######################################

rds  = y(1000:end);       % we are removing first transition states
Nmax = length(rds);       % new signal length
Nrds = round(Nsymb/M);    % samples per symbol after decimation

% Running summation of Nrds samples - looking for the "camel": (+1) --> (-1)
h = ones(1,Nrds)/Nrds; Nrds2 = round(Nrds-1)/2;             % filter weights
synch = conv(rds,h); synch = synch( Nrds2+1 : end-Nrds2 );  % convolution
thres = 0.75*max(synch);                             % threshold
up = find( synch > thres );                          % find samples above
[ dummy indmax] = max( synch( up(1):up(1)+Nrds ) );  % first maximum position        
indmax = up(1) + indmax - 1;                         % plus offset                         
indstart = indmax + round(Nrds/4);                   % plus shift to symbol
n = 1 : length(rds); 
figure; plot(n,rds,'ro-',n,synch,'bx',indmax,synch(indmax),'bs',...
             indstart,rds(indstart),'b*'); title('Synchro?'); pause

% Early-late-gate method
msgR = [ 1 ];      % received message
indR = [];         % indexes of symbol positions 
ind = indstart;
while( ind + Nrds <= Nmax )
    indR = [ indR ind ]; 
    if( rds(ind) > 0)   % detected "1", slope checking, +1/0/-1 step change 
        msgR = [ msgR 1 ];
        if     (rds(ind-1)<rds(ind) && rds(ind)<rds(ind+1))  ind = ind+1;
        elseif (rds(ind-1)>rds(ind) && rds(ind)>rds(ind+1))  ind = ind-1;
        end
    else                % detected "0", slope checking, +1/0/-1 step change
        msgR = [ msgR 0 ];
        if     (rds(ind-1)<rds(ind) && rds(ind)<rds(ind+1))  ind = ind-1;
        elseif (rds(ind-1)>rds(ind) && rds(ind)>rds(ind+1))  ind = ind+1;
        end
    end    
    ind = ind + Nrds;   % predicted position of the next symbol
end

plot(n,rds,'ro-',indR,rds(indR),'b*'); title('Synchro!'); pause

% Differential RDS decoding
rdsbits = abs(msgR(2:end) - msgR(1:end-1)),
rdsbits = rdsbits', pause

% Storing bits to file for following RDS text detection using program rds.m
% save FM_Radio_RDS.txt rdsbits -ascii
