function [Data, Check, Corrected, Bits] = CRC16FireCodeCorrect(Data)
                                                    % for DAB CRC16
%      CRC = [1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1];     % Initialization word
% Polynomial = [0 0 0 1 0 0 0 0 0 0 1 0 0 0 0];     % [MSB ... LSB]
%        x^16+ [    x^12     +    x^5        ] +1   % Generating polynomial
% Data(end-15:end) = xor( Data(end-15:end),CRC );   % complement of 16 last data

                                                    % for DAB FireCode
       CRC = [0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0];     % Initialization word
% Polynomial = [0 1 1 1 1 0 0 0 0 0 1 0 1 1 1];     % [MSB ... LSB]
%        x^16+ [  x^14+x^13+x^12+x^11 + x^5 + x^3+x^2+x^1 ] + 1  % Generating polynomial  
  
  Polynomial =   [1 1 1 0 1 0 0 0 0 0 1 1 1 1 0];     % [LSB ... MSB]
  PolyAll    = [1 0 1 1 1 1 0 0 0 0 0 1 0 1 1 1 1];   % [MSB ... LSB]

% For power=nulls=341-88=253  
% Delay =   [ 0 1 1 1 0 0 0 0 0 0 0 0 1 1 1 0 1 ];    % [MSB ... LSB] (x^15 + .... + 1)
% Delay =   [   1 0 1 1 1 0 0 0 0 0 0 0 0 1 1 1 ];    % [LSB ... MSB] ( 1 + ... + x^15)

% For power=nulls+16=269
% Delay =   [ 0 0 0 0 1 1 0 0 0 0 0 1 0 0 0 1 1 ];    % [MSB ... LSB] (x^15 + .... + 1)
  Delay =   [ 1 1 0 0 0 1 0 0 0 0 0 1 1 0 0 0 0 ];    % [LSB ... MSB] ( 1 + ... + x^15)
% Delay =   [ 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 ];    % [LSB ... MSB] ( 1 + ... + x^15)

  Data = [ Data(17:end) Data(1:16) ];    % Shift parity bits to the end
% Data = [ Data zeros(1,341-88) ];       % Appand zeros since unshortened code is (341,325)
                                         % Shortened (88,16)
% Data = [ Data zeros(1,88) ];           % Appand zeros
                                         
  %##########################################
  if(1) % Option #1 with circular shift by 16
      
% ps = [ 0 1 1 1 1 0 0 0 0 0 1 0 1 1 1 1 ]; % polynomial of 16-bit shift
% ps = [ 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 ]; % polynomial of 16-bit shift
% ps = [ 1 1 1 0 0 0 0 0 0 0 0 1 1 1 0 1 ]; % For power=nulls=341-88=253
  ps = [ 0 0 0 1 1 0 0 0 0 0 1 0 0 0 1 1 ]; % For power=nulls+16=269
  ps = fliplr(ps);
  for i = 1:length(Data)                              % CRC calculation
    if( xor( CRC(end),Data(i)*ps(1) ) == 1)
        CRC = [ 1 xor( xor( Polynomial*CRC(end),CRC(1:end-1) ), Data(i)*ps(2:end) ) ];
    else
        CRC = [ 0 xor( xor( Polynomial*CRC(end),CRC(1:end-1) ), Data(i)*ps(2:end) ) ];
    end
  end
  CRC,
  end
  %##
  
  if(sum(CRC) == 0)                     % CRC verification
     Check = 1,
  else
     Check = 0,
  end

  Corrected = 0;
  if( Check == 0 )                            % Try to correct
    tryagain = 1; n = 0;
    while( tryagain )
        
        if( sum(CRC(1:10)) == 0 )             % Correction
            Data(n+1:n+6) = xor( Data(n+1:n+6), CRC(16:-1:11) ); 
            Corrected = 1;
            tryagain=0;
        end % if zero    

        if( (tryagain==1) & (CRC(16)==1) )    % CRC circular shift
            CRC = [ 1 xor( Polynomial,CRC(1:15) ) ];
        else
            CRC = [ 0 CRC(1:15) ];
        end
        
        n = n+1;
        if( n==88) tryagain=0; end      % End condition
     %  n, CRC(1:10),CRC(11:16), pause
     %  n, CRC(1:6),CRC(7:16), pause
        
    end % while
end % if Check==0

Bits = sum(CRC(16:-1:11));
Data = [ Data(73:88) Data(1:72) ];      % Correct order of samples
