%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Masterthesis
% Zurcher Hochschule fur Angewandte Wissenschaften
% Zentrum fur Signalverarbeitung und Nachrichtentechnik
% (c) Michael Hoin
% 12.4.2011 ZSN
% info.zsn@zhaw.ch
%
% and T. Zielinski tzielin@agh.edu.pl
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% -----------------------------------------------------------------------------------------
% Viterbi Decoder
% -----------------------------------------------------------------------------------------

function [ Bits ] = DABViterbiTZ( yIn, Generator, InStates1, InStates2, RefOutValues1, RefOutValues2, offset )
% offset = 0 for FIC
%          1 for MSC

NBitStreams = length(Generator(:,1));   % =4 in DAB  % how many (bit)streams are coming IN?
NConvTabs   = length( Generator(1,:) ); % =7         % number of convolution weights
NDelays     = NConvTabs - 1;            % =6         % number of delay elements
NMemDepth   = 5*NDelays;       %5?      % 5*6=30     % number of extra rows added to the input data matrix

                                                            % input: vector --> matrix (:,NBitStreams)
y = reshape( yIn, NBitStreams, length(yIn)/NBitStreams )';  % put NBitStreams=4 data into each row
y = [ y; zeros(NMemDepth,NBitStreams) ];                    % add zeros to the end of input data 

PathBuff = zeros( 2^NDelays, NMemDepth );     % Path Buffer (64 x 30)
AccMetricBuff = -1000*ones( 2^NDelays, 2 );   % Accumulated Metric Buffer for last two symbols (64 x 2)
AccMetricBuff(1) = 0;                         % First value of set to 0
Bits = zeros( 1, length(y(:,1))-NMemDepth );  % Vector of output codewords (bits), initially filled with 0
BitIndex = 1;                                 % Index of the first codeword (bit)

mRead=1; mWrite=2;                            % column number of Matric matrix for reading/writing (1 or 2)
pLast = NMemDepth;                            % last (actual) position in PathBuff

% Fundamental feature of Viterbi trellis: two inputs into each state (node):
% UP=0:   (0,1)->(0),  (2,3)->(1),  (4,5)->(2),  (6,7)->(3),  ...,(62,63)->(31)
% DOWN=1: (0,1)->(32), (2.3)->(33), (4,5)->(34), (6,7)->(35), ...,(62,63)->(63)
% Figs. 21.12 (put left) + 21.8 (put right) from [PWN TZ 2014]

for i = 1 : length(y(:,1))-offset             % For each input 4-element symbol (one row with 4 values)
    
    for index = 1:2^NDelays                   % For each inner state 1:64  
        
      % Metric change: measure similarity of y(i,:) and REF {-1/1} values, then add metric history (accumulate)
        Value1 = RefOutValues1(index,1:NBitStreams)* y(i,:)' + AccMetricBuff( InStates1(index)+1, mRead ); % EVEN
        Value2 = RefOutValues2(index,1:NBitStreams)* y(i,:)' + AccMetricBuff( InStates2(index)+1, mRead ); % ODD 
      % Decision: what transition path is better (what similarity is higher?)  DO UPDATE !
        if(Value1 > Value2)     % EVEN                 % Even or Odd?
            AccMetricBuff(index,mWrite) = Value1;      % Update of accumulated metric
            PathBuff(index,pLast) = InStates1(index);  % Store information about better EVEN path
        else                    % ODD
            AccMetricBuff(index,mWrite) = Value2;      % Update of accumulated metric
            PathBuff(index,pLast) = InStates2(index);  % Store information about better ODD path
        end    
    end % of each inner state
  
  % DECISION ########################################################################################  
    if i > NMemDepth-offset                           % DO ONLY ... after NMemDepth input symbols
        [Nothing, Pos] = max(AccMetricBuff(:,mWrite));% Looking for the max value in the Metric table
        for p = 0 : NMemDepth-1                       % # Tracing the path "backward" in time
            new = pLast-p;                            % # !new column back
            if(new<1) new = new+NMemDepth; end        % # !correction since buffer is circular
            Pos = PathBuff(Pos,new)+1;                % # having Pos for time "k", read Pos for time "k-1" 
        end                                           % #
        Pos = Pos - 1;                                % # decrease the Pos value (since min=0 not 1)
        
        if Pos >= 2^(NDelays-1)  % =32  % Find the bit value using the best path traced "back"
            Bits( BitIndex ) = 1;       %
        else                            %
            Bits( BitIndex ) = 0;       %
        end                             %
        BitIndex = BitIndex + 1;        % update the BitIndex (counter)
    end                                 %
  
  % BUFFER CONTROL ################################################################################  
    pLast = pLast+1;                    % Update pointer to PathBuffer
    if(pLast>NMemDepth) pLast = 1; end  %
    
    if(mRead==1) mRead=2; mWrite=1;     % Flip-flop of column index in the Matric matrix (2 cols)
    else         mRead=1; mWrite=2;     %
    end                                 %
    
end % of next input 4-element symbol y(k,:) 
