function [ Generator, InStates1, InStates2, RefOutValues1, RefOutValues2 ] = DABViterbiTZinit()

Generator = [ 1 0 1 1 0 1 1;...         % 133 octal   % Code for the conv. encoder/decoder
              1 1 1 1 0 0 1;...         % 171 octal   % Data generator matrix with dim. 4(bits) x 7(weigths)
              1 1 0 0 1 0 1;...         % 145 octal   % One bit(value) IN --> Four bits OUT
              1 0 1 1 0 1 1     ];      % 133 octal   % Memory = 1+6 = 7 positions
         
NBitStreams = length(Generator(:,1));   % =4 in DAB   % how many (bit)streams are coming OUT/IN?
NConvTabs   = length( Generator(1,:) ); % =7          % number of convolution weights
NDelays     = NConvTabs - 1;            % =6          % number of delay elements

% Fundamental feature of Viterbi trellis: two inputs into each state (node):
% UP=0:   (0,1)->(0),  (2,3)->(1),  (4,5)->(2),  (6,7)->(3),  ...,(62,63)->(31)
% DOWN=1: (0,1)->(32), (2.3)->(33), (4,5)->(34), (6,7)->(35), ...,(62,63)->(63)
% Figs. 21.12 (put left) + 21.8 (put right) from [PWN TZ 2014]

                                        % Inner states: 1 = even state, 2 = odd state
InStates1 = [ 0:2:2^NDelays-2 0:2:2^NDelays-2 ];      % [ 0:2:62, 0:2:62 ] EVEN 0,2,4,..62,0,2,..62
InStates2 = [ 1:2:2^NDelays-1 1:2:2^NDelays-1 ];      % [ 1:2:63, 1:2:63 ] ODD  1,3,5,..63,1,3,..63

RefOutValues1 = zeros(2^NDelays, NBitStreams);        % Initialization
RefOutValues2 = zeros(2^NDelays, NBitStreams);        % Can be skipped

for index = 1:2^NDelays                               % Inner states 1:64
    if index <= 2^(NDelays-1)  NewBit = 0;            % index <= 32
    else                       NewBit = 1;            % index > 32
    end            
  % Next INNER state (7 taps, values 0 or 1): after addition of 0 or 1 in the beginning of 6 taps
    NextInState1 = [ NewBit  double( dec2bin( InStates1(index), NDelays )-'0' ) ];   % EVEN 0,2,4,..62,0,2,..62 
    NextInState2 = [ NewBit  double( dec2bin( InStates2(index), NDelays )-'0' ) ];   % ODD  1,3,5,..63,1,3,..63
%   pause
  % Next OUTPUT state (4 taps, values 1 or -1) for even and odd state number
    RefOutValues1(index,1:NBitStreams) = 1 - 2.*mod( NextInState1 * Generator', 2 ); % EVEN
    RefOutValues2(index,1:NBitStreams) = 1 - 2.*mod( NextInState2 * Generator', 2 ); % ODD
end
% RefOutValues1, pause
% RefOutValues2, pause
