%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Masterthesis
% Zurcher Hochschule fur Angewandte Wissenschaften
% Zentrum fur Signalverarbeitung und Nachrichtentechnik
% (c) Michael Hoin, 12.4.2011 ZSN, info.zsn@zhaw.ch
% Extended:
% AGH University of Science and Technology, Krakow, Poland
% (c) Tomasz Zielinski, 31.12.2019, tzielin@agh.edu.pl
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% -------------------------------------------------------------------------
% Reading next block of complex IQ data from a file
% DAB+ recordings: https://sdr.kt.agh.edu.pl/sdrdab-decoder/download.html
% -------------------------------------------------------------------------

function [Signal] = ReadIQData( ReadFile, ReadSize, fs_file, fs, sigtype )

persistent N1 ReadAlready

if( prod(size(ReadAlready))==0 ) ReadAlready = 0; end

if(sigtype==1)  % USRP Ettus Research FLOAT
   t = fread( ReadFile, [2, ReadSize], 'float' );
   v = t(1,:) + t(2,:)*i;
end
if(sigtype==2) % USRP Ettus Research INT16
   t = fread(ReadFile,[2,ReadSize],'int16');
   v = t(1,:) + t(2,:)*i;
end
if(sigtype==3) % Windows: SDRSharp RTL USB Stick INT8                 
   if( isempty(N1) ) N1=1; end 
   t = audioread( ReadFile, [N1, N1+ReadSize-1] ); t=t';
   N1 = N1 + ReadSize;
   v = t(1,:) - t(2,:)*i;
end
if(sigtype==4) % Linux: RTL USB Stick, 8bit ADC, recored by rtl_sdr, UINT8
    [v count] = fread(ReadFile, 2*ReadSize, 'uint8');
    if( count ~= 2*ReadSize )
        throw();
    end
    v = real(v);     % eg. rtl_sdr parameters:
    v = (v-127)/128; % rtl_sdr -f 229072000 -s 2400000 -n 24000000 -g 28 -S dab_file.raw      
    v = v';
    v = v(1:2:end) + 1i*v(2:2:end);
end

if(sigtype==5) % Linux: RTL USB Stick, 8bit ADC, recored by rtl_sdr, INT8
    [v count] = fread(ReadFile, 4*ReadSize, 'int8');
    if( count ~= 4*ReadSize )
        throw();
    end
  % v(1:10), pause
  % v = real(v);         % eg. rtl_sdr parameters:
  % v = (v-2^15+1)/2^15; % rtl_sdr -f 229072000 -s 2400000 -n 24000000 -g 28 -S dab_file.raw      
    v = v.';
    v = v(1:4:end) + j*v(3:4:end);
  % v(1:10), pause
end

[r, c] = size(v);
v = reshape(v, c, r);
if( fs_file ~= fs )
    v = resample(v, fs/1e3, fs_file/1e3);  % resampling from fs_file to fs (page 145)
end    
Signal = v;
Signal = Signal - mean(Signal);

ReadAlready = ReadAlready + ReadSize; % pause

if(0) % Cut and store to disc
   % WriteFile = fopen('DAB_real_2.048MHz_IQ_float.dat', 'wb');
     WriteFile = fopen('DAB_synt_2.048MHz_IQ_float.dat', 'wb');
     n1=100001; n2=ReadSize;
     vv = [ real(v(n1:n2))'; imag(v(n1:n2))']; 
     fwrite(WriteFile,vv,'float');
     fclose(WriteFile);
     disp('WRITTEN TO DISC!'); pause
end
