%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Masterthesis
% Zurcher Hochschule fur Angewandte Wissenschaften
% Zentrum fur Signalverarbeitung und Nachrichtentechnik
% (c) Michael Hoin, 12.4.2011 ZSN, info.zsn@zhaw.ch
% Extended:
% AGH University of Science and Technology, Krakow, Poland
% (c) Tomasz Zielinski, 31.12.2019, tzielin@agh.edu.pl
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% -------------------------------------------------------------------------
% Frame start detection
% First OFDM symbol in each TX frame = NULL signal (equal zero).
% Plus TTI (optional, info about TX) - should be very small.  
% -------------------------------------------------------------------------

function [ SignalOut, OldData ] = SynchroTimeFreq_Short( Signal, CorrectADC, ...
                                  CorrectFreqOffset, CorrectTimeSynchro, PlotON )
                              
global NSampPerFrame NSampPerFrameSymb
global NSampPerNull NSampPerSymb NSampPerPrefix Nfft 
global PhaseRefSymb sigPhaseRefSymb fs
global DABFrameFoundNr 

if( prod(size(DABFrameFoundNr))==0 ) DABFrameFoundNr = 0; end

%@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
if( length(Signal) > 2*NSampPerFrame )  % continue only if Signal is long enough
%@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
    
  disp(' NEW PhaseRef #########################################################');

  x = Signal;
  % plot(real(x)); pause

% ###################################
% Detection of first two NULL symbols
% ###################################

  if( DABFrameFoundNr == 0 )
    % First DAB frame
      xabs = abs(Signal);
      NSampPerSynchro = NSampPerFrame + NSampPerNull;
      acc = zeros(1,NSampPerSynchro); acc(1) = sum( xabs(1:NSampPerNull) );
      for n = 1 : NSampPerSynchro; 
          acc(n+1) = acc(n) - xabs(n) + xabs(n+NSampPerNull);
      end
      [dummy, imin] = min(acc);
      i1st = imin, % first sample of DAB #1 frame w/ NULL
  else
      i1st = 1;
  end

% Second DAB frame
  ADCppmMAX = 400; M = round( 0.5 * ADCppmMAX * NSampPerFrame/fs );
  i2nd = i1st + NSampPerFrame - M;
  acc = zeros(1,2*M); acc(1) = sum( abs( x(i2nd : i2nd + NSampPerNull-1) ) ); 
  for n = 1 : 2*M-1 
      acc(n+1) = acc(n) - abs(x(i2nd+n)) + abs(x(i2nd+n+NSampPerNull));
  end
  [dummy imin] = min(acc); clear xabs;
  i2nd = i2nd + imin,  % first sample of DAB #2 frame /w NULL

% Increase DAB frame counter
  DABFrameFoundNr = DABFrameFoundNr + 1,

% ADC correction of the first DAB frame starting from NULL symbol
  if( CorrectADC )
     NSampPerFrameADC = i2nd - i1st,
     if( NSampPerFrameADC ~= NSampPerFrame ) 
         step = NSampPerFrameADC / NSampPerFrame;
         x = interp1( [0:1:NSampPerFrameADC+NSampPerNull-1]', ...
             x(i1st : i1st+NSampPerFrameADC+NSampPerNull-1), ...
             [0:step:NSampPerFrameADC+NSampPerNull-1]','spline');
     end
  else
     x = x( i1st : i2nd -1 ).'; 
  end
  
% #########################################################################  
% Carrier frequency offset estimation using the PHASE REFERENCE symbol
% #########################################################################  

  i1st = 1 + NSampPerNull;    % skip NULL symbol

  if( CorrectFreqOffset == 0 )
 
    df = 0;

  else

% ESTIMATION of CFO fractional part: df_fract 
  Ncut = 25;             % Assumed max time offset error +/-
  icp   = (i1st+Ncut      : i1st+NSampPerPrefix-Ncut-1)';       % CP indexes 
  isymb = (i1st+Ncut+Nfft : i1st+NSampPerPrefix-Ncut-1+Nfft)';  % SYMBOL indexes
  z =  sum( conj(x( icp )) .* x( isymb ) );
  df_fract = angle(z)/(2*pi),
% df_fract COMPENSATION of the PHASE REFERENCE symbol
  icpsymb  = (i1st : i1st + NSampPerPrefix+Nfft-1)';            % CP+SYMBOL indexes 
  s = x(icpsymb) .* exp(-j*2*pi/Nfft*df_fract*(icpsymb));       % CFO correction

% ESTIMATION of CFO integer part: df_int
% Multiplication in time domain = cyclic convolution (corr) in frequency domain
% Cyclic correlation of signal spectrum with the PhaseRef spectrum --> find maximum
  M = 20;   % M = 10? 20?  max searach region = [-M, +M] around 0 spectrum shift 
  isymb = (NSampPerPrefix+1 : NSampPerPrefix+Nfft)';
  XC = fft( s( isymb ) .* conj( sigPhaseRefSymb( isymb ) ) );
  XC = [ XC(end-M+1:end); XC(1:1+M) ];
  [ val, indx ] = max( XC ); 
  df_int = indx - (M+1), % pause

% SUMMATION   
  df = df_int + df_fract;

  if( CorrectTimeSynchro ) 
     % START: Extra time synchronization using PHASE REFERENCE SYMBOL
     % Frequency offset correction of the Phase Reference Symbol
       s = x( icpsymb ) .* exp(-j*2*pi/Nfft*df*( icpsymb ));
     % TIME OFFSET CALCULATION IN FREQ DOMAIN - correlation in freq domain  
       S = fft( s( isymb ) ); S =S/max(abs(S));
       work = ifft( S .* conj( PhaseRefSymb ) );
       M=7; work=[ work(end-M+1:end); work(1:M+1) ]; % Assumed max +/- 7
       [ dummy indx ] = max(abs(work(1:2*M+1))); 
       ioffsA = indx-(M+1),
     % TIME OFFSET CALCULATION IN TIME DOMAINb   
       M = 7; XC = []; Nsymb = NSampPerSymb;  % Assumed max +/-7
       XC(M+1) = sum( s .* conj(sigPhaseRefSymb) )/Nsymb;
       for m = 1 : M
           XC( M+1+m ) = sum( s(1+m:Nsymb) .* conj(sigPhaseRefSymb(1:Nsymb-m)) )/(Nsymb-m);
       end
       for m = 1 : M
           XC( M+1-m ) = sum( s(1:Nsymb-m) .* conj(sigPhaseRefSymb(1+m:Nsymb)) )/(Nsymb-m);
       end
      [ dummy ioffsB ] = max(abs(XC));
       ioffsB = ioffsB - (M+1),
     % CORRECTION of DAB frame beginning (without NULL)
       ioffs = round( (ioffsA + ioffsB)/2 );
       i1st = i1st + ioffs;
       
  end %  end of CorrectTimeSynchro

  end % end of CorrectFreqOffset

% FINAL CFO CORRECTION OF THE WHOLE LONG DAB FRAME WITHOUT NULL WITH PHASEREF
  SignalOut = x(i1st : i1st + NSampPerFrameSymb-1) .* exp(-j*2*pi/Nfft*df*(0 : NSampPerFrameSymb-1)') ;
  OldData   = Signal(i2nd : end);  % data to be analyzed later
  
  return;  % <==================== % RETURN TO THE MAIN PROGAM

%@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@       
end % of if (sufficient signal length) - sampling frequency correction
%@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

if( PlotON )
end    
    
    