%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Masterthesis
% Zurcher Hochschule fur Angewandte Wissenschaften
% Zentrum fur Signalverarbeitung und Nachrichtentechnik
% (c) Michael Häner-Höin, 12.4.2011 ZSN, info.zsn@zhaw.ch
% Extended:
% AGH University of Science and Technology, Krakow, Poland
% (c) Tomasz Zielinski, 31.12.2019, tzielin@agh.edu.pl
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [ Start, Size, kbps, Protect, ErrProtect ] = ...
                      decodeFIC( DABFrameNr, DataFIC, VitTrellis, DebugOn )

global NFIBsPerFrame NCIFsPerFrame NSymbPerFIC NFIBsPerCIF NCarrPerSymb
global Generator InStates1 InStates2 RefOutValues1 RefOutValues2  % for Viterbi TZ

  % Constants for FIC only
    NSampAfterTimeDepunct = 3096; NSampAfterVit = NSampAfterTimeDepunct/4-6; % 768; 
 
  % Put bits of FIBs corresponding to one CIF in separate row -------------------------------
    NficbitsPer1CIF = NSymbPerFIC*NCarrPerSymb*2/NCIFsPerFrame; % 2304/2304/3072/2304
    FIC = zeros(NCIFsPerFrame, NficbitsPer1CIF);       
    for r=0:NCIFsPerFrame-1
        FIC(r+1,:) = DataFIC(r*NficbitsPer1CIF+1:(r+1)*NficbitsPer1CIF);
    end
    if( DebugOn==1)
        disp('#####'); FICSize = size(FIC),   % NCIFsPerFrame x NFICbitsPer1CIF
        ERROR = FICSize - [ NCIFsPerFrame, NficbitsPer1CIF ], pause,
    end
        
  % De-puncturing ---------------------------------------------------------------------------
    DataDep = [depuncturing(FIC(:,1:2016),16) depuncturing(FIC(:,2017:2292),15) depuncturing(FIC(:,2293:2304),8)];
    if( DebugOn==1)                                      %                 3096/3096/4120/3096
        disp('#####'); FICDataDepSize = size(DataDep),   % NCIFsPerFrame x NSampAfterTimeDepunct
        ERROR = FICDataDepSize - [ NCIFsPerFrame, NSampAfterTimeDepunct ], pause,
    end
        
  % Viterbi decoder -------------------------------------------------------------------------
    DataVit = zeros(NCIFsPerFrame,NFIBsPerCIF*256+6);          % Bitvector + Tail
    for f=1:NCIFsPerFrame
        
      % TEST ONLY  
      % aaa = vitdec( DataDep(f,:), VitTrellis, 1, 'trunc', 'unquant');  % Matlab. +/- real values or {-1,+1}
      % bbb = DABViterbiTZ(DataDep(f,:),Generator,InStates1,InStates2,RefOutValues1,RefOutValues2,0); % T.Z.
      % disp('FIC VITERBI ERROR'); max(abs(aaa-bbb)), pause
        
      % DataVit(f,:) = DABViterbiMH(DataDep(f,:));                                     % M. Hoin, Matlab/Octave
        DataVit(f,:) = DABViterbiTZ(DataDep(f,:),Generator,InStates1,InStates2,RefOutValues1,RefOutValues2,0); % T.Z. Matlab/Octve
      % DataVit(f,:) = vitdec( DataDep(f,:), VitTrellis, 1, 'trunc', 'unquant');       % Matlab. +/- real values or {-1,+1}
      % DataVit(f,:) = vitdec( DataDep(f,:), VitTrellis, 1, 'trunc', 'hard');          % Matlab. binary values {0,1}
    end
    DataVit = DataVit(:,1:end-6);                        % Tail removing
    if( DebugOn==1)
        disp('#####'); FICDataVitSize = size(DataVit),   % NCIFsPerFrame x NSampAfterVit
        ERROR = FICDataVitSize - [ NCIFsPerFrame, NSampAfterVit ], pause,
    end

  % Energy dispersal ------------------------------------------------------------------------
    DataEnerg = zeros(size(DataVit));
    for m=1:NCIFsPerFrame
        DataEnerg(m,:) = xor( DataVit(m,:), EnergyDispGen(NFIBsPerCIF*256) );
    end
    if( DebugOn==1)
        disp('#####'); FICDataEnergSize = size(DataEnerg),   % NCIFsPerFrame x NSampAfterVit
        ERROR = FICDataEnergSize - [ NCIFsPerFrame, NSampAfterVit ], pause,
    end

  % FIBs building ---------------------------------------------------------------------------
    FIB = reshape(DataEnerg',256,NFIBsPerFrame)';
    if( DebugOn==1)
        disp('#####'); FIBSize = size( FIB ),   % NFIBsPerFrame x 256
        ERROR = FIBSize - [ NFIBsPerFrame, 256 ], pause,
    end 
        
  % CRC checking of FIBS --------------------------------------------------------------------
    FIBCRCCheck = zeros(1,NFIBsPerFrame);
    for k=1:NFIBsPerFrame
        FIBCRCCheck(k) = CRC16(FIB(k,:));
    end    
    if( DebugOn==1) FIBCRCCheckSize = size(FIBCRCCheck), pause, end
        
    if sum(FIBCRCCheck) == NFIBsPerFrame % 12?
       disp('CRC OK');                   % FIGs building from FIBS 
       for FIBNr = 1:NFIBsPerFrame       % to 12
           pos = 1;
           while pos < 241
                 if FIB(FIBNr,pos:pos+7) == [1 1 1 1 1 1 1 1], break, end
               % FIG type and length finding
                 Type = BinToDec( FIB(FIBNr,pos:pos+2), 3 );
                 FIGDataLength = BinToDec( FIB(FIBNr,pos+3:pos+7), 5 ); % in bytes
               % FIG building (reconstruction)
                 FIGType( DABFrameNr, FIBNr, Type, FIGDataLength, FIB(FIBNr,pos+8:pos+8+8*FIGDataLength-1) )
                 pos = pos + (1+FIGDataLength)*8;
           end
       end
       disp('################################## Ready !');
       disp('PRESS ANY KEY !'); % pause
    else
       disp(['CRC Fail! Frame: ',num2str( DABFrameNr )]);
    end
        
  % To be changed / done  
    Start = 0; Size = 0; kbps = 0; Protect = ['3A']; ErrProtect = true;
