%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Masterthesis
% Zurcher Hochschule fur Angewandte Wissenschaften
% Zentrum fur Signalverarbeitung und Nachrichtentechnik
% (c) Michael Hoin, 12.4.2011 ZSN, info.zsn@zhaw.ch
% Extended:
% AGH University of Science and Technology, Krakow, Poland
% (c) Tomasz Zielinski, 31.12.2019, tzielin@agh.edu.pl
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% -------------------------------------------------------------------------
% FFT of all OFDM symbols in one TX DAB frame, D-QPSK demodulation
% On output OFDM symbols are in consecutive rows.
% -------------------------------------------------------------------------

function [ DX ] = demodulateDAB_Short( x, OFDMOffset, PlotON )

global NSymbPerFrame NSampPerSymb NCarrPerSymb Nfft NSampPerPrefix

Ncp = NSampPerPrefix; Ncar = NCarrPerSymb;  % shorter
offs = 0;                                   % take a few last CP samples
X = zeros( [NSymbPerFrame, Ncar] );         % creating empty array for all FFTs
for nsymb = 1 : NSymbPerFrame               % processing all OFDM symbols
   i = (nsymb-1) * NSampPerSymb;            % index of the first sample
   s = x( 1+i : Ncp+Nfft+i );               % samples of one whole OFDM symbol
   s = s( Ncp+1-offs : Ncp+Nfft-offs );     % take only Nfft part minus offs
   S = fft( s );                            % FFT of selected samples
   S = fftshift( S );                       % negative frequencies to the left
   X( nsymb, 1 : Ncar/2 )      = S( Nfft/2-Ncar/2+1 : Nfft/2 );   % store to FFT array
   X( nsymb, Ncar/2+1 : Ncar ) = S( Nfft/2+2 : Nfft/2+Ncar/2+1 ); % used carriers only
end
DX = X( 2:NSymbPerFrame, : ) .* conj( X( 1:NSymbPerFrame-1, :) ); % phase shift

if( PlotON )
end    
