function x = disturbDAB( x )
% AGH University of Science and Technology, Krakow, Poland
% (c) Tomasz Zielinski, 31.12.2019, tzielin@agh.edu.pl

   N = length(x);
% Parameter values  
  fs = 2.048e6;           % sampling frequency used in the DAB algorithm
  cfo = 100;    % 1234;   % carrier frequency offset (CFO) [Hz]
  npwr = -40;   % -40;    % noise power [dB]: -160, ...,-60,-50,-40,-30,-20
  ADCppm = 50;  %  50;    % ADC error [ppm]: from -400 to +400
  G0=0.5; G1=0.1; G2=0.02; D0=0; D1=10; D2=20;  % Multi-path: gains, delays

  % Disturbing signal
  NADC = round((1+ADCppm/1000000)*N); step = N/NADC; % ADC error ----------
  if( step ~= 1 )
      disp('ADC ERROR -> generation !');
      NADC = length(0:step:N-step);
      x(1:NADC,1)=interp1( [0:N-1]', x(1:N,1), [0:step:N-step]', 'spline' );
      N = NADC; %x(NADC+1:end,1)=[];
  end
  x = G0*[ x(end-D0+1:end,1); x(1:end-D0,1) ] + ...       % multipath ---------- 
      G1*[ x(end-D1+1:end,1); x(1:end-D1,1) ] + ...
      G2*[ x(end-D2+1:end,1); x(1:end-D2,1) ];
  x = x .* exp( j*2*pi*cfo/fs*(0:N-1).');                     % CFO -------
  x = x + 10^(npwr/20) / sqrt(2) * (randn(N,1)+j*randn(N,1)); % AWGN ------
