
% ---------------------------------------------------
% CALLING FUNCTIONS - generation of different tables
% ---------------------------------------------------  
% Generation of a Pseudo-Random Binary Sequence (PRBS), used in energy de-dispersal (ETSI pp. 128-129)
  DispSequense = EnergyDispGen( 100000 );
% Generation of a Frequency Inteleaving and Frequency De-inteleaving Tables (ETSI pp. 157-161)
  [FreqInterleaverTab]   = FreqInterleaverTabGen( Nfft );                       
  [FreqDeinterleaverTab] = FreqDeinterleaverTabGen( FreqInterleaverTab, Nfft );
% Generation of Phase Reference Symbol (used for frame synchro) (ETSI pp. 147-149)
  [PhaseRefSymb, sigPhaseRefSymb] = PhaseRefGen( Plot_ON );
% Generation of trellis for polynomial (ETSI, page 129-130, below fig. 72)
  VitTrellis = poly2trellis(7,[133 171 145 133]); % 7 = delay of each bitstream, then octal numbers
% Generation of matrices for OUR Viterbi decoder (SLOW!) working in Matlab and Octave
% [ Generator, Transition, Ref1, Ref2 ] = DABViterbiTZinit();
  [ Generator, InStates1, InStates2, RefOutValues1, RefOutValues2 ] = DABViterbiTZinit();
% Generation of polynomial for Reed-Solomon decoder being part of DAB+ (ETSI 2010, page 15-16)
  RSpolynomial = rsgenpoly(255,245,285,0); % 285 = 100011101 = x^8+x^4+x^3+x^2+1
                                               
% ##########################################################################################
% These parameters should be set AFTER! FIC decoding and choosing a broadcast by a listener!
% ##########################################################################################

% Distinguishing between DAB and DAB+ programme!
% Depends on type of a channel to be decoded and played.
  if( UEP == 1 )
    % DAB: Parameters for Unequal Error Protection (UEP), used in depuncturing, pages 133-135, table 31
      [L1Range, L2Range, L3Range, L4Range, PI1, PI2, PI3, PI4, Padding] = UEPTab(Audiokbps, Protect);
    %    836,    2012,    5184,     5364,  11,  5,   6,   7,   0        % Example  
  else
    % DAB+: Parameters for Equal Error Protection (EEP), used in depuncturing, pages 135-136
      [Audiokbps, PI1, PI2, L1Range, L2Range] = EEPTab(Protect, SubChSize); % pause
      PI3=0; PI4=0; L3Range = 0; L4Range = 0; Padding=0;
  end
% Buffer for time-interleaved MSC data (ETSI, pp. 137-142)
% buffer used for MSC time interleaving: 16frames, each 4CIFs with 64bits    
  Delay = 4*SubChSize;                                              % Example: SubChSize=84, Delay=336
  DataLastRound = zeros( 16, 16*Delay + (NCIFsPerFrame-1)*Delay );  % Example: NCIFs=4 -> 16 x 6384 

% #################################
% CALCULATION DIMENSION OF MATRICES
% #################################  
  if( UEP == 1 )
      v1 = PIDet( PI1 );                          % puncturing 0/1 32-element vector
      NBits1 = L1Range           * 32/sum(v1);    % bit number after depuncturing
      v2 = PIDet( PI2 );                          % puncturing 0/1 32-element vector
      NBits2 = (L2Range-L1Range) * 32/sum(v2);    % bit number after depuncturing
      v3 = PIDet( PI3 );                          % puncturing 0/1 32-element vector
      NBits3 = (L3Range-L2Range) * 32/sum(v3);    % bit number after depuncturing
      v4 = PIDet( PI4 );                          % puncturing 0/1 32-element vector
      NBits4 = (L4Range-L3Range) * 32/sum(v4);    % bit number after depuncturing
      v5 = PIDet( 8 );                            % puncturing 0/1 32-element vector
      NBits5 = 12                * 32/sum(v5);
      NBitsAfterTimeDepunct = NBits1 + NBits2 + NBits3 + NBits4 + NBits5;
  else
      v1 = PIDet( PI1 );                          % puncturing 0/1 32-element vector
      NBits1 = L1Range           * 32/sum(v1);    % bit number after depuncturing
      v2 = PIDet( PI2 );                          % puncturing 0/1 32-element vector
      NBits2 = (L2Range-L1Range) * 32/sum(v2);    % bit number after depuncturing
      v3 = PIDet( 8 );                            % puncturing 0/1 32-element vector
      NBits3 = 12                * 32/sum(v3);
      NBitsAfterTimeDepunct = NBits1 + NBits2 + NBits3;
  end
