% lab23_ex_dab_all.m - with simpler DAB synchronization version

% WARNING!
% In comparison with the program lab23_ex_dab_fic_msc.m
% there is no adaptive smoothing of synchronization parameters
% and worse program performance is observed!
%
% Masterthesis
% Zurcher Hochschule fur Angewandte Wissenschaften
% Zentrum fur Signalverarbeitung und Nachrichtentechnik
% (c) Michael Häner-Höin, 12.4.2011 ZSN, info.zsn@zhaw.ch
% Extended:
% AGH University of Science and Technology, Krakow, Poland
% (c) Tomasz Zielinski, 31.12.2019, tzielin@agh.edu.pl

  clc; clear all; close all; fclose('all');

% --------------------------
% Files: IN and OUT
% --------------------------

% DAB+ recordings: https://sdr.kt.agh.edu.pl/sdrdab-decoder/download.html
  ReadFile = 'DAB_PolishRadio_TX_IQ.dat'; sigtype=1; fs_file = 2.048e+6;  % synthesized
% ReadFile = 'DAB_PolishRadio_RX_IQ.wav'; sigtype=2; fs_file = 2.048e+6;  % recorded, DISTURBED!
  
  Nframes = 100;                   % number of DAB frames to read    
  if(0) SubChStartAdr = 0;   SubChSize = 84; Audiokbps=0; Protect=['3A']; UEP=0; end
  if(1) SubChStartAdr = 84;  SubChSize = 96; Audiokbps=0; Protect=['3A']; UEP=0; end
  if(0) SubChStartAdr = 180; SubChSize = 84; Audiokbps=0; Protect=['3A']; UEP=0; end
  if(0) SubChStartAdr = 264; SubChSize = 84; Audiokbps=0; Protect=['3A']; UEP=0; end
  if(0) SubChStartAdr = 348; SubChSize = 48; Audiokbps=0; Protect=['3A']; UEP=0; end
  if(0) SubChStartAdr = 396; SubChSize = 48; Audiokbps=0; Protect=['3A']; UEP=0; end
  if(0) SubChStartAdr = 444; SubChSize = 72; Audiokbps=0; Protect=['3A']; UEP=0; end
  if(0) SubChStartAdr = 516; SubChSize = 78; Audiokbps=0; Protect=['3A']; UEP=0; end

  WriteFile = 'AudioDAB+.aac';    % audio output

% --------------------------
% PROGRAM CONTROL PARAMETERS
% --------------------------

  DecodeFIC_ON = 0;          % 0/1 decoding FIC information about available services
  DecodeMSC_ON = 1;          % 0/1 decoding MSC content of radio audio service
  SuperFrame2AAC_ON = 1;     % 0/1 DAB+ to AAC transcoding in this program
  CorrectADC_ON = 1;         % 0/1 CORRECT ADC sampling frequency by resampling 
  CorrectFreqOffset_ON = 1;  % 0/1 CORRECT carrier frequency offset (CFO)
  CorrectTimeSynchro_ON = 1; % 0/1 CORRECT time synchro after CFO correction
  Disturbances_ON = 0;       % 0/1 ADD disturbances
  Plot_ON = 0;               % 0/1 plot figures
  
% ----------------------
% GENERAL DAB PARAMETERS
% ----------------------

  fs = 2.048e6;            % sampling frequency used in the DAB algorithm

  NSymbPerFrame = 76;      % L, number of OFDM symbols per one DAB frame
  NCarrPerSymb  = 1536;    % K, number of transmitted carriers
  NSampPerFrame = 196608;  % Tf, duration of the transmission frame (in samples)
  NSampPerNull  = 2656;    % TNULL, duration of the Null symbol (in samples)
  NSampPerSymb  = 2552;    % Ts, duration of OFDM symbols with indices: l=1...L
  Nfft          = 2048;    % Tu, inverse of the carrier spacing
  NSampPerPrefix = 504;    % delta, duration of the guard interval

  NFIBsPerFrame = 12;      % (FIC) number of FIBs per one DAB frame (non-interleaved)
  NCIFsPerFrame =  4;      % (MSC) number of CIFs per one DAB frame (time-interleaved)
  NSymbPerFIC   =  3;      % (FIC) number of OFDM frames per one FIC
  NFIBsPerCIF   =  3;      % (FIC) number of FIBs per one CIF

  NSampPerFrameSymb = NSymbPerFrame*NSampPerSymb; % DAB frame length wihout NULL

% ----------------------
% TABLES INITIALIZATION
% ----------------------

% Generation of Phase Reference Symbol (used for frame synchro) (ETSI pp. 147-149)
  [PhaseRefSymb, sigPhaseRefSymb] = PhaseRefGen( 0 );
% Generation of a Frequency Inteleaving and De-inteleaving Tables (ETSI pp. 157-161)
  [FreqInterleaverTab]   = FreqInterleaverTabGen( Nfft );                       
  [FreqDeinterleaverTab] = FreqDeinterleaverTabGen( FreqInterleaverTab, Nfft );
% Generation of trellis for polynomial (ETSI, page 129-130, below fig. 72)
  VitTrellis = poly2trellis(7,[133 171 145 133]); % 7 = delay, then octal numbers
% Generation of matrices for OUR Viterbi (not Matlab) working in Matlab/Octave
% [ Generator, Transition, Ref1, Ref2 ] = DABViterbiTZinit();
  [ Generator, InStates1, InStates2, RefOutValues1, RefOutValues2 ] = DABViterbiTZinit();
% Generation of a Pseudo-Random Binary Sequence (PRBS), used in energy de-dispersal (ETSI pp. 128-129)
  DispSequense = EnergyDispGen( 100000 );
% Generation of polynomial for Reed-Solomon decoder being part of DAB+ (ETSI 2010, page 15-16)
  RSpolynomial = rsgenpoly(255,245,285,0); % 285 = 100011101 = x^8+x^4+x^3+x^2+1

% ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
% These parameters are set when Radio Station is already chosen!
% ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

% Distinguishing between DAB and DAB+ programme!
% Depends on type of a channel to be decoded and played.
  if( UEP == 1 )
    % DAB: Parameters for Unequal Error Protection (UEP), used in depuncturing, pages 133-135, table 31
      [L1Range, L2Range, L3Range, L4Range, PI1, PI2, PI3, PI4, Padding] = UEPTab(Audiokbps, Protect);
    %    836,    2012,    5184,     5364,  11,  5,   6,   7,   0        % Example  
  else
    % DAB+: Parameters for Equal Error Protection (EEP), used in depuncturing, pages 135-136
      [Audiokbps, PI1, PI2, L1Range, L2Range] = EEPTab(Protect, SubChSize); % pause
      PI3=0; PI4=0; L3Range = 0; L4Range = 0; Padding=0;
  end
% Buffer for time-interleaved MSC data (ETSI, pp. 137-142)
% buffer used for MSC time interleaving: 16frames, each 4CIFs with 64bits    
  Delay = 4*SubChSize;                                              % Example: SubChSize=84, Delay=336
  DataLastRound = zeros( 16, 16*Delay + (NCIFsPerFrame-1)*Delay );  % Example: NCIFs=4 -> 16 x 6384 

% +++++++++++++++++++++++++++++++++++++
% CALCULATION OF DIMENSIONS OF MATRICES
% +++++++++++++++++++++++++++++++++++++  
  if( UEP == 1 )
      v1 = PIDet( PI1 );                          % puncturing 0/1 32-element vector
      NBits1 = L1Range           * 32/sum(v1);    % bit number after depuncturing
      v2 = PIDet( PI2 );                          % puncturing 0/1 32-element vector
      NBits2 = (L2Range-L1Range) * 32/sum(v2);    % bit number after depuncturing
      v3 = PIDet( PI3 );                          % puncturing 0/1 32-element vector
      NBits3 = (L3Range-L2Range) * 32/sum(v3);    % bit number after depuncturing
      v4 = PIDet( PI4 );                          % puncturing 0/1 32-element vector
      NBits4 = (L4Range-L3Range) * 32/sum(v4);    % bit number after depuncturing
      v5 = PIDet( 8 );                            % puncturing 0/1 32-element vector
      NBits5 = 12                * 32/sum(v5);
      NBitsAfterTimeDepunct = NBits1 + NBits2 + NBits3 + NBits4 + NBits5;
  else
      v1 = PIDet( PI1 );                          % puncturing 0/1 32-element vector
      NBits1 = L1Range           * 32/sum(v1);    % bit number after depuncturing
      v2 = PIDet( PI2 );                          % puncturing 0/1 32-element vector
      NBits2 = (L2Range-L1Range) * 32/sum(v2);    % bit number after depuncturing
      v3 = PIDet( 8 );                            % puncturing 0/1 32-element vector
      NBits3 = 12                * 32/sum(v3);
      NBitsAfterTimeDepunct = NBits1 + NBits2 + NBits3;
  end

% #########################################################################
% MAIN LOOP - START #######################################################
% #########################################################################

% Open files
  if( sigtype==1) fin = fopen( ReadFile, 'rb');   % open IQ file with DAB
  else            n1st = 1;                       % counter for WAV
  end                                             %   
  fout = fopen( WriteFile, 'wb');                 % open output audio file
  ReadSize  = 3*NSampPerFrame;                    % number of samples to read

% DAB+ Super Frame initialization
  SuperFrameSync = false;     % is Audio Super Frame synchronised? (false/true)
  CIFCount = 0;               % CIF counter on output of Energy Dispersal
  au_start = 0;               % starting position in synchronous access unit
  num_aus = 0;                % number of audiopacketes in one Audio Super Frame
  AllAudio = [];              % variable for extracted audiofile

% General parameters 
  DABFrameNr = 0;             % set DAB frame counter
  OldData = [];               % previous IQ samples

  while ( DABFrameNr <= Nframes )
  
 % #########################
 % READIND DATA IF NECESSARY
 % #########################

  [NOldSamples,dummy] = size( OldData );  
  if( NOldSamples < 3*NSampPerFrame)
      disp('RRRRRRRRRRRRRRRRRRRRRRRRRRRR Reading new data from disc ...');
      if( sigtype==1)    % synthesized
          try    w = fread( fin, [2, ReadSize], 'float' );
          catch  break;
          end    
          w = w(1,:) + j*w(2,:); w = w.' / 2^15;
      end
      if( sigtype==2 )   % recorded
          try    [w, fs] = audioread( ReadFile, [n1st, n1st+ReadSize-1] );
          catch  break;
          end    
          w = w(:,1) - j*w(:,2);
          n1st = n1st + ReadSize;
      end
      
      if( fs_file ~= fs )         % obligatory when different sampling frequency 
          w = resample(w, fs/1e3, fs_file/1e3);  % resampling from fs_file to fs
      end
      
    % +++++++++++++++++++++++++++++++++  
    % Optional addition of disturbances
    % +++++++++++++++++++++++++++++++++  
      if( Disturbances_ON ) w = disturbDAB( w ); end
      
  else
      w = [];  % nothing should be read
  end
  if( Plot_ON )
      figure; plot(real(w)); xlabel('n'); title('I(n)'); grid; pause
  end
  
  Data = [ OldData; w ];
  x = Data;

% #########################
% Detection of NULL symbols
% #########################

  if( DABFrameNr == 0 )
    % First DAB frame
      xabs = abs(x);
      NSampPerSynchro = NSampPerFrame + NSampPerNull;
      acc = zeros(1,NSampPerSynchro); acc(1) = sum( xabs(1:NSampPerNull) );
      for n = 1 : NSampPerSynchro; 
          acc(n+1) = acc(n) - xabs(n) + xabs(n+NSampPerNull);
      end
      [dummy, imin] = min(acc);
      i1st = imin, % first sample of DAB #1 frame w/ NULL
  else
      i1st = 1;
  end

% Second/next DAB frame
  ADCppmMAX = 400; M = round( 0.5 * ADCppmMAX * NSampPerFrame/fs );
  i2nd = i1st + NSampPerFrame - M;
  acc = zeros(1,2*M+1); acc(1) = sum( abs( x(i2nd : i2nd + NSampPerNull-1) ) ); 
  for n = 1 : 2*M 
      acc(n+1) = acc(n) - abs( x(i2nd+n) ) + abs( x(i2nd+n+NSampPerNull) );
  end
  [dummy imin] = min(acc); clear xabs;
  i2nd = i2nd + imin,                  % first sample of DAB #2 frame /w NULL
  
  OldData   = Data( i2nd : end, 1 );   % data to be analyzed later
  DABFrameNr = DABFrameNr + 1;         % increment DAB frame counter     

% ADC correction of the first DAB frame starting from NULL symbol
  if( CorrectADC_ON )
     NSampPerFrameADC = i2nd - i1st,
     if( NSampPerFrameADC ~= NSampPerFrame ) 
         step = NSampPerFrameADC / NSampPerFrame;           % resampling
         x = interp1( [0:1:NSampPerFrameADC+NSampPerNull-1]', ...
             x(i1st : i1st+NSampPerFrameADC+NSampPerNull-1), ...
             [0:step:NSampPerFrameADC+NSampPerNull-1]','spline');
     end
  else
     x = x( i1st : i2nd-1 );   % starting from the beginning of NULL symbol
  end
  
% ####################################################################  
% Carrier frequency offset estimation using the PHASE REFERENCE symbol
% ####################################################################  

  i1st = 1 + NSampPerNull;    % skip NULL symbol with index, stop at CP

  if( CorrectFreqOffset_ON == 0 )
 
    df = 0;

  else

% ESTIMATION of CFO fractional part: df_fract 
  Ncut = 25;             % Assumed max time offset error +/-
  icp   = (i1st+Ncut      : i1st+NSampPerPrefix-Ncut-1)';       % CP indexes 
  isymb = (i1st+Ncut+Nfft : i1st+NSampPerPrefix-Ncut-1+Nfft)';  % SYMBOL indexes
  z =  sum( conj(x( icp )) .* x( isymb ) );
  df_fract = angle(z)/(2*pi),
% df_fract COMPENSATION of the PHASE REFERENCE symbol
  icpsymb  = (i1st : i1st + NSampPerPrefix+Nfft-1)';            % CP+SYMBOL indexes 
  s = x(icpsymb) .* exp(-j*2*pi/Nfft*df_fract*(icpsymb));       % CFO correction

% ESTIMATION of CFO integer part: df_int
% Multiplication in time domain = cyclic convolution (corr) in frequency domain
% Cyclic correlation of signal spectrum with the PhaseRef spectrum --> find maximum
  M = 20;   % M = 10? 20?  max searach region = [-M, +M] around 0 spectrum shift 
  isymb = (NSampPerPrefix+1 : NSampPerPrefix+Nfft)';
  XC = fft( s( isymb ) .* conj( sigPhaseRefSymb( isymb ) ) );
  XC = [ XC(end-M+1:end); XC(1:1+M) ];
  [ val, indx ] = max( XC ); 
  df_int = indx - (M+1), % pause

% SUMMATION   
  df = df_int + df_fract;

  if( CorrectTimeSynchro_ON ) 
     % START: Extra time synchronization using PHASE REFERENCE SYMBOL
     % Frequency offset correction of the Phase Reference Symbol
       s = x( icpsymb ) .* exp(-j*2*pi/Nfft*df*( icpsymb ));
      % TIME OFFSET CALCULATION IN FREQ DOMAIN - mult in freq -> conv/corr in time  
     % s = circshift(sref,-5); % for test only
       S = fft( s( isymb ) );
       work = ifft( S .* conj( PhaseRefSymb ) ) / NSampPerSymb; % ifft( mult in freq )
       M=7; work=[ work(end-M+1:end); work(1:M+1) ]; % Assumed max +/- 7
       [ dummy indx ] = max(abs(work(1:2*M+1))); 
       ioffsA = indx-(M+1),
     % TIME OFFSET CALCULATION IN TIME DOMAIN - direct correlation in time  
       M = 7; XC = []; Nsymb = NSampPerSymb;  % Assumed max +/-7
       XC(M+1) = sum( s .* conj(sigPhaseRefSymb) )/Nsymb;
       for m = 1 : M
           XC( M+1+m ) = sum( s(1+m:Nsymb) .* conj(sigPhaseRefSymb(1:Nsymb-m)) )/(Nsymb-m);
           XC( M+1-m ) = sum( s(1:Nsymb-m) .* conj(sigPhaseRefSymb(1+m:Nsymb)) )/(Nsymb-m);
       end
      [ dummy ioffsB ] = max(abs(XC));
       ioffsB = ioffsB - (M+1),
     % CORRECTION of DAB frame beginning (without NULL)
       ioffs = round( (ioffsA + ioffsB)/2 );
       i1st = i1st + ioffs;
 
  end %  end of CorrectTimeSynchro

  end % end of CorrectFreqOffset

% FINAL CFO CORRECTION OF THE WHOLE LONG DAB FRAME WITHOUT NULL WITH PHASEREF
  x = x(i1st : i1st + NSampPerFrameSymb-1) .* exp(-j*2*pi/Nfft*df*(0 : NSampPerFrameSymb-1)') ;

% ##################
% FFT ##############
% ##################

  Ncp = NSampPerPrefix; Ncar = NCarrPerSymb;  % shorter names
  offs = 0;                                   % take a few last CP samples
  Y = zeros( [NSymbPerFrame, Ncar] );         % creating empty array for all FFTs
  for nsymb = 1 : NSymbPerFrame               % processing all OFDM symbols
    i = (nsymb-1) * NSampPerSymb;             % index of the first sample
    s = x( 1+i : Ncp+Nfft+i );                % samples of one whole OFDM symbol
    s = s( Ncp+1-offs : Ncp+Nfft-offs );      % take only Nfft part minus offs
    S = fft( s );                             % FFT of selected samples
    S = fftshift( S );                        % negative frequencies to the left
    X( nsymb, 1 : Ncar/2 )      = S( Nfft/2-Ncar/2+1 : Nfft/2 );    % store to FFT array
    X( nsymb, Ncar/2+1 : Ncar ) = S( Nfft/2+2 : Nfft/2+Ncar/2+1 );  % used carriers only
  end
  FFTFrame = X( 2:NSymbPerFrame, : ) .* conj( X( 1:NSymbPerFrame-1, :) ); % phase shift
  
  W = FFTFrame;
  
  if( Plot_ON )
    figure; plot(angle(W(:,:)),'b*'); xlabel('symbol k'); title('Phase Shifts'); grid; pause
    figure; plot(real(W(:,100)),imag(W(:,100)),'b*'); xlabel('I'); title('Q(k)=f(I(k))'); grid; pause
  end
  
% Frequency de-interleaver
  DeintFFTFrame = zeros( size(FFTFrame) );
  for r = 1 : length( FreqDeinterleaverTab(:,4) )                               
      DeintFFTFrame( :, FreqDeinterleaverTab(r,4) ) = FFTFrame(:,r);
  end

% ##################
  if( DecodeFIC_ON )
% ##################

% Constants for FIC
  NSampAfterTimeDepunct = 3096; NSampAfterVit = NSampAfterTimeDepunct/4-6; % 768; 

% D-QPSK demodulation: complex --> real, imag
% D-QPSK symbol demapper - from 2D matrix to 1D vector, ETSI pp. 157 &&&&&&
  DataFIC = zeros(1, NSymbPerFIC*NCarrPerSymb*2);                  % 3*1536*2
  for f = 1 : NSymbPerFIC % FIC DATA from OFDM symbols, first real, then imag 
      DataFIC( 1+(f-1)*NCarrPerSymb*2 : (f)*NCarrPerSymb*2 ) = ...
                     [ real(DeintFFTFrame(f,:)) imag(DeintFFTFrame(f,:)) ];
  end
 
% Put bits of FIBs corresponding to one CIF in separate row -----------------------
  NficbitsPer1CIF = NSymbPerFIC*NCarrPerSymb*2/NCIFsPerFrame; % 2304/2304/3072/2304
  FIC = zeros(NCIFsPerFrame, NficbitsPer1CIF);       
  for r = 0:NCIFsPerFrame-1
      FIC(r+1,:) = DataFIC( r*NficbitsPer1CIF+1 : (r+1)*NficbitsPer1CIF );
  end
        
% De-puncturing using special funcion --------------------------------------------
  DataDep = [ depuncturing( FIC(:,1:2016),16 ) depuncturing( FIC(:,2017:2292),15 ) ...
              depuncturing( FIC(:,2293:2304),8 )];
        
 % Viterbi decoder using Matlab function -----------------------------------------
   DataVit = zeros(NCIFsPerFrame,NFIBsPerCIF*256+6);  % Bitvector + Tail
   for f=1:NCIFsPerFrame                              % Matlab. +/-real or {-1,+1}
      % DataVit(f,:) = vitdec( DataDep(f,:), VitTrellis, 1, 'trunc', 'unquant');  % Matlab  
        DataVit(f,:) = DABViterbiTZ(DataDep(f,:),Generator,InStates1,InStates2,RefOutValues1,RefOutValues2,0); % TZ Matlab, Octave          
   end
    DataVit = DataVit(:,1:end-6);                     % Tail removing

% Energy dispersal using special function ---------------------------------
  DataEnerg = zeros(size(DataVit));
  for m=1:NCIFsPerFrame
      DataEnerg(m,:) = xor( DataVit(m,:), EnergyDispGen( NFIBsPerCIF*256 ) );
  end

% FIBs building -----------------------------------------------------------
  FIB = reshape( DataEnerg', 256,NFIBsPerFrame )';
        
% CRC checking of FIBS ----------------------------------------------------
  FIBCRCCheck = zeros(1,NFIBsPerFrame);
  for k = 1 : NFIBsPerFrame
      FIBCRCCheck(k) = CRC16( FIB(k,:) );  % special function
  end    
  if sum(FIBCRCCheck) == NFIBsPerFrame     % 12?
     disp('CRC OK');                       % FIGs building from FIBS 
     for FIBNr = 1  : NFIBsPerFrame        % to 12
         pos = 1;
         while pos < 241
               if FIB(FIBNr,pos:pos+7) == [1 1 1 1 1 1 1 1], break, end
             % FIG type and length finding
               Type = BinToDec( FIB(FIBNr,pos:pos+2), 3 );
               FIGDataLength = BinToDec( FIB(FIBNr,pos+3:pos+7), 5 ); % in bytes
             % FIG building (reconstruction) using special function
               FIGType( DABFrameNr, FIBNr, Type, FIGDataLength, ...
                        FIB(FIBNr, pos+8:pos+8+8*FIGDataLength-1) )
               pos = pos + (1+FIGDataLength)*8;
         end
     end
     disp('################################## Ready !');
   % disp('PRESS ANY KEY !'); % pause
  else
     disp(['CRC Fail! Frame: ',num2str( DABFrameNr )]);
  end

% ##################
  end % of DecodeFIC
% ##################

% ##################
  if( DecodeMSC_ON )
% ##################

   % D-QPSK symbol demapper - from 2D matrix to 1D vector ----------------------------
   % ETSI pp. 157
     DataMSC = zeros(1,(NSymbPerFrame-NSymbPerFIC-1)*NCarrPerSymb*2); % 76-3-1 symbols
     for f = NSymbPerFIC+1 : NSymbPerFrame-1 % MSC DATA from OFDM symbols - first real 
         ff = f - NSymbPerFIC;
         DataMSC( 1+(ff-1)*NCarrPerSymb*2 : (ff)*NCarrPerSymb*2 ) = ...
                  [real(DeintFFTFrame(f,:)) imag(DeintFFTFrame(f,:))]; % real +/-
     end
     
   % Building convolutional codeword for each CIF (NCIFsPerFrame=4/1/1/2 for Mode=1/2/3/4)
     NData = length(DataMSC); NCIF=55296;   % Ndata = (NSymbPerFrame-NCIFsPerFrame-1) * NCarrPerSymb*2bits = 221184 bits (for Mode=1)
     MSC = zeros( NCIFsPerFrame, NCIF );    % NCIF = 864 * 64bits (1CU) = 55296 bits = one CIF = 24 msec        
     for r = 0 : (NCIFsPerFrame-1)          % put CIF codewords in rows; for Mode 1/2/3/4 = 4/1/1/2 CIFs (rows) 
         MSC(r+1, :) = DataMSC( 1 + r*NCIF : NCIF + r*NCIF );
     end
           
   % Extracting data of one sub-channel requested by the user (CU=64bits = the smallest addressable unit)----
     SubChData(1:NCIFsPerFrame, :) = MSC(:, 64*SubChStartAdr+1 : 64*SubChStartAdr + 64*SubChSize ); % from 4/1/1/2 CIFs
   % Here we have a matrix with dimension NCIFsPerFrame x N SubChan bits in one CIF
       
   % Time de-interleaving: output = rows: NCIFsPerFrame, columns: SubChSize*64bits(1CU) = 84*64 = 5376 (our example) 
     [TimeDeintSubChData, DataLastRound] = TimeDeinterleaving( SubChData, SubChSize, DataLastRound, 0 );
   
   % ######################################  % Continue decoding only when the buffer
     if( DABFrameNr * NCIFsPerFrame >  16 )  % of time-interleaved data is already filled
   % ######################################  % with 16 CIFs (1 CIF = 24ms = 55296 samples)

   % -----------------------------------------------------------------------------------------
   % DAB or DAB+
   % -----------------------------------------------------------------------------------------
        
     if UEP == 1 % DAB
      
      % (De)puncturing: different parts of CIFs are protected in different manner -------------------------------------
      %  Output = rows: NCIFsPerFrame, cols = 12312 (our example) WHY? GENERAL RULE?
         DataDep = [ depuncturing( TimeDeintSubChData(:,1:L1Range),            PI1 )...  % part 1: 4 rows x  836 cols
                     depuncturing( TimeDeintSubChData(:,L1Range+1:L2Range),    PI2 )...  % part 2: 4 x 1176 =837...2012
                     depuncturing( TimeDeintSubChData(:,L2Range+1:L3Range),    PI3 )...  % part 3: 4 x 3172 =2013...5184
                     depuncturing( TimeDeintSubChData(:,L3Range+1:L4Range),    PI4 )...  % part 4: 4 x 180  =5185...5364
                     depuncturing( TimeDeintSubChData(:,L4Range+1:L4Range+12), 8   ) ];  % tail:   4 x 12   =5365...5376

       % Viterbi decoder, CIF after CIF, removing 4x redundancy: (xn0,xn1,xn2,xn3) (punctured) --> xn ---------------------
         DataVit = zeros(length(DataDep(:,1)),length(DataDep(1,:))/4+1); % rows=NCIFsPerFrame, cols/4 - 4x stream reduction
         for f=1:length(DataDep(:,1)) % each CIF
           % DataVit(f,:) = vitdec( [DataDep(f,:) 0 0 0 0], VitTrellis, 1, 'cont', 'unquant');  % each 4 samples--> 1 sample; Matlab 
             DataVit(f,:) = DABViterbiTZ( [DataDep(f,:) 0 0 0 0],Generator,InStates1,InStates2,RefOutValues1,RefOutValues2,1); % TZ Matlab Octave
         end
         DataVit = DataVit(:, 2:end-6);      % tail removing (6 last numbers); in our example: 12312/4-6=3078-6=3072 columns
          
       % Energy de-dispersal(XOR with generated PRBS sequence); for our example input has NCIFsPerFrame=4 rows, 3072 columns
         DataEnerg = zeros(size(DataVit));
         for m=1:length(DataEnerg(:,1))
             DataEnerg(m,:) = xor( DataVit(m,:), DispSequense(1:length(DataEnerg(1,:))) );    % 
         end
       
       % Audio data creation (from bits to bytes): scan by rows (CIF by CIF), write bits into 8-element rows,
       % inner product with vertical weights 2^p, tranpose to horizontal orientation
         Audio=( reshape( DataEnerg',8,[] )' * [128 64 32 16 8 4 2 1]' )';  % 1536 8-bit numbers = NCIFsPerFrame(4)*3072 bits / 8 bits
           
       % Write to a file (bytes to a file) --------------------------------
       % if DABFrameNr >= 6
            disp('WWWWWWWWWWWWWWWWWWWWWWWWWWWW Audio MP2 written to HD!');
            fwrite(fid, Audio, 'uint8');
       % end
           
     else % DAB+
           
       % (De)puncturing ---------------------------------------------------
                                                                                                % 4x  1536  3072  6144  5376  4608  4992
          DataDep = [ depuncturing( TimeDeintSubChData(:,1:L1Range),            PI1)...         % 4x  1344  2880  5952  5184  4416  4800
                      depuncturing( TimeDeintSubChData(:,L1Range+1:L2Range),    PI2)...         % 4x   180   180   180   180   180   180 
                      depuncturing( TimeDeintSubChData(:,L2Range+1:L2Range+12), 8) ];  % Tail   % 4x    12    12    12    12    12    12
          
       % Viterbi Decoder --------------------------------------------------
         DataVit = zeros(length(DataDep(:,1)),length(DataDep(1,:))/4+1); % Audio vector + Tail (6) + Matlab Viterbi (1)
         for f=1:length(DataDep(:,1))
           % DataVit(f,:) = vitdec( [DataDep(f,:) 0 0 0 0], VitTrellis, 1, 'cont', 'unquant');  % each 4 samples--> 1 sample; Matlab 
             DataVit(f,:) = DABViterbiTZ( [DataDep(f,:) 0 0 0 0],Generator,InStates1,InStates2,RefOutValues1,RefOutValues2,1); % TZ OK for MSC; last 1; Matlab/Octave
         end
         DataVit = DataVit(:,2:end-6);      % Tail removing

       % Energy dispersal -------------------------------------------------
         DataEnerg = zeros(size(DataVit));
         for m=1:length(DataEnerg(:,1))
             DataEnerg(m,:) = xor( DataVit(m,:), DispSequense(1:length(DataEnerg(1,:))) );
         end

       % ------------------------------------------------------------------  
       % Super Frame Handling ---------------------------------------------
       % ------------------------------------------------------------------ 
       
         if(SuperFrame2AAC_ON==0) % Super Frame decoding - by outside program DABp2AAC.EXE
            % Audio data creation (from bits to bytes) --------------------
              Audio=( reshape( DataEnerg',8,[] )' * [128 64 32 16 8 4 2 1]' )';
            % Write to a file (bytes to a file) ---------------------------
            fwrite(fout, Audio, 'uint8');  % file extension ".dat"
         end
            
         if(SuperFrame2AAC_ON==1) % Super Frame decoding - below
               
              DataEng = reshape( DataEnerg', 1, [] );
              NBitsPerCIF = (NBitsAfterTimeDepunct/4-6);
              NBitsPerFrame = NCIFsPerFrame * NBitsPerCIF;
              DataEnergChain( 1 + NBitsPerCIF*CIFCount : NBitsPerFrame + NBitsPerCIF*CIFCount ) = DataEng(1:NBitsPerFrame);
                
            % Super Frame Synchronisation and transcoding to ADTS container
            % Additional delay of 5 CIFs = one DAB+ Super Frame. Together we have delay of 16+5 = 21 CIFs
              [ CIFCount, SuperFrameSync, DataEnergChain, au_start, num_aus ] = ...
                decodeSuperFrame_Short( CIFCount, SuperFrameSync, DataEnergChain, ...
                au_start, Audiokbps, num_aus, NBitsPerCIF, NCIFsPerFrame, ...
                RSpolynomial, 0, fout);

         end    % end of Super Frame decoding
     
     end    % end of UEP/EEP
   
   % ###########################################  
     end    % end of if (TimeInt buffer is full)
   % ###########################################  

% ##################
  end % of DecodeMSC
% ##################

% #########################################################################
end  % of MAIN LOOP########################################################  
% #########################################################################
  
fclose( 'all' );
