% lab23_ex_dab_fic_msc.m

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Masterthesis
% Zurcher Hochschule fur Angewandte Wissenschaften
% Zentrum fur Signalverarbeitung und Nachrichtentechnik
% (c) Michael Häner-Höin, 12.4.2011 ZSN, info.zsn@zhaw.ch
% Extended:
% AGH University of Science and Technology, Krakow, Poland
% (c) Tomasz Zielinski, 31.12.2019, tzielin@agh.edu.pl
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clc; clear all; close all; fclose('all');

include_params_global        % GLOBAL parameters (declaration only)
include_params_dab           % DAB parameters (setting)
include_params_control       % PROGRAM CONTROL parameters (setting)
include_IQfile_choice        % IQ file and Audio channel sellection
include_tables               % initialization of tables after audio channel sellection
include_init_work_params     % initialization of work parameters

disp('Work in progress ... '); tic;

% MAIN PROCESSING LOOP ##########################################################
% Delay = 1 Frame (ADC) + 16 CIFs (Time Interleaving) + 5 CIFs (DAB+ Super Frame)

while read == true
  % READ NEW IQ DATA IF NECESSARY &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&  
    [MDAT,NDAT] = size(OldData);  
    if( MDAT < ReadSize)
      try   [NewData] = ReadIQData(ReadFile, ReadSize, fs_file, fs, sigtype); % new block
      catch read = false; break;                                              % not enough
      end
    else
        NewData = [];
    end
    
  % FRAME SYNCHRONIZATION &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
    [Signal, OldData ] = SynchroTimeFreq_Short( [OldData; NewData], ...
         CorrectADC_ON, CorrectDeltaFreq_ON, CorrectTimeSynchro_ON, Plot_ON );
 
  % FRAME DECODING - FFT and D-QPSK demodulation, ETSI pp. 153-157 &&&&&&&&&&&&&&
    if( ~isempty(Signal) & (DABFrameFoundNr>1) )     % first frame is not decoded
        DABFrameNr = DABFrameNr + 1; % pause
        FFTFrame = demodulateDAB_Short( Signal, OFDMOffset, Plot_ON);
        
      % Frequency de-interleaving, ETSI pp. 157-161 &&&&&&&&&&&&&&&&&&&&&&&&&&&&&
        DeintFFTFrame = zeros(size(FFTFrame));
        for r = 1 : length( FreqDeinterleaverTab(:,4) )                               
            DeintFFTFrame( :, FreqDeinterleaverTab(r,4) ) = FFTFrame(:,r);
        end
        
      % D-QPSK symbol demapper - from 2D matrix to 1D vector, ETSI pp. 157 &&&&&&
        DataFIC = zeros(1, NSymbPerFIC*NCarrPerSymb*2);                  % 3*1536*2
        DataMSC = zeros(1,(NSymbPerFrame-NSymbPerFIC-1)*NCarrPerSymb*2); % 76-3-1 symbols
        for f = 1 : NSymbPerFIC % FIC DATA from OFDM symbols, first real, then imag 
            DataFIC( 1+(f-1)*NCarrPerSymb*2 : (f)*NCarrPerSymb*2 ) = ...
                     [real(DeintFFTFrame(f,:)) imag(DeintFFTFrame(f,:))]; % real +/-
        end
        for f = NSymbPerFIC+1 : NSymbPerFrame-1 % MSC DATA from OFDM symbols, first real 
            ff = f - NSymbPerFIC;
            DataMSC( 1+(ff-1)*NCarrPerSymb*2 : (ff)*NCarrPerSymb*2 ) = ...
                     [real(DeintFFTFrame(f,:)) imag(DeintFFTFrame(f,:))]; % real +/-
        end
        
      % &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&  
      % Decode FIC (Fast Information Channel ) - info about services &&&&&&&&&&&&
        if( DecodeFIC_ON )
            [ Start, Size, kbps, Protect, ErrProtect ] = ...
            decodeFIC( DABFrameNr, DataFIC, VitTrellis, Debug_ON );
        end
        
      % &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&  
      % Decode MSC (Main Service Channel ) - music, etc. &&&&&&&&&&&&&&&&&&&&&&&&  
        if( DecodeMSC_ON )
            decodeMSC( DABFrameNr, DataMSC, SubChStartAdr, SubChSize,...
                       Audiokbps, Protect, UEP, VitTrellis, RSpolynomial,...
                       SuperFrame2AAC_ON, NBitsAfterTimeDepunct, Debug_ON );
        end
        close all;     % close figures                   
    end         % end of ~isempty(Signal)
end   % end of while (reading data) #############################################

fprintf('Time ellapsed %3i sec \n',round(toc));
fclose('all');
