% lab23_ex_rs.m
% Reed-Solomon in DAB
% SuperFrameIN.dat comes from the function decodeMSC()
% In Octave: >>pkg load communications[ENTER]
clear all; close all;

ErrPerRS = 5;                        % injected errors per 120 bytes, per one RS 

load SuperFrameIN.dat;               % DataEnergChain from the function decodeMSC() 
BITS = SuperFrameIN;                 % new, shorter name                   
NBitsPerCIF = 2688;                  % number of bits per one CIF
RSpoly = rsgenpoly(255,245,285,0);   % 285 = 100011101 = x^8+x^4+x^3+x^2+1

% Reformating bit-stream to byte-stream
BYTES = ( reshape(BITS(1:5*NBitsPerCIF)',8,[])' * [128 64 32 16 8 4 2 1]' )';
     
% Main Reed-Solomon loop - processing Nrows-th polyphase sequences, i.e. matrix rows
Nlen = length(BYTES); Ncols=120; Nrows=Nlen/Ncols;
for k = 1 : Nrows       % for each polyphase sequence having 120 bytes each
  % Take data  
    DataIN  = BYTES(k:Nrows:end);  % take 120 bytes, k-th polyphase sequence
  % Add errors
    DataINCopy = DataIN;
    for m = 1 : ErrPerRS                      % consecutive errors
        pos = round( rand(1,1)*119 )+1;       % random error position
        val = gf( round(rand(1,1)*255), 8 );  % Galois field error
        oldval = gf( DataIN(pos),8 );         % Galois field old value         
        newval = oldval + val;                % Galois field new value
        DataIN(pos) = double( newval.x );     % modification of the input
    end
  % RS decoder  
    if( 0 )  % RS: Original code, Matlab only
        [DataOUT, num_errs] = rsdec( gf( DataIN, 8), 120, 110, RSpoly);
    else     % RS: Longer code that is shortened, Matlab and Octave
        [DataOUT, num_errs] = rsdec( gf([ zeros(1,135) DataIN], 8), 255, 245, RSpoly);
        DataOUT = DataOUT(136:245);
    end
    DataOUT = double( DataOUT.x );
  % Checking correction results
    RS_ERRORS_BEFORE = num_errs, % in bytes
    ERRORS_AFTER_RS = sum( sum( abs( DataINCopy(1:110) - DataOUT ) ) ),
  % Put corrected bits back into the bit-stream
    start = 1+(k-1)*8;
    step = Nrows*8;
    for m = 1:length(DataOUT)
        BITS(start+(m-1)*step + 0 : 1 : start+(m-1)*step + 7) = DecToBin( DataOUT(m), 8 );
    end
end % of Main RS loop
