% lab23_ex_viterbi.m
% Viterbi decoder of convolutional codes 
clear all; close all;

x = [ 0 1 1 1 ];               % input
g = [ 1 1 0 1;                 % filter #1
      1 1 1 1 ];               % filter #2

Mtaps = length(g(1,:));        % number of coefficients/delays
Kfilt = length(g(:,1));        % number of filters
MaxIndx = 2^(Mtaps-1)-1;       % 
mult = 2.^[(Mtaps-2):-1:0];    % used for binary to decimal conversion  

% Lattice diagram of analyzed convolutional encoder
y0ref = zeros(2^(Mtaps-1),Kfilt); y1ref = y0ref;         % reference states
transit = zeros(2^(Mtaps-1),3);                          % transition table
for indx = 0: MaxIndx
    xbuf = str2num( reshape(dec2bin(indx,Mtaps-1), Mtaps-1, 1 )); xbuf=xbuf'; % pause
    for k = 1 : Kfilt
        y0ref(indx+1,k) = mod( [ 0 xbuf ] * g(k,:)', 2); % up
        y1ref(indx+1,k) = mod( [ 1 xbuf ] * g(k,:)', 2); % down
    end
    transit(indx+1,1:3) = [ indx, [0 xbuf(1:Mtaps-2)]*mult', [1 xbuf(1:Mtaps-2)]*mult'];
end
y0ref, y1ref, transit, pause

% Encoder
% x(n) --> y12(n)
xbuf = zeros(1,Mtaps);
yout = [];
for n = 1 : length(x)
    xbuf = [ x(n) xbuf(1:Mtaps-1) ];
    for k = 1 : Kfilt
         y(1,k) = mod( xbuf * g(k,:)', 2);           % convolution modulo 2
    end    
    yout = [ yout; y(1,1:Kfilt) ];
end
disp('ENCODER OUTPUT and DECODER INPUT'); 
yout,

% Optional addition of errors
yin = yout;
if( yin(1)==0 ) yin(1)=1; else yin(1)=0; end           % errors
%if( yin(end)==0 ) yin(end)=1; else yin(end)=0; end    % errors
yin, pause

% Decoder
npoints = 1; step = 2^(Mtaps-1);
metric = zeros(2^(Mtaps-1),2);               % accumulated metrics                        
path = zeros(2^(Mtaps-1),length(x)+1);       % path history, initialized
path(1,1) = 1;                               % starting point
for ndata = 1 : Mtaps-1                      % first steps, incomplete tree
    for indx = 1 : 2^(Mtaps-1)
        if( path(indx,ndata) ~= 0 )
            path( transit(indx,2)+1, ndata+1 ) = indx;
            path( transit(indx,3)+1, ndata+1 ) = indx;
            val0 = sum( y0ref(indx,:)==yin(ndata,:) ); 
            val1 = sum( y1ref(indx,:)==yin(ndata,:) );
            metric( transit(indx,2)+1,2 ) = val0 + metric( indx,1 );
            metric( transit(indx,3)+1,2 ) = val1 + metric( indx,1 );
        end
    end
    [metric(:,2), path],pause
    metric(:,1) = metric(:,2);
end
for ndata = Mtaps : length(x)                    % next steps, regular tree
    for indx = 1 : 2 : 2^(Mtaps-1)               % two branches DOWN and UP
        val0A = sum( y0ref(indx,:)  ==yin(ndata,:) ) + metric( indx,1 ); 
        val0B = sum( y0ref(indx+1,:)==yin(ndata,:) ) + metric( indx+1,1 );
        val1A = sum( y1ref(indx,:)  ==yin(ndata,:) ) + metric( indx,1 ); 
        val1B = sum( y1ref(indx+1,:)==yin(ndata,:) ) + metric( indx+1,1 );
      % [ val0A, val0B], [ val1A, val1B ], pause
        idown = (indx+1)/2; iup = idown + 2^(Mtaps-1)/2; 
        if( val0A > val0B )                                  % down
            metric( idown, 2 ) = val0A;
            path(   idown, ndata+1 )  = indx;
        else
            metric( idown, 2 ) = val0B;
            path(   idown, ndata+1 ) =  indx+1;
        end
        if( val1A > val1B )                                  % up
            metric( iup, 2 ) = val1A;
            path(   iup, ndata+1) = indx;
        else
            metric( iup, 2 ) = val1B;
            path(   iup, ndata+1) =  indx+1;
        end
    end
    [metric(:,2), path], pause
    metric(:,1) = metric(:,2);
end

% Decision taking
[dummy,pos] = max(metric(:,1));
for p = 0:ndata-1
    if( pos  >= 2^(Mtaps-2) ) bits(p+1) = 1; else bits(p+1)=0; end
    pos = path(pos,end-p),
end    
bits = bits(end:-1:1), pause
