% lab24_ex_4g_lte.m
% Example of recovery of Physical Broadcast Channel (PBCH)
% QPSK constallation points in 4G LTE digital telephony
% In Octave: >>pkg load signal communications[ENTER] 
  clear all; close all;

  do_LPfilter = 0;      % 0/1 low-pass filtration, passing only used PBCH carriers
  do_disturbing = 0;    % 0/1 disturbing the signal
  do_ADCcorrect = 0;    % 0/1 correcting ADC sampling rate
  do_plots = 1;         % 0/1 displaying figures
   
% #########################################################################
% Load input signal - un-comment one file only
% #########################################################################

% Real-world recorded by the book author using ADALM PLLUTO
  load('LTE_803.505MHz_30.72MHz_Short.mat'); fs=30.72e6; fs_lte=7.68e6; THR=15; REPEAT=0;  y = samples; clear samples;

% Synthesized, taken from ADALM PLUTO support software
% load('LTE20.mat'); fs=30.72e+6; fs_lte=30.72e6; THR = 15; REPEAT=1; y = new.'; clear new;
% load('LTE10.mat'); fs=15.36e+6; fs_lte=15.36e6; THR = 15; REPEAT=1; y = new.'; clear new;
% load('LTE5.mat');  fs=7.68e+6;  fs_lte=7.68e+6; THR = 15; REPEAT=1; y = new.'; clear new;

% From other sources
%  yy = dlmread('LTE_TM3p1_10MZ_18p22dBFS.txt', ' '); y = complex(yy(:,1), yy(:,2)); 
%  y=y.'; fs=15.36e6; fs_lte=15.36e6; THR = 5; REPEAT=1;  clear yy; 

% #########################################################################
% LTE PARAMETERS
% #########################################################################

% LTE mode ? fs_lte --> LTE mode  
  if( fs_lte ==  1.92e+6 )  LTEmode = 1;  end
  if( fs_lte ==  3.84e+6 )  LTEmode = 2;  end
  if( fs_lte ==  7.68e+6 )  LTEmode = 3;  end
  if( fs_lte == 15.36e+6 )  LTEmode = 4;  end
  if( fs_lte == 23.04e+6 )  LTEmode = 5;  end
  if( fs_lte == 30.72e+6 )  LTEmode = 6;  end

 % Table of LTE parameters - for all working modes ------------------------
 %           1        2      3       4       5       6
 LTEtab = [  1.4,     3,     5,     10,     15,     20;  ... % channel bandwidth [MHz]
               6,    15,    25,     50,     75,    100;  ... % number of resource blocks (RBs)
              72,   180,   300,    600,    900,   1200;  ... % number of used subcarriers
             128,   256,   512,   1024,   1536,   2048;  ... % DFT/FFT size
              15,    15,    15,     15,     15,     15;  ... % subcarrier spacing [kHz]
            1.92,  3.84,  7.68,  15.36,  23.04,  30.72;  ... % sampling rate [MHz]
             960,  1920,  3840,   7680,  11520,  15360];     % samples per slot 
 %         first   next symbols              
 CPtab = [  5.21,   4.69;   ... % microsec - NORMAL   CP: 15kHz,  7 symbols/slot
           16.67,  16.67;   ... % microsec - EXTENDED CP: 15kHz,  6 symbols/slot
           33.33,  33.33 ];     % microsec - EXTENDED CP: 7.5kHz, 6 symbols/slot
            
 % LTE mode --> parameter values
   CPmode  = 1;                                 % cyclic prefix mode
   Nfft    = LTEtab( 4, LTEmode );              % FFT length
   fs_lte  = LTEtab( 6, LTEmode ) * 1e6;        % sampling frequency [Hz]
   df_lte  = LTEtab( 5, LTEmode ) * 1e3;        % subcarrier spacing [Hz]
   dt_lte  = 1/fs_lte;                          % sampling period [s]
   BW_lte  = LTEtab( 1, LTEmode ) * 1e6;        % channel bandwidth [Hz]
   Nrb     = LTEtab( 2, LTEmode );              % number of resource blocks (RBs)
   Nsc     = LTEtab( 3, LTEmode );              % number of subcarrier used
   CPtime1 = CPtab(CPmode,1) * 1e-6;            % CP 1 duration [s]
   CPtime2 = CPtab(CPmode,2) * 1e-6;            % CP 2 duration [s]
   Ncp1    = round(  CPtime1 / dt_lte );        % CP 1 length (samples)
   Ncp2    = round(  CPtime2 / dt_lte );        % CP 2 length (samples)             
   Nslot   = (Nfft+Ncp1)+6*(Nfft+Ncp2);         % slot length (samples)
   Nsymb1  = Nfft+Ncp1;                         % OFDM 1 symbol length
   Nsymb2  = Nfft+Ncp2;                         % OFDM 2 symbol length
   Nframe  = 10*2*Nslot;                        % frame length (samples)
   Nframe2 = Nframe/2;                          % sub-frame length

% #########################################################################
% 1. LTE SIGNAL PRE-PROCESSING
% #########################################################################

% Observe high-resolution signal spectrum
  N = length(y);                                              % signal length
  y = y .* exp( -j*2*pi*(0 : N-1) * ( 5*df_lte )/ fs );       % TEST integer CFO
  plot_freq(y, fs); title('Input signal spectrum'); pause     % function call
  figure; pwelch(y,2048,2048-1024,2048,fs,'centered'); pause  % PSD Welch
  figure; f = fs/2048*(-1024:1023); spectrogram(y,2048,1024,f,fs); pause  % STFT 

% Observe low-resolution signal spectrum
  Nfftup = Nfft*fs/fs_lte; Ncut = 16*Nfftup; df = fs/Nfftup;
  plot_freqbins(y(1:Ncut), Nfftup); title('|Y(k)| before shift, 15kHz step'); pause

% Calculate or choose required integer spectrum shift, then perform it upon the signal
  Kmax = 20;                                            % select search size
  s = reshape( y(1:Ncut), Nfftup, Ncut/Nfftup );        % create a signal matrix
  S = fftshift( fft(s) / Nfftup );                      % do FFTs of its columns
  S = mean( abs(S.') );                                 % accumulate all spectra
  kcentr = Nfftup/2+1;                                  % index of spectrum center 
  [val, indx ] = min( S(kcentr-Kmax:kcentr+Kmax) );     % find minimum and its position
  SHIFT = indx-Kmax-1, % 1234?                          % choose/calculate shift value
  y = y .* exp( -j*2*pi*(0 : N-1) * (SHIFT*df) / fs );  % do integer shift !
% y = y .* exp( -j*2*pi*(0 : N-1) * (-1250)/ fs );      % do fractional shift?

% Observe spectrum of the resultant signal
  plot_freqbins(y(1:Ncut), Nfftup); title('|Y(k)| after shift, 15kHz step'); pause

% Signal re-sampling - when actual sampling frequency is not a standard one   
if( fs ~= fs_lte )  
    y = resample( y, 1, fs/fs_lte ); fs = fs_lte;    % LP filtering and down-sampling
    N = length(y);                                   % new signal length
    plot_freq(y, fs); title('Down-sampled'); pause
end

% Optional low-pass filtration - passing only carriers used by LTE or PBCH
if( do_LPfilter )
  Mtaps = 125;                                       % number of filter weights 
  freq = [0, Nsc * df_lte,  BW_lte, fs] / fs;        % only used LTE sub-carriers
% freq = [0, 72 * df_lte, 4*72*df_lte, fs] / fs;     % only PBCH sub-carriers
  gain    = [ 1 1 0 0 ];                             % gains
  weights = [ 1   0.8 ];                             % bands significance
  h = firls( Mtaps-1, freq, gain, weights);          % least-squares design
  size(h), pause
  y = conv(y, h, 'same');                            % filtering
  plot_freq(y,fs); title('Filtered'); pause          % result
  plot_freqbins(y(1:Ncut), Nfft); title('Filtered'); pause
end

% Optional signal enlarging - recommended for short synthesized signals
  if( REPEAT == 1 )
      y = [ y y y y y ]; N = length(y);
  end

% #########################################################################
% 2. Optional addition of disturbances
% #########################################################################

if(do_disturbing)
  % Parameter values 
    cfo=0; npwr=-160; ADCppm=0; G0=1; G1=0; G2=0; D0=0; D1=10; D2=25;
  % npwr=-30;             % noise power [dB]: -160, ...,-60,-50,-40,-30,-20
  % cfo=1245;             % carrier frequency offset (CFO) [Hz]
  % ADCppm=400;           % ADC error [ppm]: from -400 to +400
  % G0=0.5; G1=0.25; G2=0.1; D0=0; D1=10; D2=25; % Multi-path: gains, delays
  
  % Disturbing signal
  NADC = round((1+ADCppm/1000000)*N); step = N/NADC;           % ADC error
  if( step ~= 1 )
      disp('ADC ERROR -> generation !');
      NADC = length(0:step:N-step);
      y(1,1:NADC)=interp1( [0:N-1], y(1:N), [0:step:N-step], 'spline' );
      N = NADC; y(NADC+1:end)=[];
  end
  y = G0*[ y(end-D0+1:end) y(1:end-D0) ] + ...                 % multi-path 
      G1*[ y(end-D1+1:end) y(1:end-D1) ] + ...                 %
      G2*[ y(end-D2+1:end) y(1:end-D2) ];                      %
  y = y .* exp( j*2*pi*cfo/fs*(0:N-1));                        % CFO
  y = y + 10^(npwr/20) / sqrt(2) * (randn(1,N)+j*randn(1,N));  % AWGN
  
    if( do_plots ) 
        pwelch(y,2048,2048-1024,2048,fs,'centered');
        title('LTE_Spectrum_f_30MHz_Welch_WithAllDisturb'); pause
    end

end

% #########################################################################
% 3. Signal Auto Correlation - fractional carrier frequency offset estimation
% #########################################################################

  Ryy=[];
  for n = 1 : Nframe % find correlation coefficients
      n1 = n:n+Ncp2-1; n2 = n1 + Nfft;
      a1 =  abs( sum( y(n1) .* conj( y(n2) ) ) );
      b1 =  abs( sum( y(n1) .* conj( y(n1) ) + sum( y(n2) .* conj( y(n2) ) )));
      Ryy(n) = (a1*a1) / (b1*b1); 
  end
  figure; plot(Ryy); grid; title('Original Ryy(k)'); pause
  for k = 1 : 20 % find 20 max peaks
      [val,indx(k)] = max(Ryy);                                     % find max
      N1 = max(1,indx(k)-Nsymb2); N2= min(length(Ryy),indx(k)+Nsymb2);
      Ryy( N1 : N2 ) = zeros(1,N2-N1+1); % zero around
  end
  indx = sort(indx);
  for k = 1 : 20 % use CP for fractional CFO estimation
      n1 = indx(k):indx(k)+Ncp1-1;
      CFO_cp(k) = fs/Nfft*mean( angle( conj( y(n1) ) .* y( n1+Nfft ) ) / (2*pi));
  end
  
    if( do_plots ) % Fractional CFO
       plot( indx(1:20), CFO_cp(1:20),'bo-' ); grid;
       xlabel('sample index n'); ylabel('[Hz]'); title('\Delta f (n)'); pause
    end
    
  CFO = mean( CFO_cp ), pause                     % fractional CFO estimation
  y = y .* exp(-j*2*pi*(0:length(y)-1)*(CFO/fs)); % fractional CFO correction
  CFO = 0;                                        % already corrected
  
% #########################################################################
% PSS: Primary Signal Synchronization - using Zadoff-Chu sequences
% #########################################################################
 
% PSS generation in frequency domain - Zadoff-Chu sequences
  ZC = zeros(Nfft,3);   % NFFT=512  (226-288), NFFT=1024 (482-544)
  Nc = Nfft/2+1;        % NFFT=1536 (738-800), NFFT=2048 (994-1056 
  n = (0:62)';
  ZC(Nc-31:Nc+31,1) = exp( -j*pi*25*n.*(n+1)/63 );     % from ZC definition
  ZC(Nc-31:Nc+31,2) = exp( -j*pi*29*n.*(n+1)/63 );     %
  ZC(Nc-31:Nc+31,3) = exp( -j*pi*34*n.*(n+1)/63 );     %
  ZC(Nc, 1:3)       = zeros(1,3);                      %
 
    if( do_plots ) % phasor plots of Zadoff-Chu spectra
        figure
        alf = 0:2*pi/5000:2*pi; c=cos(alf); s=sin(alf);
        subplot(131); plot(real(ZC(:,1)),imag(ZC(:,1)),'bx',c,s,'r-'); grid; title('ZC #1');
        subplot(132); plot(real(ZC(:,2)),imag(ZC(:,2)),'bx',c,s,'r-'); grid; title('ZC #2');
        subplot(133); plot(real(ZC(:,3)),imag(ZC(:,3)),'bx',c,s,'r-'); grid; title('ZC #3');
        pause
    end    

% Going to time domain
  for nn = 0 : 2
      ZCC(:,nn+1) = ifftshift( ZC(:,nn+1) );
  end
  zc = ifft( ZCC )*sqrt(Nfft);
 
      if( do_plots ) % Zadoff-Chu sequences and auto-correlations
         for k=1:3
           figure;
           subplot(211); plot(1:Nfft,real(zc(:,k)),'ro-',1:Nfft,imag(zc(:,k)),'bx-'); title('ZC IQ signal');
           subplot(212); plot(abs(xcorr(zc(:,k)))); title('ZC auto-corr');
           pause
         end
      end  

% PSS: signal cross-correlation with Zadoff-Chu (zc) sequences 
 for k = 1:3
     Rpss(:,k) = abs( conv( y.', conj( zc(1:end,k)) ) ); 
 end
 
      if( do_plots )
        figure;
        subplot(131); plot(Rpss(:,1)); grid; title('PPS and ZC#1 x-corr');
        subplot(132); plot(Rpss(:,2)); grid; title('PSS and ZC#2 x-corr');
        subplot(133); plot(Rpss(:,3)); grid; title('PSS and ZC#3 x-corr');
        pause
      end
      
% Choosing the best sequence (with the highest Rpss), finding arguments of its peaks
% Detection N_PSS_ID
  [dummy, PSS_ID ]  = max( max( Rpss ) );
  Rpss = Rpss(:,PSS_ID); zc = zc(:,PSS_ID); ZC = ZC(:,PSS_ID).';
  PSS_ID = PSS_ID-1, 
  threshold = THR;                 % 0.85*max(Rpss); % <--- CHOOSE THRESHOLD
  counter = 1; 
  for m=2:length(Rpss)-1
      if( (Rpss(m) > threshold) && (Rpss(m) > Rpss(m-1)) && (Rpss(m) > Rpss(m+1)) )
          imax(counter) = m; counter = counter+1;
      end    
  end
  
      if( do_plots )
         figure;
         plot( Rpss,'b'); grid; hold on;
         if( isempty(imax)==0)
             plot(imax(1:2:end), Rpss(imax(1:2:end)),'ro',imax(2:2:end), Rpss(imax(2:2:end)),'go');
         end
         pause
      end   
        
% Final calculations
  PSS_idxs = imax - Nsymb2;
  SSS_idxs = PSS_idxs - Nsymb2;
  display('PSS beginnings:'); PSS_idxs'
  pause
  display('Frame/HalfFrame lengths:'); (PSS_idxs(2:end)-PSS_idxs(1:end-1))',
  
      if(1)
      % Testing integer CFO, the same way as in the DAB+ radio
        figure
        PSS_td = y( PSS_idxs(1)+Ncp2 : PSS_idxs(1)+Ncp2+Nfft-1 ); % OFDM symbol samples
        CFO_integer = fft( PSS_td .* conj(zc.') );
      % D = 10; CFO_integer = fft( [zc(D+1:end); zc(1:D)] .* conj(zc) ); % FOR TEST
        stem( -Nfft/2:1:Nfft/2-1, abs( fftshift( CFO_integer )) ); title('CFO_int');
        pause
      end  

% ADC sampling rate error estimation and correction
if( do_ADCcorrect )
  indx = sort( indx );
  Nperiod = PSS_idxs(end) - PSS_idxs(1);
  ratio = Nperiod/Nframe2;
  Nframe2real = Nperiod / round( ratio ), % number of samples per half-frame
  pause
  if( (Nframe2real - Nframe2) > 1 )
       N = length(y); x = []; 
       disp('ADC ERROR -> correction !');
     % step = 1/step; % for test only
       step = Nframe2real / Nframe2,
       x(1,1:length([0:step:N-1]),1)=interp1( [0:N-1], y, [0:step:N-1], 'spline' );
     % figure; plot(0:N-1,real(y),'ro-',0:step:N-1, real(x), 'bx-'); pause
       y = x; clear x;
       N = length(y);
     % Repeat PSS signal synchronization!  
  end
end

  CFO_history = zeros( 1, numel(PSS_idxs) );   % initialization before the loop
  CFO_history(1) = CFO;                        % 

% #########################################################################
% Main Loop ###############################################################
% #########################################################################

% Initial carrier frequency offset(CFO) estimation  -----------------------
  alpha = 0.85; Ncut1 = 10; Ncut2 = 5;                     % for CFO update
  k_pss  = [((Nfft/2-30):Nfft/2)'; (Nfft/2+2:Nfft/2+32)']; % PSS indexes
  k_pbch = [((Nfft/2-35):Nfft/2)'; (Nfft/2+2:Nfft/2+37)']; % PBCH indexes

%#################################  
for iter = 1 : numel( PSS_idxs )-1  % iteration loop, last data can be too short
%#################################

  iter

% Next iteration  
  PSS_idx = PSS_idxs( iter ),             % position of the next PSS
  n1sss = SSS_idxs( iter );               % position of the next SSS
 
% CFO estimation using PSS
  n = PSS_idx+Ncut1 : PSS_idx+Ncp2-1-Ncut2;                 % sample numbers 
  CFO_pss = fs/Nfft * angle( sum( conj(y(n)) .* y(n+Nfft) ) ) / (2*pi);
  CFO = alpha*CFO + (1-alpha)*CFO_pss,                      % estimation update
  display( sprintf( 'Estimated CFO: %f Hz', CFO ) );
  CFO_history(iter+1) = CFO;                                % store it

% CFO correction of the LTE frame half (using calculated CFO) - starting from SSS
  n = n1sss : n1sss + Nframe2-1;                            % sample numbers 
  y( n ) = y( n ).* exp(-j*2*pi*(0:Nframe2-1)*(CFO/fs));    % CFO correction

% Channel estimation using PSS
  PSS_td = y( PSS_idx+Ncp2 : PSS_idx+Ncp2+Nfft-1 );         % OFDM symbol samples
  PSS_fd = fftshift( fft( PSS_td ) ) / sqrt(Nfft);          % FFT         
  Hest = PSS_fd( k_pss ) .* conj( ZC( k_pss  ) );           % LS channel estimate

% Extraction of SSS sequence and its equalization using the estimated channel
  SSS_td = y(n1sss+Ncp2 : n1sss+Ncp2+Nfft-1);               % OFDM symbol samples
  SSS_fd = fftshift( fft( SSS_td ) ) / sqrt(Nfft);          % FFT
  SSS_fd = SSS_fd( k_pss  ) ./ Hest;                        % correction

% Detection of N_SSS_ID and subframe number using the SSS
  [SSS_ID, subframe, SSS_corr] = lte_sss_detect( SSS_fd.', PSS_ID );
  if( do_plots )
     figure;
     plot(0:167, SSS_corr(:,1), 'r', 0:167, SSS_corr(:,2), 'b');
     title('SSS detection'); xlabel('N^{(1)}_{ID}'); ylabel('Correlation');
     legend('Subframe 0', 'Subframe 5'); pause
  end
  % Calculating the physical radio cell indentifier
    Ncell_ID = 3 * SSS_ID + PSS_ID;
    display(sprintf('N_cell_ID: %0d; Subframe: %0d; Sample: %0d', Ncell_ID, subframe, PSS_idx)),

  % Scatter plot of the OFDM Symbol before the SSS
    OFDM_td = y(n1sss-Nsymb2+Ncp2 : n1sss-Nsymb2+Ncp2+Nfft-1); % OFDM symb before SSS
    OFDM_fd = fftshift( fft( OFDM_td ) ) / sqrt(Nfft);         % FFT
    OFDM_fd = OFDM_fd( k_pss  ) ./ Hest;                       % correction
    figure; scatterplot( OFDM_fd,1,0,'bx' ); title('Scatter before SSS'); pause
    
 % #############################################   
 % PBCH PROCESSING #############################   
 % #############################################   

   cp_type = 'normal'; % prefix type, number of antena ports checked
   
  % PBCH is in the first sub-frame only
  if subframe == 0
      
   % starting positions of 4 PBCH symbols
     PBCH_idxs = PSS_idx + Ncp2 + cumsum([Nfft+Ncp1, Nfft+Ncp2, Nfft+Ncp2, Nfft+Ncp2]);
   % OFDM demodulation of 4 PBCH symbols, extraction of center 72 RE (resource elements)
   % (TS36.211 sec. 6.6.4, 1 RE = 1 subcarrier)
     PBCH_grid = zeros(72,4);                                      % initialization
     for n_sym = 1 : numel( PBCH_idxs )                            % for 4 PBCH
         PBCH_td = y( PBCH_idxs(n_sym) : PBCH_idxs(n_sym)+Nfft-1); % samples
         PBCH_fd = fftshift( fft(PBCH_td) ) / sqrt(Nfft);          % FFT
         PBCH_grid(:,n_sym) = PBCH_fd( k_pbch );                   % store
     end
   % plot( real(PBCH_grid),imag(PBCH_grid),'o'); grid; title('PBCH_grid'); pause,

   % symbol numbers containing CRSs (Cell Specific Reference Signals) for a given antenna port
     l_crs_ant = [0, 0, 1, 1];
    
   % constants
     N_PBCH_symb = 4;   % number of PBCH symbols
     N_PBCH_RB   = 6;   % number of Resource Blocks used by PBCH
     N_ap_max    = 4;   % maximum number of antenna
    
   % pre-allocate memory
     x_crs      = zeros( N_PBCH_RB *2, N_ap_max);  % CRS transmitted (ref)
     y_crs      = zeros( N_PBCH_RB *2, N_ap_max);  % CRS received
     k_crs      = zeros( N_PBCH_RB *2, N_ap_max);  % CRS indexes 
     H_est_crs  = zeros( N_PBCH_RB *2, N_ap_max);  % H estimated for CRS
     H_est_PBCH = zeros( N_PBCH_RB*12, N_ap_max);  % H estimated for PBCH
     CRS_corr   = zeros( N_ap_max, 1 );            % correlation of TX and RX CRS

   % Iterate for all four antenna ports
     for ant = 1 : N_ap_max
      % Generate ideal CRS for slot 1 and current antenna port
        [x_crs(:,ant), k_crs(:,ant)] = lte_crs_gen( N_PBCH_RB, Ncell_ID, 1, ...
                                                   l_crs_ant(ant), ant-1, cp_type);
      % De-map CRS from the received signal 
        y_crs(:,ant) = PBCH_grid( k_crs(:,ant)+1, l_crs_ant(ant)+1 );
 
      % Find correlation coefficient between CRS TX and RX  
      %  CRS_corr(ant) = abs( corr( x_crs(:,ant), y_crs(:,ant) ) );
         CRS_corr(ant) = abs( sum( conj(x_crs(:,ant)) .* y_crs(:,ant) ) ) / ...
                (  sqrt( abs( sum( conj(x_crs(:,ant)) .* x_crs(:,ant)) ) * ...
                   sqrt( abs( sum( conj(y_crs(:,ant)) .* y_crs(:,ant)) ) ) ));
     
      % Estimate channel (least squares) on pilot sub-carriers
        H_est_crs(:,ant) = conj(x_crs(:,ant)) .* y_crs(:,ant);

      % Interpolate to non-pilot frequencies
        H_est_PBCH(:,ant) = lte_crs_interp_hest( H_est_crs(:,ant) ); % better

     end
    
    display(sprintf('CRS xcor for PBCH: AP0=%1.4f, AP1=%1.4f, AP2=%1.4f, AP3=%1.4f', ...
                     CRS_corr(1), CRS_corr(2), CRS_corr(3), CRS_corr(4)));
    
    N_ap = length( CRS_corr( CRS_corr > 0.75 ) ), pause
                 
  % Determine REs that are occupied by PBCH by removing the CRS sub-carriers
    PBCH_grid_RE_idx = ones( N_PBCH_RB*12, N_PBCH_symb );         % init: all 1s
    for ant = 1 : N_ap_max                                        % for all antenna ports   
        PBCH_grid_RE_idx( k_crs(:,ant)+1, l_crs_ant(ant)+1 ) = 0; % 0s for CRS
    end                                                           %
    
  % Un-map PBCH modulation symbols and channel estimates from the RE grid 
    y_pbch = PBCH_grid( PBCH_grid_RE_idx ~= 0 );       % take non-zero PBCH
    h_pbch = zeros( length(y_pbch), N_ap );            % initialize h_pbch
    for ant = 1 : N_ap                                 % for antenna ports
        H_est_PBCH_ant = repmat( H_est_PBCH(:,ant), [1,N_PBCH_symb] ); % copy to N_ap cols  
        h_pbch(:,ant)  = H_est_PBCH_ant( PBCH_grid_RE_idx ~= 0 );      % only for RE
    end

  % Equalize TX diversity and do layer de-mapping to get constellation symbols
    PBCH_IQ = lte_eq_and_demap_tx_div( y_pbch, h_pbch(:,1:N_ap) );

    figure; scatterplot( PBCH_IQ,1,0,'bx' ); title('Scatter for PBCH'); pause

       if(0)
           figure; p=scatterplot( PBCH_IQ,1,0,'bx');   
         % FigType=2; MyFigFile='LTE_PBCH_Disturb_Noise_m20'; grid; % FigType = 1=long, 2=short
         % FigType=2; MyFigFile='LTE_PBCH_Disturb_FractionalCFO_1234_corrected'; grid; % FigType = 1=long, 2=short
         % FigType=2; MyFigFile='LTE_PBCH_Disturb_FractionalCFO_1234_NOT_corrected'; grid; % FigType = 1=long, 2=short
         % FigType=2; MyFigFile='LTE_PBCH_Disturb_ADCSampling_400_corrected'; grid; % FigType = 1=long, 2=short
         % FigType=2; MyFigFile='LTE_PBCH_Disturb_ADCSampling_400_NOT_corrected'; grid; % FigType = 1=long, 2=short
         % FigType=2; MyFigFile='LTE_PBCH_Disturb_Channel_corrected'; grid; % FigType = 1=long, 2=short
           FigType=2; MyFigFile='LTE_PBCH_Disturb_Channel_NOT_corrected'; grid; % FigType = 1=long, 2=short
         % FigType=2; MyFigFile='LTE_PBCH_Disturb_ALL'; grid; % FigType = 1=long, 2=short
           FigsTZ
           pause
        end   
    
  end % of subframe==0

%#################################  
end % of iter over PSS_indxs
%#################################  

figure; plot( CFO_history, 'bo-' ); grid; xlabel('estimate number m'); ylabel('f [Hz]'); title('CFO(m) [Hz]'); pause
