function [r, k] = lte_crs_gen(N_RB, N_cell_ID, slot, l, p, cyclic_prefix)
  assert(ismember(p, 0:3), 'up to 4 antenna layers are supported');

  if nargin < 6
    cyclic_prefix = 'normal';
  end
    
  % Largest downlink bandwidth configuration allowed in LTE
  N_max_DL_RB = 110;

  % return empty r and k if the symbol does not carry CRS
  if ~ismember(l, lte_crs_symbols(p,slot,cyclic_prefix));
    r = [];
    k = [];
    return;
  end

  if strcmp(cyclic_prefix, 'normal')
    N_cp_i = 1;
  else
    N_cp_i = 0;
  end

  % determine v as per 6.10.1.2
  if p == 0 && l == 0
    v = 0;
  elseif p == 0 && l ~= 0
    v = 3;
  elseif p == 1 && l == 0
    v = 3;
  elseif p == 1 && l ~= 0
    v = 0;
  elseif p == 2
    v = 3 * mod(slot, 2);
  elseif p == 3
    v = 3 + 3 * mod(slot, 2);
  end
  
  v_shift = mod(N_cell_ID, 6);

  m = 0 : 2*N_RB-1;
  k = 6 * m + mod(v + v_shift, 6);
  mp = m + N_max_DL_RB - N_RB;

  c_init = 2^10 * (7 * (slot + 1) + l + 1) * (2 * N_cell_ID + 1) + 2 * N_cell_ID + N_cp_i;
  c = gold31seq(c_init, 4 * N_max_DL_RB);

  rr = complex(1 - 2 * c(1:2:end), 1 - 2 * c(2:2:end)) / sqrt(2);

  r = rr(1+mp);
end