% TS36.211 sec. 6.11.2.1
% Generate Secondary Synchronization Signal (SSS) sequence
% using given N_1_ID and N_2_id.

function [d_sf0, d_sf5] = lte_sss_gen(N_1_id, N_2_id)
  assert( ismember(N_1_id, 0:167 ), 'invalid N_1_id: allowed range is 0 to 167');
  assert( ismember(N_2_id, 0:2 ),   'invalid N_2_id: allowed values are 0, 1 or 2');

  % generate m0 and m1
  %(NOTE: can use Table 6.11.2.1-1 directly as LUT instead of below calculations)
  q_p = floor(N_1_id / 30);
  q   = floor((N_1_id + q_p*(q_p+1)/2) / 30);
  m_p = N_1_id + q*(q+1)/2;
  m0  = mod(m_p, 31);
  m1  = mod(m0 + floor(m_p/31) + 1, 31);

  % generate s0 and s1 sequences
  [s0, s1] = sss_seq_scrambler([0,2], m0, m1);
   
  % generate c0 and c1 sequences
  [c0, c1] = sss_seq_scrambler([0,3], N_2_id, N_2_id+3);

  % generate z0 and z1 sequences
  [z0, z1] = sss_seq_scrambler([0,1,2,4], mod(m0,8), mod(m1,8));

  % generate SSS sequences for subframe 0 and 5
  d_sf0 = zeros(62,1);
  d_sf5 = zeros(62,1);
  n = 0:30;

  d_sf0(1+2*n)   = s0(n+1) .* c0(n+1);
  d_sf0(1+2*n+1) = s1(n+1) .* c1(n+1) .* z0(n+1);
  
  d_sf5(1+2*n)   = s1(n+1) .* c0(n+1);
  d_sf5(1+2*n+1) = s0(n+1) .* c1(n+1) .* z1(n+1);
end

% Helper function - generator for s, c and z sequences
function [seq0, seq1] = sss_seq_scrambler(polynomial, shift0, shift1)
  x = zeros(31,1); x(4+1) = 1;
  for i = (0 : 25)+1
    for p = polynomial
        x(i + 5) =  mod(x(i + 5) + x(i+p), 2);
    end
  end
  y = 1 - 2*x;
  seq0 = circshift(y, -shift0);
  seq1 = circshift(y, -shift1);
end